/********************************************************************************/
/*    H   H Y   Y TTTTT EEEEE  CCC       HYTEC ELECTRONICS LTD                  */
/*    H   H  Y Y    T   E     C          5 Cradock Road,                        */
/*    HHHHH   Y     T   EEE   C          Reading, Berks.    Tel: 0118 9757770	*/
/*    H   H   Y     T   E     C          RG2 0JT            Fax: 0118 9757566	*/
/*    H   H   Y     T   EEEEE  CCC       Web: www.hytec-electronics.co.uk       */
/********************************************************************************/
/*                                                                              */
/*			IOC9010 PROJECT	--- Hytec IP Card API Library Testing Program		*/
/*                                                                              */
/********************************************************************************/
/*  Source file name      :-      Test8516.c									*/
/*  Definition header file:		  HYTECIPAPI.h									*/
/*								  Hy8516.h										*/
/*  Platform			  :		  Linux, Windows XP, Windows 2000               */
/*  Initial creation date :-      21-OCT-2009                                   */
/*                                                                              */
/*  Original Developers   :-      Jim Chen	                                    */
/********************************************************************************/
/*                                                                              */
/*  Description :-                                                              */
/*                                                                              */
/*  This is the master node test program to test Hytec 8516 RS485 communication.*/
/*  It sends a message every 3 second with header changed from 1 to 7 to      	*/
/*  simulate message calls for slave 1 to 7. When slave gets its call, it       */
/*	replies. Note, if there is not reply from slave, this program hangs.        */
/*																				*/
/*  (C)2009 Hytec Electronics Ltd.												*/
/*                                                                              */
/********************************************************************************/
/*                                                                              */
/*  Modification record :-                                                      */
/*                                                                              */
/*  Date modified |     Person    |   Description of modification               */
/* ---------------|---------------|-------------------------------------------- */
/*    21/10/09    |	    JC        |	  Initial copy								*/
/*                |               |                                             */
/*                                                                              */
/********************************************************************************/


#include "../include/HytecIPAPI.h"
#include "../include/Hy85156.h"

#include <pthread.h>
#include <math.h>
#include <linux/input.h>

/* Global variables */
  
static CHy85156 COM1;
static long status;
char *request_to_slave[] = {"Message is sent to ",
                            "Message sent from me",
                             "3 hi number three",
                             "4 i am talking to you",
                             "5 who are you",
                             "6 are you there",
                             "7 last port call"};

int no;
static int port = 0;

int IPslot = 'D';
char vector = 0x0;				/* no interrupt */
int delay = 0;
int halfduplex = 2;
int delay485 = 5;
int vmeslot = 0;
	
ULONG baud = 9600;
char parity = 'N'; 
int stopbits = 1;
int databits = 8;
char flowctl = 0;
int i;

/* Global variables */


/**************To simulate getch() function **********/
#include <termios.h>
#include <unistd.h>
int getsch( ) 
{
  struct termios oldt, newt;
  int            ch;
  tcgetattr( STDIN_FILENO, &oldt );
  newt = oldt;
  newt.c_lflag &= ~( ICANON | ECHO );
  tcsetattr( STDIN_FILENO, TCSANOW, &newt );
  ch = getchar();
  tcsetattr( STDIN_FILENO, TCSANOW, &oldt );
  return ch;
}
/**************To simulate getch() function **********/

int main(void)
{ 
	status = 0;
    UCHAR buf[128];

	printf("\nTesting Program starts......\n");
	int flag=0, startport, i;

	do
	{
		printf("Please type IP slot (A ~ F):\n");
		IPslot = getsch() & 0xFF;		
		if(IPslot >0x60) IPslot=IPslot-0x20;
		if((IPslot-0x40>0) && (IPslot-0x40<6)) flag = 1;
	}while(flag == 0);

	flag = 0;
	do
	{
		printf("Please type IP starting port number (2 ~ 8):\n");
		startport = getsch() & 0xFF;		
		if((startport-0x32>=0) && (startport-0x30<9)) 
		{
			flag = 1;
			startport = startport-0x30;
		}
	}while(flag == 0);

	printf("IP Slot: %c, start port: SK%d\n", IPslot, startport);

	COM1.IOC9010SetIPClock(IPslot, 0);

	//generic settings for the 8515 COM1
	status = COM1.Hy85156_Configure(vmeslot, IPslot, vector, delay);
	if (status < 0)
	{
		printf("Configure 8516 for COM1 error! %d\n", (int) status);
		return 0;	
	}else
		printf("Configured slot %c OK\n", IPslot);

	//setup com1 port 0 for sending
	status = COM1.Hy85156_ComsSetup(port, baud, parity, stopbits, databits, flowctl, halfduplex, delay485);
	if (status < 0)
	{
		printf("Setup COM1 error! %d\n", (int) status);
		return 0;	
	}else
		printf("Setup UART OK.\n\n");

	/* main business */
    int num;

	for(i=startport; i<9; i++)
	{
		if(i==startport)
		{
			printf("\nWhen receiver program is running and cable to SK%d is ready, hit any key.\n", i);
			getchar();
		}			
		else
		{
			printf("\nNow try to test PORT %d...\n", i-1);
			getchar();
//			sleep(2);
		}
   	    sprintf((char*)buf, "%s SK%d\n\a", "Message is sent to", i);
   	    printf("Sent:%s", buf);

		no = 0;
		do
		{
			sleep(1);							      /* wait for 3 second */
    	    sprintf((char*)buf, "%s SK%d\n\a", "Message from SK1 to", i);
//printf("before send port=%d\n", port);
    	    status = COM1.Hy85156_Write(port, buf, strlen((char*)buf));
    	    
    	    num = 0;
    	    do
    	    {
//getchar();
    	        status = COM1.Hy85156_Read(port, &buf[num]);
//printf("port=%d\n", port);
    	        if(status > 0)
    	        {
    	            num += status;
					buf[num]=0;
    	            if (strstr((const char*)buf, "\n\a") != NULL)
    	            {
    	                printf("RcvNo=%d : %s", num, buf);
    	                num = 0;
    	                break; 
    	            }else
					{
						for(i=0;i<num;i++)
							printf("%x ", buf[i]);   
						printf("num=%d\n", num);
					}
    	            status = 0;
    	        }
				usleep(2000);
    	    }while(status == 0);
			no++;
			if(no >= 5) break;
    	}while(1);
	}
    return(0);
}

	
