/********************************************************************************/
/*    H   H Y   Y TTTTT EEEEE  CCC       HYTEC ELECTRONICS LTD                  */
/*    H   H  Y Y    T   E     C          5 Cradock Road,                        */
/*    HHHHH   Y     T   EEE   C          Reading, Berks.    Tel: 0118 9757770	*/
/*    H   H   Y     T   E     C          RG2 0JT            Fax: 0118 9757566	*/
/*    H   H   Y     T   EEEEE  CCC       Web: www.hytec-electronics.co.uk       */
/********************************************************************************/
/*                                                                              */
/*			IOC9010 PROJECT	--- Hytec IP Card API Library Testing Program		*/
/*                                                                              */
/********************************************************************************/
/*  Source file name      :-      IOC9010FATTest.c								*/
/*  Definition header file:		  HYTECIPAPI.h									*/
/*								  Hy85056.h										*/
/*  Platform			  :		  Linux, Windows XP, Windows 2000               */
/*  Initial creation date :-      23-DEV-2011                                   */
/*                                                                              */
/*  Original Developers   :-      Jim Chen		                                */
/********************************************************************************/
/*                                                                              */
/*  Description :-                                                              */
/*                                                                              */
/*  This is a test program to test IOC9010 connectors.                         	*/
/*                                                                              */
/*																				*/
/*  (C)2008 Hytec Electronics Ltd.												*/
/*                                                                              */
/********************************************************************************/
/*                                                                              */
/*  Modification record :-                                                      */
/*                                                                              */
/*  Date modified |     Person    |   Description of modification               */
/* ---------------|---------------|-------------------------------------------- */
/*    23/12/2011  |	    JC        |	  Initial copy								*/
/*                |               |                                             */
/*                                                                              */
/********************************************************************************/
/* To include Concurrent support, make the program by using
   make -f makefile_CCT */

#include "../include/HytecIPAPI.h"
#include "../include/Hy85056.h"
 
#include <math.h>

/**************To simulate getch() function **********/
#include <termios.h>
#include <unistd.h>
int getsch( ) 
{
  struct termios oldt, newt;
  int            ch;
  tcgetattr( STDIN_FILENO, &oldt );
  newt = oldt;
  newt.c_lflag &= ~( ICANON | ECHO );
  tcsetattr( STDIN_FILENO, TCSANOW, &newt );
  ch = getchar();
  tcsetattr( STDIN_FILENO, TCSANOW, &oldt );
  return ch;
}
/**************To simulate getch() function **********/

int main(void)
{
    int i,j;
    long status;
	int vmeslot = 2;        //ignored by IOC9010
	status = 0;

	printf("\nTesting Program starts...... \n\n");


//===============Read Identities from IOC Box example===============

	CHy85056 ioc;

	USHORT id_buffer[14];
	USHORT carrier;
//	USHORT CSR;

//	status = ioc.Read3331CSR(&CSR);
//    printf("3331 CSR is %04X\n",CSR);

    //ioc.IOC9010SetCarrierFanSpeed(1,1);
	status = ioc.IOC9010GetCarrier(vmeslot, &carrier);

    printf("Carrier Card is %04X\n",carrier);

    if (carrier == 0x8002)
    {
        printf("Detect IP card installed.....\n\n");
        for (j=0;j<4;j++)
        {
            status = ioc.IOC9010GetConfig(vmeslot, j, 14, id_buffer);
            if (status == C_ERR_NODEVICE)
            {
                printf("Device open failed!");
                break;
            }
            else
            {
                printf("Slot %c IP identity: ", (char) (j+0x41));
                for (i=0;i<14;i++)
    				printf("%X ",id_buffer[i]);
                printf("\n");
            }
	   }
    }
    else
    {
        //printf("This Carrier Has NO IP Cards !\n"); 

        printf("Detect IP card installed.....\n\n");
        for (j=0;j<6;j++)
        {
            status = ioc.IOC9010GetConfig(vmeslot, j, 14, id_buffer);
            if (status == C_ERR_NODEVICE)
            {
                printf("Device open failed!");
                break;
            }
            else
            {
                printf("Slot %c IP identity: ", (char) (j+0x41));
                for (i=0;i<14;i++)
    				printf("%X ",id_buffer[i]);
                printf("\n");
            }
	   }
    }	

//
// If the 8506 modules are predsent we can now create 2 threads to run the tests
//
    char IPslot_out;
    int debrate = 0;
    USHORT debmask = 0;
    int scanrate = 0;
    USHORT vectornum1 = 0;
    USHORT intmask = 0;
    int pwidth = 0;
    USHORT pmask = 0;
    int clock = 0;
    int dir;

	int flag;
	do
	{
		printf("Please type IP slot (A ~ F):\n");
		IPslot_out = getsch() & 0xFF;		
		if(IPslot_out >0x60) IPslot_out=IPslot_out-0x20;
		if((IPslot_out-0x40>0) && (IPslot_out-0x40<6)) flag = 1;
	}while(flag == 0);

	dir = 3;		//all outputs
	status = ioc.Hy85056_Configure(vmeslot, IPslot_out, 0, debrate, debmask, 
                scanrate, dir, vectornum1, intmask, pwidth, pmask, clock);
	if (status == C_ERR_NODEVICE)
	{
		printf("Driver error!\n");
		return 0;	
	}
	status = ioc.Hy85056_Configure(vmeslot, IPslot_out, 1, debrate, debmask, 
                scanrate, dir, vectornum1, intmask, pwidth, pmask, clock);
	if (status == C_ERR_NODEVICE)
	{
		printf("Driver error!\n");
		return 0;	
	}
	status = ioc.Hy85056_Configure(vmeslot, IPslot_out, 2, debrate, debmask, 
                scanrate, dir, vectornum1, intmask, pwidth, pmask, clock);
	if (status == C_ERR_NODEVICE)
	{
		printf("Driver error!\n");
		return 0;	
	}
	
	printf("\nAll LEDs on.\n");
	status = ioc.Hy85056_Write(0, 0xFFFF, 0xFFFF);
	status = ioc.Hy85056_Write(1, 0xFFFF, 0xFFFF);
	status = ioc.Hy85056_Write(2, 0xFFFF, 0xFFFF);
	getchar();

	printf("All LEDs off.\n");
	status = ioc.Hy85056_Write(0, 0xFFFF, 0);
	status = ioc.Hy85056_Write(1, 0xFFFF, 0);
	status = ioc.Hy85056_Write(2, 0xFFFF, 0);
	getchar();

	printf("LED Test ");
	for(j=0; j<3; j++)
	{
		for (i=0; i<16; i++)
		{
			status = ioc.Hy85056_Write_Bit(j, i, 1);
			if (status != 0) 
			{
				printf("Write error! %d\n", (int)status);
				return 0;
			}else
				printf(".");

			usleep(200000);

			status = ioc.Hy85056_Write_Bit(j, i, 0);
			if (status != 0) 
			{
				printf("Write error! %d\n", (int)status);
				return 0;
			}else
				printf(".");

			usleep(200000);
		}
	}
	printf("Test finished!\n");

    return(0);

}


