/********************************************************************************/
/*    H   H Y   Y TTTTT EEEEE  CCC       HYTEC ELECTRONICS LTD                  */
/*    H   H  Y Y    T   E     C          5 Cradock Road,                        */
/*    HHHHH   Y     T   EEE   C          Reading, Berks.    Tel: 0118 9757770	*/
/*    H   H   Y     T   E     C          RG2 0JT            Fax: 0118 9757566	*/
/*    H   H   Y     T   EEEEE  CCC       Web: www.hytec-electronics.co.uk       */
/********************************************************************************/
/*                                                                              */
/*			IOC9010 PROJECT	--- Hytec IP Card API Library Testing Program		*/
/*                                                                              */
/********************************************************************************/
/*  Source file name      :-      Test8506.c									*/
/*  Definition header file:		  HYTECIPAPI.h									*/
/*								  Hy85056.h										*/
/*  Platform			  :		  Linux, Windows XP, Windows 2000               */
/*  Initial creation date :-      25-NOV-2008                                   */
/*                                                                              */
/*  Original Developers   :-      Mark Woodward                                 */
/********************************************************************************/
/*                                                                              */
/*  Description :-                                                              */
/*                                                                              */
/*  This is a test program to try and reproduce a possible problem between the  */
/*  3331 controller and 8506 ip cards.                                       	*/
/*                                                                              */
/*																				*/
/*  (C)2008 Hytec Electronics Ltd.												*/
/*                                                                              */
/********************************************************************************/
/*                                                                              */
/*  Modification record :-                                                      */
/*                                                                              */
/*  Date modified |     Person    |   Description of modification               */
/* ---------------|---------------|-------------------------------------------- */
/*    25/11/08    |	    MW        |	  Initial copy								*/
/*                |               |                                             */
/*                                                                              */
/********************************************************************************/


#include "../include/HytecIPAPI.h"
#include "../include/Hy85056.h"

#include <pthread.h>
#include <math.h>

void *test_function(void *);

int vmeslot = 2;        //ignored by IOC9010
int TRUN[3];

int main(void)
{
    int i,j;
    long status;
    pthread_t thread1,thread2,thread3;

	status = 0;

	printf("\nTesting Program starts...... \n\n");


//===============Read Identities from IOC Box example===============

	CHy85056 ioc;

	USHORT id_buffer[14];
	USHORT carrier;
	USHORT CSR;

	status = ioc.Read3331CSR(&CSR);
    printf("3331 CSR is %04X\n",CSR);

    //ioc.IOC9010SetCarrierFanSpeed(1,1);
	status = ioc.IOC9010GetCarrier(vmeslot, &carrier);

    printf("Carrier Card is %04X\n",carrier);

    if (carrier == 0x8002)
    {
        printf("Detect IP card installed.....\n\n");
        for (j=0;j<4;j++)
        {
            status = ioc.IOC9010GetConfig(vmeslot, j, 14, id_buffer);
            if (status == C_ERR_NODEVICE)
            {
                printf("Device open failed!");
                break;
            }
            else
            {
                printf("Slot %c IP identity: ", (char) (j+0x41));
                for (i=0;i<14;i++)
    				printf("%X ",id_buffer[i]);
                printf("\n");
            }
	   }
    }
    else
    {
        //printf("This Carrier Has NO IP Cards !\n"); 

        printf("Detect IP card installed.....\n\n");
        for (j=0;j<6;j++)
        {
            status = ioc.IOC9010GetConfig(vmeslot, j, 14, id_buffer);
            if (status == C_ERR_NODEVICE)
            {
                printf("Device open failed!");
                break;
            }
            else
            {
                printf("Slot %c IP identity: ", (char) (j+0x41));
                for (i=0;i<14;i++)
    				printf("%X ",id_buffer[i]);
                printf("\n");
            }
	   }
    }	

//
// If the 8506 modules are predsent we can now create 2 threads to run the tests
//
    TRUN[0]=TRUN[1]=TRUN[2]=0;
    pthread_create(&thread1, NULL, (void*(*)(void*))test_function, (void *)(0));
    pthread_create(&thread2, NULL, (void*(*)(void*))test_function, (void *)(1));
    pthread_create(&thread3, NULL, (void*(*)(void*))test_function, (void *)(2));

    sleep(2);
    while(TRUN[0] || TRUN[1] || TRUN[2])
        sleep(1); 

}

/**********************************************************************************/
void *test_function(void *ptr)
{

    printf("Starting thread for port %d\n",(int)ptr);
    TRUN[(int)ptr]=1;

	CHy85056 IO8506_in;
	CHy85056 IO8506_out;

    int i;
    long status;
    char IPslot_in;
    char IPslot_out;
    int debrate = 0;
    USHORT debmask = 0;
    int scanrate = 0;
    USHORT vectornum1 = 0;
    USHORT intmask = 0;
    int pwidth = 0;
    USHORT pmask = 0;
    int clock = 0;
	int IOport;
    USHORT Wvalue;
    USHORT Rvalue;
    int dir;

    IOport = (int)ptr;

    switch (IOport)
    {
        case 0:
        case 2:
            IPslot_in = 'A';
            IPslot_out = 'D';
            break;

        case 1:
            IPslot_in = 'D';
            IPslot_out = 'A';
            break;
    }

	printf("THREAD %d Initialise IPslot %c port %d as Output\n", (int)ptr, IPslot_out, (int)IOport);
    dir = 3;
	status = IO8506_out.Hy85056_Configure(vmeslot, IPslot_out, IOport, debrate, debmask, 
                scanrate, dir, vectornum1, intmask, pwidth, pmask, clock);
	if (status == C_ERR_NODEVICE)
	{
		printf("Driver error!\n");
		return 0;	
	}
//	printf(" -Success %d\n\n",(int)status);	
	
	printf("THREAD %d Initialise IPslot %c port %d as Input\n", (int)ptr, IPslot_in, (int)IOport);
	dir = 0;
	status = IO8506_in.Hy85056_Configure(vmeslot, IPslot_in, IOport, debrate, debmask, 
                scanrate, dir, vectornum1, intmask, pwidth, pmask, clock);
	if (status == C_ERR_NODEVICE)
	{
		printf("Driver error!\n");
		return 0;	
	}
//	printf(" -Success %d\n\n",(int)status);	


	for (i=0; i<50; i++)
	{
   		Wvalue = rand() & 0xFFFF;

//		printf("Set IPslot %c port 0 to 0x%X\n", IPslot_out,(int)value);
		status = IO8506_out.Hy85056_Write(IOport, 0xFFFF, Wvalue);
		if (status != 0) 
		{
			printf("Write error! %d\n", (int)status);
			return 0;
		}
		usleep(rand() & 0x7ffff);


        status = IO8506_in.Hy85056_Read(IOport, &Rvalue, 0xFFFF);
		if (status != 0) 
		{
			printf("Read error! %d\n", (int)status);
			return 0;
		}
        if(Rvalue != Wvalue)
        	printf("THREAD %d ERROR port %d - WROTE %X  READ %x\n", (int)ptr, (int)IOport, (int)Wvalue, (int)Rvalue);
//		printf(" ----> Read IPslot %c port 0 value 0x%02X\n", IPslot_in,(int)value);
	}

    TRUN[(int)ptr]=0;

    return(0);
}
