/********************************************************************************/
/*    H   H Y   Y TTTTT EEEEE  CCC       HYTEC ELECTRONICS LTD                  */
/*    H   H  Y Y    T   E     C          5 Cradock Road,                        */
/*    HHHHH   Y     T   EEE   C          Reading, Berks.    Tel: 0118 9757770	*/
/*    H   H   Y     T   E     C          RG2 0JT            Fax: 0118 9757566	*/
/*    H   H   Y     T   EEEEE  CCC       Web: www.hytec-electronics.co.uk       */
/********************************************************************************/
/*                                                                              */
/*			IOC9010 PROJECT	--- Hytec IP Card API Library Testing Program		*/
/*                                                                              */
/********************************************************************************/
/*  Source file name      :-      Test8516.c									*/
/*  Definition header file:		  HYTECIPAPI.h									*/
/*								  Hy8516.h										*/
/*  Platform			  :		  Linux, Windows XP, Windows 2000               */
/*  Initial creation date :-      21-OCT-2009                                   */
/*                                                                              */
/*  Original Developers   :-      Jim Chen	                                    */
/********************************************************************************/
/*                                                                              */
/*  Description :-                                                              */
/*                                                                              */
/*  This is the master node test program to test Hytec 8516 RS485 communication.*/
/*  It sends a message every 3 second with header changed from 1 to 7 to      	*/
/*  simulate message calls for slave 1 to 7. When slave gets its call, it       */
/*	replies. Note, if there is not reply from slave, this program hangs.        */
/*																				*/
/*  (C)2009 Hytec Electronics Ltd.												*/
/*                                                                              */
/********************************************************************************/
/*                                                                              */
/*  Modification record :-                                                      */
/*                                                                              */
/*  Date modified |     Person    |   Description of modification               */
/* ---------------|---------------|-------------------------------------------- */
/*    21/10/09    |	    JC        |	  Initial copy								*/
/*                |               |                                             */
/*                                                                              */
/********************************************************************************/


#include "../include/HytecIPAPI.h"
#include "../include/Hy85156.h"

#include <pthread.h>
#include <math.h>

/* Global variables */
  
static CHy85156 COM1;
static long status;
char *request_to_slave[] = {"1 call to number one",
                             "2 message to two please",
                             "3 hi number three",
                             "4 i am talking to you",
                             "5 who are you",
                             "6 are you there",
                             "7 last port call"};

int slave = 0;
static int port = 0;

int IPslot = 'D';
char vector = 0x0;				/* no interrupt */
int delay = 0;
int halfduplex = 2;
int delay485 = 5;

int vmeslot = 0;
	
ULONG baud = 9600;
char parity = 'N'; 
int stopbits = 1;
int databits = 8;
char flowctl = 0;
int i;

/* Global variables */

int main(void)
{ 
	status = 0;
    UCHAR buf[128];

	printf("\nTesting Program starts......");

	//generic settings for the 8515 COM1
	status = COM1.Hy85156_Configure(vmeslot, IPslot, vector, delay);
	if (status < 0)
	{
		printf("Configure 8515 for COM1 error! %d\n", (int) status);
		return 0;	
	}else
		printf("Configured slot %c OK..", IPslot);

	//setup com1 port 0 for sending
	status = COM1.Hy85156_ComsSetup(port, baud, parity, stopbits, databits, flowctl, halfduplex, delay485);
	if (status < 0)
	{
		printf("Setup COM1 error! %d\n", (int) status);
		return 0;	
	}else
		printf("Setup UART OK.\n\n");

	/* main business */
    int num;
    int msgno = 0;
	do
	{
		printf("Before sending...\n");
		sleep(1);							      /* wait for 3 second */
        status = COM1.Hy85156_Write(port, (UCHAR*)request_to_slave[msgno], strlen((char*)request_to_slave[msgno]));
        printf("Sent:%s\n\r", request_to_slave[msgno]);
        
        int counter = 0;
        num = 0;
        do
        {
            status = COM1.Hy85156_Read(port, &buf[num]);
            if(status > 0)
            {
                num += status;
                if (num >= 23)
                {
                    buf[num]=0;
                    printf("RcvNo=%d : %s\n\r", num, buf);

                    num = 0;
                    break;
                }
				//printf("received %d chars: %x\n\r", num, buf[0]);   
                status = 0;
            }else
            {
                counter++;
                if (counter < 0) break;
            }
        }while(status == 0);
    }while(1);

    return(0);
}

	
