/********************************************************************************/
/*    H   H Y   Y TTTTT EEEEE  CCC       HYTEC ELECTRONICS LTD                  */
/*    H   H  Y Y    T   E     C          5 Cradock Road,                        */
/*    HHHHH   Y     T   EEE   C          Reading, Berks.    Tel: 0118 9757770	*/
/*    H   H   Y     T   E     C          RG2 0JT            Fax: 0118 9757566	*/
/*    H   H   Y     T   EEEEE  CCC       Web: www.hytec-electronics.co.uk       */
/********************************************************************************/
/*                                                                              */
/*			Linux PROJECT	--- Hytec Sky Scanner Testing Program				*/
/*                                                                              */
/********************************************************************************/
/*  Source file name      :-      TipTest.c										*/
/*  Definition header file:		  HYTECIPAPI.h									*/
/*								  Hy8402.h										*/
/*								  Hy85056.h										*/
/*								  Hy85156.h										*/
/*								  HyTip120.h									*/
/*  Platform			  :		  Linux, Windows XP, Windows 2000               */
/*  Initial creation date :-      01-Feb-2010                                   */
/*                                                                              */
/*  Original Developers   :-      Jim S Chen                                    */
/********************************************************************************/
/*                                                                              */
/*  Description :-                                                              */
/*                                                                              */
/*  This is for testing Hytec IP card API library for Sky Scanner job. It 		*/
/*	inlcudes testing IP cards such as 8402, 8505/8506, 8515/8516 and Tip120		*/
/*                                                                              */
/*  Uncomment each macro define on the top that associates to the particular 	*/
/*	IP card to run the test.													*/
/*																				*/
/*  (C)2010 Hytec Electronics Ltd.												*/
/*                                                                              */
/********************************************************************************/
/*                                                                              */
/*  Modification record :-                                                      */
/*                                                                              */
/*  Date modified |     Person    |   Description of modification               */
/* ---------------|---------------|-------------------------------------------- */
/*    17/01/10    |	  Jim S Chen  |	  Initial copy								*/
/*                |               |                                             */
/*                                                                              */
/********************************************************************************/

//#define TEST_ID
//#define LOOP_TEST_8402_TO_8411 
//#define TEST_8005 
//#define TEST_8402 
//#define TEST_8403 
//#define TEST_8404 
//#define TEST_8408 
//#define TEST_8411 
//#define TEST_8413
//#define TEST_8505 
//#define TEST_8506 
//#define TEST_8512 
//#define TEST_85156 
//#define TEST_8601
#define TEST_TIP120

#include "../include/HytecIPAPI.h"
#include "../include/Hy8005.h"
#include "../include/Hy8402.h"
#include "../include/Hy8403.h"
#include "../include/Hy8404.h"
#include "../include/Hy8408.h"
#include "../include/Hy8411.h"
#include "../include/Hy8413.h"
#include "../include/Hy85056.h"
#include "../include/Hy8512.h"
#include "../include/Hy85156.h"
#include "../include/Hy8601.h"
#include "../include/HyTip120.h"

#include <pthread.h>
#include <math.h>
// global because its function need to be called by interrupt callback function which is run in another thread
CHyTip120 tp120;

EN_VME_INT_INFO int_info;		//interrupt structure that holds the interrupt information
int status;
int tipStatus;
int vmeslot = 3;        
char IPslot;
int level = 6;
int flag;						//flag indicates the motor movement has been stopped if set.

//CHy85056 IO1;

// interrupt callback function
void callback(void *drv)
{
	int i;
	int intreason;

	printf("\n*****************************\n");
	printf("         Interrupted         \n");
	printf("=============================\n");

#ifdef TEST_TIP120
	EN_VME_INT_INFO *d = (EN_VME_INT_INFO *)drv;
	printf("number of interrupt=%d\n", d->numOfInts);
	if (d->numOfInts > 0)
	{
		for(i=0; i<(int)d->numOfInts; i++)
		{
			switch(d->vectors[i])
			{
				case 0x88:
					status = tp120.HyTip120_INTReason(0, &intreason);
					if (status !=0)
					{
						printf("Get interrupt reason error. %x \n",status);
					}else
					{
						if (intreason != 0)
						{
							printf("intreason=0x%x\n", intreason);
							if(intreason & INT_TRAJECTORYCOMPLETE) flag = 1;
							status = tp120.HyTip120_ResetInterrupt(0, intreason);
							printf("vec[%d]=%x\n", i, d->vectors[i]);
						}
					}
					break;
			}
		}
	}	
#endif

#ifdef TEST_8505
	status = IO1.Hy85056_ResetInterrupt();
	if (status !=0)
	{
		printf("Get intreason error. %x \n",status);
	}
#endif
	printf("*****************************\n\n");
}


int main(void)
{
	int i, j;
	USHORT value;	
	ULONG lvalue;	

	i = 0;					//to make the compiler happy
	j = 0;
	IPslot = 'B';
	value = 0;
	lvalue = 0;
	status = 0;

	printf("\nTesting Program starts...... \n\n");


//====================TIP120 test example===========================
//              Assume TIP120 IP is in slot D                      =
//==================================================================
#ifdef TEST_TIP120

//-----------C++ version--------
//-----------C++ version--------

//-----------C version--------
	//CHyTip120 *tp120;
	//tp120 = CreateCHyTIP120();
	//tp120->HyTIP120_Configure(...);
	//.....replace every . operator to ->
//-----------C version--------
   
    //configuration data for DAC
    IPslot = 'D';
	vmeslot = 3;

    int tip120vector = 0x88;
	int channel = 0;
	flag = 0;

//    int speed = 100;
//    int acceleration = 10;
    int degree;
	ULONG desiredp;
	int INTMask = INT_INDEXPULSE | INT_TRAJECTORYCOMPLETE | INT_ENABLELM628;

	// Configure the Tip120 IP card
printf("Configure TIP120......");
	status = tp120.HyTip120_Configure(vmeslot, IPslot, level, tip120vector);
	if (status !=0)
	{
		printf("error! -- %d\n", status);
		return 0;	
	}else
		printf("OK\n");

	sleep(1);

	// set up axis 0 for position
printf("Set up motor channel %d......", channel);
	status = tp120.HyTip120_MotorSetup(channel, 1, INTMask, 0,
                &callback, &int_info);
	if (status !=0)
	{
		printf("error! -- %d\n", status);
		return 0;	
	}else
		printf("ok\n");

	// enable interrupt
	status = tp120.HyTip120_EnableInterrupt(channel);

	sleep(1);

	// move to 70 degree
	degree =1;
//again:
	printf("Command motor to run at 60 rpm......");
	
	status = tp120.HyTip120_Run(channel, 60, 200, degree);
	if (status !=0)
	{
		printf("error! -- %d\n", status);
		return 0;	
	}else
		printf("ok\n\n");


//try to see the status and so on
	for(i=0;i<10;i++)
	{
		printf("i=%d, Status......", i);
		status = tp120.HyTip120_Status(channel, &tipStatus);
		if (status !=0)
		{
			printf("error. %x \n",status);
			break;	
		}
		printf("0x%x\n",tipStatus);
	
		printf("Desired position......");
		status = tp120.HyTip120_ReadDesiredPosition(channel, &desiredp);
		if (status !=0)
		{
			printf("error. %d \n",status);
			break;	
		}
		printf("%d\n",(int)desiredp);
		printf("Real position......");
		status = tp120.HyTip120_ReadRealPosition(channel, &desiredp);
		if (status !=0)
		{  
			printf("error. %x \n",status);
			break;	
		}
		printf("%d\n",(int)desiredp);
		printf("Desired velocity......");
		status = tp120.HyTip120_ReadDesiredVelocity(channel, &desiredp);
		if (status !=0)
		{
			printf("error. %d \n",status);
			break;	
		}
		printf("%d\n",(int)desiredp);
		
		printf("Real velocity......");
		status = tp120.HyTip120_ReadRealVelocity(channel, (unsigned int*) &desiredp);
		if (status !=0)
		{  
			printf("error. %x \n",status);
			break;	
		}
		printf("%d\n\n",(int)desiredp);

		if(flag) break;

		usleep(1000000);
	}
	printf("Move completed. \n");

//	getchar();
//	degree =80;
//	goto again;

	status = tp120.HyTip120_Stop(channel, 0);		//stop immediately
	if (status !=0)
	{  
		printf("error. %x \n",status);	
	}

	printf("TIP120 Test finished. \n\n");

//-----------C version--------
	//DeleteCHyTip120(tip120);
//-----------C version--------

	return 0;
#endif
//==================End of 8402 test example=========================



//===============Read Identities from IOC Box example===============
#ifdef TEST_ID

	CHy8402 ioc;

	USHORT id_buffer[14];
	USHORT carrier;
	USHORT CSR;

	status = ioc.Read5331CSR(&CSR);
    printf("5331 CSR is %04X\n",CSR);

    //ioc.IOC9010SetCarrierFanSpeed(1,1);
	status = ioc.IOC9010GetCarrier(vmeslot, &carrier);

    printf("Carrier Card is %04X\n",carrier);

    if (carrier == 0x8002)
    {
        printf("Detect IP card installed.....\n\n");
        for (j=0;j<4;j++)
        {
            status = ioc.IOC9010GetConfig(vmeslot, j, 14, id_buffer);
            if (status == C_ERR_NODEVICE)
            {
                printf("Device open failed!");
                break;
            }
            else
            {
                printf("Slot %c IP identity: ", (char) (j+0x41));
                for (i=0;i<14;i++)
    				printf("%X ",id_buffer[i]);
                printf("\n");
            }
	   }
    }
    else
    {
        //printf("This Carrier Has NO IP Cards !\n"); 

        printf("Detect IP card installed.....\n\n");
        for (j=0;j<6;j++)
        {
            status = ioc.IOC9010GetConfig(vmeslot, j, 14, id_buffer);
            if (status == C_ERR_NODEVICE)
            {
                printf("Device open failed!");
                break;
            }
            else
            {
                printf("Slot %c IP identity: ", (char) (j+0x41));
                for (i=0;i<14;i++)
    				printf("%X ",id_buffer[i]);
                printf("\n");
            }
	   }
    }	
	printf("\n\n");
	status = ioc.Read5331CSR(&CSR);
    printf("5331 CSR is %04X\n",CSR);

//    printf("Resetting 3331\n");
//	status = ioc.Reset3331();
//    printf("3331 Reset status = %x\n",(int)status);

//	status = ioc.Read5331CSR(&CSR);
//    printf("5331 CSR is %04X\n",CSR);
//	printf("\n\n");

#endif
//===============Read Identities from IOC Box example===============

//====================8402 test example=============================
//              Assume 8402 IP is in slot B                        =
//==================================================================
#ifdef TEST_8402
	unsigned short int readback;

//-----------C++ version--------
	CHy8402 DAC2;
//-----------C++ version--------

//-----------C version--------
	//CHy8402 *DAC1;
	//DAC1 = CreateCHy8402();
	//DAC1->Hy8402_Configure(...);
	//.....replace every . operator to ->
//-----------C version--------
   
    //configuration data for DAC
    char dacIPslot1 = 'C';
    int vectornum2 = 0;
    int doram1 = 0;
    int clockSource1 = 0;
    int dacClockRate1 = 13;
    int inhibit1 = 0;

vmeslot = 3;

	status = DAC2.Hy8402_Configure(vmeslot, dacIPslot1, vectornum2, 
                doram1, clockSource1, dacClockRate1, inhibit1);
	if (status == C_ERR_NODEVICE)
	{
		printf("Driver error!\n");
		return 0;	
	}else
		printf("Configure IP card on slot %c successfully. Return value = %d \n", dacIPslot1, (int)status);

	//loop 10 times to update ADC channel 0 from 0 to 65530 with interval of 6553
	for (i = 0; i<10; i++)
	{
		status = DAC2.Hy8402_Write(0, i*6553);
		sleep(2);
		status = DAC2.IOC9010IPRead(vmeslot, 2, 0x10, &readback);
		if (status >= 0)
		{
			printf("Output %d to DAC channel %d. Read back = %d \n", i*6553, 0, readback);
		}else

		{
			printf("Write error! %d \n", (int)status);
			return 0;
		} 
	}

	printf("8402 Test finished. Return value = %d \n\n", (int)status);

//-----------C version--------
	//DeleteCHy8402(DAC2);
//-----------C version--------

	//return 0;
#endif
//==================End of 8402 test example=========================


//====================8404 test example=============================
//              Assume 8404 IP is in slot C                        =
//==================================================================
#ifdef TEST_8404
    unsigned short wave[32768];

//-----------C++ version--------
	CHy8404 DAC2;
//-----------C++ version--------

    //configuration data for DAC
    char dacIPslot1   = 'C';
    int vectornum2    = 0;      // Interrupt Vector
    int doram1        = 0;      // 1 - Use RAM 0 - Register 
    int clockSource1  = 0;      // 1 - External 0 - Internal
    int dacClockRate1 = 12;     // 10KHz
    int inhibit1      = 0;      // External Inhibit 1 - Enabled 0 - Disabled

	status = DAC2.Hy8404_Configure(vmeslot, dacIPslot1, vectornum2, 
                doram1, clockSource1, dacClockRate1, inhibit1);
	if (status == C_ERR_NODEVICE)
	{
		printf("Driver error!\n");
		return 0;	
	}
    else
    {
		printf("Configure IP card on slot %c successfully. Return value = %d \n", dacIPslot1, (int)status);
    }

/*
	// loop 10 times to update ADC channel 0 from 0 to 65530 in steps of 6553
    // For RANGE_10V_BIPOLAR_8404 i.e. -10V to +10V
	for (i = 0; i<=10; i++)
	{
		status = DAC2.Hy8404_Write(0, i*6553, RANGE_10V_BIPOLAR_8404);
		sleep(2);
		status = DAC2.IOC9010IPRead(0, 2, 0x10, &readback);
		if (status == 0)
		{
			printf("Output %d to DAC channel %d. Read back = %d \n", i*6553, 0, readback);
		}
        else
		{
			printf("Write error! %d \n", (int)status);
			return 0;
		} 
	}

	// loop 10 times to update ADC channel 0 from 0 to 65530 in steps of 6553
    // For RANGE_5V_BIPOLAR_8404 i.e. -5V to +5V
	for (i = 0; i<=10; i++)
	{
		status = DAC2.Hy8404_Write(0, i*6553, RANGE_5V_BIPOLAR_8404);
		sleep(2);
		status = DAC2.IOC9010IPRead(0, 2, 0x10, &readback);
		if (status == 0)
		{
			printf("Output %d to DAC channel %d. Read back = %d \n", i*6553, 0, readback);
		}
        else
		{
			printf("Write error! %d \n", (int)status);
			return 0;
		} 
	}
*/
	// Load 8404 DAC channels 0 ~ 15 with Waveforms 
    // --------------------------------------------
	
	// Load Wave Array with Waveform Data
    i = 0;
    while ( i < 32768 )
	{
        // 100 Step Saw Tooth Wave Form
        wave[i] = (i % 100) * 650 + 300;

        // 100 Step Square Wave Form
        //(i % 200 > 100)? wave[i] = 65000: wave[i] = 300;

        i++;
    }
    
    // Pass Array to 8404 Memory
    // For RANGE_10V_BIPOLAR_8404 i.e. -10V to +10V
	status = DAC2.Hy8404_Write(0, wave, RANGE_10V_BIPOLAR_8404);
	if (status == 0)
    {
		printf("Output Wave to DAC channel OK\n");
    }
    else
    {
        printf("Wave Update 8402 channel error! %d\n", (int)status);
    } 

	printf("8404 Test finished. Return value = %d \n\n", (int)status);

#endif
//==================End of 8404 test example=========================


//====================8408 test example=============================
//              Assume 8408 IP is in slot E                        =
//==================================================================
#ifdef TEST_8408

//-----------C++ version--------
	CHy8408 ADC2;
//-----------C++ version--------


    //configuration data for ADC
    char adcIPslot1 = 'E';
//    int adcClockRate1 = 15;     // 100KHz
    int adcClockRate1 = 5;      // 50Hz
    USHORT adc_data[256][16];   // Array Format for FIFO Data adc_data[sample][channel]

	status = ADC2.Hy8408_Configure(vmeslot, adcIPslot1, adcClockRate1);
	if (status == C_ERR_NODEVICE)
	{
		printf("Driver error!\n");
		return 0;	
	}
    else
    {
		printf("Configure IP card successfully. Return value = %d \n",(int)status);
    }

/*
    // Individual ADC Channel Reading
	for (i = 0; i<16; i++)
	{
		for(j=0; j<16;j++)
		{	
			status = ADC2.Hy8408_Read(j, &value);
			if (status == 0)
			{
				printf("Channel %d value = 0x%X\n", j, (int) value);
			}else
			{
				printf("Read channel %d error! %d \n", j, (int)status);
				return 0;
			}
		} 
		printf("\n");

		// Wait for Keyboard Press for Next Channel
		getchar();
	}
*/

    // ADC FIFO Reading
	status = ADC2.Hy8408_Read(&adc_data[0][0]);
    
    // For Each ADC Channel...
	for (i = 0; i<16; i++)
	{
        // Display All 256 Samples from the FIFO...
		for(j=0; j<256;j++)
		{	
			printf("Channel %d, value %d = %d\n", i, j, (int) adc_data[j][i]);
		} 
		printf("\n");

		// Wait for Keyboard Press for Next Channel
        getchar();
	}


	printf("8408 Test finished. Return value = %d \n\n", (int)status);

#endif
//====================End of 8408 test example======================


//====================Loop test example=============================
//   This test loops the 8411 SCSI cable back to the 8402 IP       =
//   card and then output data to the 8402 channels to update      =
//   8411 channel readings.                                        =
//   Note: The only constrain to this test is that we can't use    =
//   DC-DC converters for these two cards.                         =
//                                                                 =
//   Assume:                                                       =
//      1. 8411 IP is in slot A and 8402 IP is in slot B           =
//      2. both ADC and DAC ranges are -10V to 10V                 =
//==================================================================
#ifdef LOOP_TEST_8402_TO_8411

    CHy8411 ADC1;
    CHy8402 DAC1;

    //configuration data for ADC
    char adcIPslot = 'C';
    int adcClockRate = 15;
    
    //configuration data for DAC
    char dacIPslot = 'C';
    int vectornum = 0;
    int doram = 0;
    int clockSource = 0;
    int dacClockRate = 13;
    int inhibit = 0;

	status = ADC1.Hy8411_Configure(vmeslot, adcIPslot, adcClockRate);
	if (status != 0)
	{
		printf("Driver error! Status = %d\n", (int) status);
		return 0;	
	}else
		printf("Configure 8411 IP card successfully. Return value = %d \n",(int)status);

	status = DAC1.Hy8402_Configure(vmeslot, dacIPslot, vectornum, 
            doram, clockSource, dacClockRate, inhibit);
	if (status != 0)
	{
		printf("Driver error! Status = %d\n", (int) status);
		return 0;	
	}else
		printf("Configure 8402 IP card successfully. Return value = %d \n", (int)status);

	//update 8402 DAC channel 0 ~ 15 one by one by outputing data from 0 to 65530 
    //with interval of 4095 and read them back via 8411 ADC on the same channel
	
	for (i = 0; i<16; i++)
	{
        //output to 8402 channel i
		status = DAC1.Hy8402_Write(i, i*4095);
		if (status == 0)
			printf("Output %d to DAC channel %d.", i*4095, i);
        else
		{
			printf("Update 8402 channel error! %d\n", (int)status);
			return 0;
		} 
        
		sleep(2);
        //read back from 8411 on the same channel i
		status = ADC1.Hy8411_Read(i, &value);
		if (status == 0)
			printf(" ---- Read ADC channel %d with value = %d \n", i, (int) value);
		else
		{
			printf("Read 8411 channel error! %d \n", (int)status);
			return 0;
		} 
	}
	printf("8411/8402 Loop Test finished. Return value = %d \n\n", (int) status);

    return 0;
#endif
//====================End of loop test for 8411/8402 example======================


//====================8403 test example=============================
//              Assume 8403 VME card is in slot 3                  =
//==================================================================
#ifdef TEST_8403
 
//-----------C++ version--------
	CHy8403 IO8403;
//-----------C++ version--------

//-----------C version--------
	//CHy8403 *IO1;
	//IO1 = CreateCHy8403();
	//IO1->Hy8403_Configure(...);
	//.....replace every . operator to ->
//-----------C version--------

    char IPslot0;
    char IPslot1;
    char IPslot2;
    char IPslot3;
//  type = 0 - Dif	- Differential
//  type = 1 - S/E	- Single Ended
//  type = 2 - PT100	- PT-100 (original)
//  type = 3 - TC	- Thermocouples
//  type = 4 - PTB	- PT-100 with 8212 Transistion Card
//  type = 5 - TCB	- Thermocouples with 8212 Transistion Card
    int type_8403;
    int gain_8403;
	double	Gain;
	int Thermocouple_Type;
#define TYPE_K 1
#define TYPE_T 2
#define TYPE_J 3

    IPslot0 = 'A';
    IPslot1 = 'B';
    IPslot2 = 'C';
    IPslot3 = 'D';

//#define PT100_TEST 1
#define THERM_TEST 1


#ifdef PT100_TEST
    type_8403 = 4;
    gain_8403 = 4;
	status = IO8403.Hy8403_Configure(vmeslot, IPslot1, type_8403, gain_8403);
#endif
#ifdef THERM_TEST
    type_8403 = 5;
    gain_8403 = 64;
	status = IO8403.Hy8403_Configure(vmeslot, IPslot1, type_8403, gain_8403);
	Thermocouple_Type = TYPE_K;
#endif
	if (status)
	{
		printf("Config error!  - return code = %d\n",(int)status);
		return 0;
	}

	printf("Port 0 Initialisation Status = %d\n", (int)status);	
	printf("\n");

    for(i=0;i<1;i++)
    {
//        for(ADCchan=0;ADCchan<16;ADCchan++)
//        {
//        	status = IO8403.Hy8403_Read(ADCchan,(ULONG *)&chanValue);
//        	if (status == C_ERR_NODEVICE)
//        	{
//        		printf("Driver error!\n");
//        		return 0; 	
//            }
////            chanValue = chanValue * ((2.5/  (1<<gain_8403)  ) / (1<<24));
//            printf("Pass %d  Channel %d - data %X\n",i+1,ADCchan,(int)chanValue);
//    	}
        sleep(2);

//        showall();
/**************************************/
#ifdef PT100_TEST

    short            iChannelCount;
    unsigned long    ADC_Value;

    double           Voltage;

    double           Temperature;
    double           Rref = 100.0; /* Reference/Vish Resistor is 100R */
    double           IrefA;
    double           IrefB;
    double           A = 0.00390802;
    double           B = -0.000000580195;

	Gain = gain_8403;
   
    /* Measure ADC A's Reference Voltage */
   	IO8403.Hy8403_Read(7,(ULONG *)&ADC_Value);
//    Hy8403_read_ai( (word)(card),
//                    (word)(7),
//                    "SMOO",
//                    &ADC_Value);
    
    /* Calculate ADC A's Common Current */
    Voltage = (double)(2.5 * ADC_Value) / (double)(16777216);
    IrefA = Voltage/Rref;

   
    /* Measure ADC B's Reference Volatage */
   	IO8403.Hy8403_Read(15,(ULONG *)&ADC_Value);
//    Hy8403_read_ai( (word)(card),
//                    (word)(15),
//                    "SMOO",
//                    &ADC_Value);
    
    /* Calculate ADC B's Common Current */
    Voltage = (double)(2.5 * ADC_Value) / (double)(16777216);
    IrefB = Voltage/Rref;


    /* Display ADC A */ 
    for(iChannelCount = 0; iChannelCount < 7; iChannelCount++)
    {
       	IO8403.Hy8403_Read(iChannelCount,(ULONG *)&ADC_Value);
                              
        Voltage = (double)(2.5 * ADC_Value) / (double)(16777216);
        
        Temperature = (Rref - (Voltage/IrefA)) / (-0.5 * (A*Rref + sqrt( (A*A * Rref*Rref) - 4*B*Rref * (Rref - Voltage/IrefA))));                               
        
        printf("Channel %2d- 0x%08x - %8d - %1.6f V - %1.1f°C\n",iChannelCount,
                                            (unsigned int)ADC_Value,
                                            (unsigned int)ADC_Value,
                                            Voltage/4,
                                            Temperature); 
    }
    
    
    /* Display ADC B */ 
    for(iChannelCount = 8; iChannelCount < 15; iChannelCount++)
    {
       	IO8403.Hy8403_Read(iChannelCount,(ULONG *)&ADC_Value);
                              
        Voltage = (double)(2.5 * ADC_Value) / (double)(16777216);
        
        Temperature = (Rref - Voltage/IrefB)/(-0.5 * (A*Rref + sqrt((A*A * Rref*Rref) - 4*B*Rref * (Rref - Voltage/IrefB))));                               
        
        printf("Channel %2d- 0x%08x - %8d - %1.6f V - %1.1f°C\n",iChannelCount,
                                            (unsigned int)ADC_Value,
                                            (unsigned int)ADC_Value,
                                            Voltage/4,
                                            Temperature); 
    }

	printf("\n");

#endif
/**************************************/
#ifdef THERM_TEST

/* READ_AI Offsets for Reading Calibration Values */
#define ADC_A_ZERO_CAL  16  /* ADC A - Zero Raw Calibration */
#define ADC_A_POS_CAL   17  /* ADC A - Pos  Raw Calibration */
#define ADC_A_NEG_CAL   18  /* ADC A - Neg  Raw Calibration */
#define ADC_B_ZERO_CAL  19  /* ADC B - Zero Raw Calibration */
#define ADC_B_POS_CAL   20  /* ADC B - Pos  Raw Calibration */
#define ADC_B_NEG_CAL   21  /* ADC B - Neg  Raw Calibration */
    short           iChannelCount;
    unsigned long   ADC_Value;

	/* PT-100 / Cold Junction Temperature Variables */
    double			Vpt100;		/* PT100 Voltage */
    double			Ipt100;		/* PT100 Current */
    double			Rpt100;		/* PT100 Resistance */
    double			Tpt100;		/* PT100 Temperature */
    double           Rref = 100.0; /* Reference/Vish Resistor is 100R */
    double           A = 0.00390802;
    double           B = -0.000000580195;

	double           Voltage;
    double           Temperature;
    double           TV;
	double d0,d1,d2,d3,d4,d5,d6,d7,d8,d9,d10,d11,d12,d13;
	double a0,a1,a2;
   
	Gain = gain_8403;

    /* Measure PT100 / Cold Junction Temperature ADC B's Reference Voltage */
   	IO8403.Hy8403_Read(15,(ULONG *)&ADC_Value);

	/* Convert PT100 Value into Volts */
	Vpt100 = ((double)(ADC_Value) - 0x800000)*(double)(2.5/Gain) / (double)(16777216);

	/* Calculate Current through PT100 */
	Ipt100 = (double)(2.5 - Vpt100)/(25000);

	/* Calculate Resistance of PT100 */
	Rpt100 = Vpt100/Ipt100;

	/* Calculate Temperature of PT100 */
	Tpt100 = (Rref - Rpt100)/(-0.5 * (A*Rref + sqrt((A*A * Rref*Rref) - 4*B*Rref * (Rref - Rpt100))));                               

    printf("PT100 - 0x%08x - %8d - %1.6f V - %1.6f A - %1.6f R -  %1.2f°C\n",	(unsigned int)ADC_Value,
															                    (unsigned int)ADC_Value,
															                    Vpt100,
															                    Ipt100,
															                    Rpt100,
															                    Tpt100); 

	switch (Thermocouple_Type)
	{
	  case TYPE_K:
	    /* Convert PT100 Temperature to an equivalent K Type Thermocouple Voltage */
		if(Tpt100 < 0.000) {
			d0 =  3.94501280250E-02;
			d1 =  2.36223735980E-05;
			d2 = -3.28589067840E-07;
			d3 = -4.99048287770E-09;
			d4 = -6.75090591730E-11;
			d5 = -5.74103274280E-13;
			d6 = -3.10888728940E-15;
			d7 = -1.04516093650E-17;
			d8 = -1.98892668780E-20;
			d9 = -1.63226974860E-23;
	
			TV =  d0 * (double)(pow(Tpt100,1))
		      	+ d1 * (double)(pow(Tpt100,1))
		        + d2 * (double)(pow(Tpt100,2))   
		        + d3 * (double)(pow(Tpt100,3))   
		        + d4 * (double)(pow(Tpt100,4))   
		        + d5 * (double)(pow(Tpt100,5))   
		        + d6 * (double)(pow(Tpt100,6))   
		        + d7 * (double)(pow(Tpt100,7))   
		        + d8 * (double)(pow(Tpt100,8))   
		        + d9 * (double)(pow(Tpt100,9));   
		}
		else {
			d0 = -1.76004136860E-02;
		 	d1 =  3.89212049750E-02;
		 	d2 =  1.85587700320E-05;
			d3 = -9.94575928740E-08;
		 	d4 =  3.18409457190E-10;
			d5 = -5.60728448890E-13;
		 	d6 =  5.60750590590E-16;
			d7 = -3.20207200030E-19;
		 	d8 =  9.71511471520E-23;
			d9 = -1.21047212750E-26;
			a0 =  0.118597600000E00;
			a1 = -0.118343200000E-03;
			a2 =  0.126968600000E03;
	
			TV =  d0
		      	+ d1 * (double)(pow(Tpt100,1))
		        + d2 * (double)(pow(Tpt100,2))   
		        + d3 * (double)(pow(Tpt100,3))   
		        + d4 * (double)(pow(Tpt100,4))   
		        + d5 * (double)(pow(Tpt100,5))   
		        + d6 * (double)(pow(Tpt100,6))   
		        + d7 * (double)(pow(Tpt100,7))   
		        + d8 * (double)(pow(Tpt100,8))   
		        + d9 * (double)(pow(Tpt100,9));   
	            + ((double) a0 * exp(a1 * (Tpt100-a2) * (Tpt100-a2))); 
		}
		break;

	  case TYPE_T:
	    /* Convert PT100 Temperature to an equivalent T Type Thermocouple Voltage */
		if(Tpt100 < 0.000) {
			d0 =  0.387481063640E-01;
			d1 =  0.441944343470E-04;
			d2 =  0.118443231050E-06;
			d3 =  0.200329735540E-07;
			d4 =  0.901380195590E-09;
			d5 =  0.226511565930E-10;
			d6 =  0.360711542050E-12;
			d7 =  0.384939398830E-14;
			d8 =  0.282135219250E-16;
			d9 =  0.142515947790E-18;
			d10 =  0.487686622860E-21;
			d11 =  0.107955392700E-23;
			d12 =  0.139450270620E-26;
			d13 =  0.797951539270E-30;

			TV =  d0 * (double)(pow(Tpt100,1))
		      	+ d1 * (double)(pow(Tpt100,2))
		        + d2 * (double)(pow(Tpt100,3))   
		        + d3 * (double)(pow(Tpt100,4))   
		        + d4 * (double)(pow(Tpt100,5))   
		        + d5 * (double)(pow(Tpt100,6))   
		        + d6 * (double)(pow(Tpt100,7))   
		        + d7 * (double)(pow(Tpt100,8))   
		        + d8 * (double)(pow(Tpt100,9))   
		        + d9 * (double)(pow(Tpt100,10))   
		        + d10 * (double)(pow(Tpt100,11))   
		        + d11 * (double)(pow(Tpt100,12))   
		        + d12 * (double)(pow(Tpt100,13))   
		        + d13 * (double)(pow(Tpt100,14));   
		}
		else {
			d0 =  0.387481063640E-01;
			d1 =  0.332922278800E-04;
			d2 =  0.206182434040E-06;
			d3 = -0.218822568460E-08;
			d4 =  0.109968809280E-10;
			d5 = -0.308157587720E-13;
			d6 =  0.454791352900E-16;
			d7 = -0.275129016730E-19;

			TV =  d0 * (double)(pow(Tpt100,1))
		      	+ d1 * (double)(pow(Tpt100,2))
		        + d2 * (double)(pow(Tpt100,3))   
		        + d3 * (double)(pow(Tpt100,4))   
		        + d4 * (double)(pow(Tpt100,5))   
		        + d5 * (double)(pow(Tpt100,6))   
		        + d6 * (double)(pow(Tpt100,7))   
		        + d7 * (double)(pow(Tpt100,8));   
		}
		break;

	  case TYPE_J:
	    /* Convert PT100 Temperature to an equivalent J Type Thermocouple Voltage */
		if(Tpt100 < 760.000) {

			d0 =  0.503811878150E-01;
			d1 =  0.304758369300E-04;
			d2 = -0.856810657200E-07;
			d3 =  0.132281952950E-09;
			d4 = -0.170529583370E-12;
			d5 =  0.209480906970E-15;
			d6 = -0.125383953360E-18;
			d7 =  0.156317256970E-22;

			TV =  d0 * (double)(pow(Tpt100,1))
		      	+ d1 * (double)(pow(Tpt100,2))
		        + d2 * (double)(pow(Tpt100,3))   
		        + d3 * (double)(pow(Tpt100,4))   
		        + d4 * (double)(pow(Tpt100,5))   
		        + d5 * (double)(pow(Tpt100,6))   
		        + d6 * (double)(pow(Tpt100,7))   
		        + d7 * (double)(pow(Tpt100,8));   
		}
		else {
			d0 =  0.296456256810E03;
			d1 = -0.149761277860E01;
			d2 =  0.317871039240E-02;
			d3 = -0.318476867010E-05;
			d4 =  0.157208190040E-08;
			d5 = -0.306913690560E-12;

			TV =  d0
		      	+ d1 * (double)(pow(Tpt100,1))
		        + d2 * (double)(pow(Tpt100,2))   
		        + d3 * (double)(pow(Tpt100,3))   
		        + d4 * (double)(pow(Tpt100,4))   
		        + d5 * (double)(pow(Tpt100,5));
		}
		break;
	}

    /* Display ADC A */ 

    for(iChannelCount = 0; iChannelCount < 7; iChannelCount++)
    {
        /* Read Calibrated Value from ADC */
       	IO8403.Hy8403_Read(iChannelCount,(ULONG *)&ADC_Value);

	    /* Convert Thermocouple Raw ADC Value into Volts */
	    Vpt100 = ((double)(ADC_Value) - 0x800000)*(double)(2.5/Gain) / (double)(16777216);


/**** TEST CODE TO FORCE SPECIFIC VALUES FOR DIFFERENT THERMOCOUPLE TYPES THROUGH CALCULATIONS ****/
/*	switch (Thermocouple_Type)
	{
	  case TYPE_K:
		switch (iChannelCount)
		{
		  case 0:
			Vpt100 = -0.005891 - (TV/1000);
			break;
		  case 1:
			Vpt100 = -0.003554 - (TV/1000);
			break;
		  case 2:
			Vpt100 = 0.0000 - (TV/1000);
			break;
		  case 3:
			Vpt100 = 0.004096 - (TV/1000);
			break;
		  case 4:
			Vpt100 = 0.016397 - (TV/1000);
			break;
		  case 5:
			Vpt100 = 0.033275 - (TV/1000);
			break;
		  case 6:
			Vpt100 = 0.048838 - (TV/1000);
			break;
		}
	  	break;

	  case TYPE_T:
		switch (iChannelCount)
		{
		  case 0:
			Vpt100 = -0.005603 - (TV/1000);
			break;
		  case 1:
			Vpt100 = -0.003379 - (TV/1000);
			break;
		  case 2:
			Vpt100 = 0.0000 - (TV/1000);
			break;
		  case 3:
			Vpt100 = 0.004279 - (TV/1000);
			break;
		  case 4:
			Vpt100 = 0.009288 - (TV/1000);
			break;
		  case 5:
			Vpt100 = 0.014862 - (TV/1000);
			break;
		  case 6:
			Vpt100 = 0.020872 - (TV/1000);
			break;
		}
		break;

	  case TYPE_J:
		switch (iChannelCount)
		{
		  case 0:
			Vpt100 = -0.007890 - (TV/1000);
			break;
		  case 1:
			Vpt100 = -0.004633 - (TV/1000);
			break;
		  case 2:
			Vpt100 = 0.0000 - (TV/1000);
			break;
		  case 3:
			Vpt100 = 0.005269 - (TV/1000);
			break;
		  case 4:
			Vpt100 = 0.027393 - (TV/1000);
			break;
		  case 5:
			Vpt100 = 0.051877 - (TV/1000);
			break;
		  case 6:
			Vpt100 = 0.063792 - (TV/1000);
			break;
		}
	}
*/
        /* Add Cold Junction Voltage to Thermocouple Voltage and Convert to Microvolts (i.e. x 1000000) */
        Voltage = ((double)(TV / 1000) + (double)(Vpt100)) * (double)(1000);

		switch (Thermocouple_Type)
		{
		  case TYPE_K:
        /* Convert Voltage to Temperature - Using K Type Formula */
			if(Voltage < 0.00) {
				d0 =  0.0000000E00;
				d1 =  2.5173462E01;
				d2 = -1.1662878E00;
				d3 = -1.0833638E00;
				d4 = -8.9773540E-01;
				d5 = -3.7342377E-01;
				d6 = -8.6632643E-02;
				d7 = -1.0450598E-02;
				d8 = -5.1920577E-04;
				d9 =  0.0000000E00;
			}
			else if(Voltage <= 20.644) {
				d0 =  0.000000E00;
				d1 =  2.508355E01;
				d2 =  7.860106E-02;
				d3 = -2.503131E-01;
				d4 =  8.315270E-02;
				d5 = -1.228034E-02;
				d6 =  9.804036E-04;
				d7 = -4.413030E-05;
				d8 =  1.057734E-06;
				d9 = -1.052755E-08;
			}
			else {
				d0 = -1.318058E02;
				d1 =  4.830222E01;
				d2 = -1.646031E00;
				d3 =  5.464731E-02;
				d4 = -9.650715E-04;
				d5 =  8.802193E-06;
				d6 = -3.110810E-08;
				d7 =  0.000000E00;
				d8 =  0.000000E00;
				d9 =  0.000000E00;
			}

	        Temperature = d0
        			+ d1 * (double)(Voltage)
                    + d2 * (double)(pow(Voltage,2))   
                    + d3 * (double)(pow(Voltage,3))   
                    + d4 * (double)(pow(Voltage,4))   
                    + d5 * (double)(pow(Voltage,5))   
                    + d6 * (double)(pow(Voltage,6))   
                    + d7 * (double)(pow(Voltage,7))   
                    + d8 * (double)(pow(Voltage,8))   
                    + d9 * (double)(pow(Voltage,9));   
			break;

		  case TYPE_T:
        /* Convert Voltage to Temperature - Using T Type Formula */
			if(Voltage < 0.00) {
		        d0 =  0.0000000E00;
		        d1 =  2.5949192E01;
		        d2 = -2.1316967E-01;
		        d3 =  7.9018692E-01;
		        d4 =  4.2527777E-01;
		        d5 =  1.3304473E-01;
		        d6 =  2.0241446E-02;
		        d7 =  1.2668171E-03;
			}
			else {
		        d0 =  0.000000E00;
		        d1 =  2.592800E01;
		        d2 = -7.602961E-01;
		        d3 =  4.637791E-02;
		        d4 = -2.165394E-03;
		        d5 =  6.048144E-05;
		        d6 = -7.293422E-07;
		        d7 =  0.000000E00;
			}

	        Temperature = d0
        			+ d1 * (double)(Voltage)
                    + d2 * (double)(pow(Voltage,2))   
                    + d3 * (double)(pow(Voltage,3))   
                    + d4 * (double)(pow(Voltage,4))   
                    + d5 * (double)(pow(Voltage,5))   
                    + d6 * (double)(pow(Voltage,6))   
                    + d7 * (double)(pow(Voltage,7));   
			break;

		  case TYPE_J:
        /* Convert Voltage to Temperature - Using J Type Formula */
			if(Voltage < 0.00) {

        		d0 =  0.0000000E00;
        		d1 =  1.9528268E01;
        		d2 = -1.2286185E00;
        		d3 = -1.0752178E00;
        		d4 = -5.9086933E-01;
        		d5 = -1.7256713E-01;
        		d6 = -2.8131513E-02;
        		d7 = -2.3963370E-03;
        		d8 = -8.3823321E-05;

			}
			else if(Voltage <= 42.919) {

				d0 =  0.000000E00; 
        		d1 =  1.978425E01;
        		d2 = -2.001204E-01;
        		d3 =  1.036969E-02; 
        		d4 = -2.549687E-04;
        		d5 =  3.585153E-06; 
        		d6 = -5.344285E-08;
        		d7 =  5.099890E-10; 
        		d8 =  0.000000E00; 

			}
			else {

        		d0 = -3.11358187E03;
        		d1 =  3.00543684E02;
        		d2 = -9.94773230E00;
        		d3 =  1.70276630E-01;
        		d4 = -1.43033468E-03;
        		d5 =  4.73886084E-06;
        		d6 =  0.00000000E00;
        		d7 =  0.00000000E00;
        		d8 =  0.00000000E00;

			}
	        Temperature = d0
        			+ d1 * (double)(Voltage)
                    + d2 * (double)(pow(Voltage,2))   
                    + d3 * (double)(pow(Voltage,3))   
                    + d4 * (double)(pow(Voltage,4))   
                    + d5 * (double)(pow(Voltage,5))   
                    + d6 * (double)(pow(Voltage,6))   
                    + d7 * (double)(pow(Voltage,7))   
                    + d8 * (double)(pow(Voltage,8));   
			break;
		}		

        printf("ADC A Channel %2d - Raw ADC - 0x%08x  Temp -  %1.1f°C\n",   (int)iChannelCount+1, 
                                                                            (int)ADC_Value,
                                                                            Temperature); 
    }
    
    /* Display ADC B */ 

    for(iChannelCount = 8; iChannelCount < 14; iChannelCount++)
    {
        /* Read Calibrated Value from ADC */
       	IO8403.Hy8403_Read(iChannelCount,(ULONG *)&ADC_Value);

	    /* Convert Thermocouple Raw ADC Value into Volts */
	    Vpt100 = ((double)(ADC_Value) - 0x800000)*(double)(2.5/Gain) / (double)(16777216);
                              
        /* Add Cold Junction Voltage to Thermocouple Voltage and Convert to Microvolts (i.e. x 1000000) */
        Voltage = ((double)(TV / 1000) + (double)(Vpt100)) * (double)(1000);

		switch (Thermocouple_Type)
		{
		  case TYPE_K:
        /* Convert Voltage to Temperature - Using K Type Formula */
			if(Voltage < 0.00) {
				d0 =  0.0000000E00;
				d1 =  2.5173462E01;
				d2 = -1.1662878E00;
				d3 = -1.0833638E00;
				d4 = -8.9773540E-01;
				d5 = -3.7342377E-01;
				d6 = -8.6632643E-02;
				d7 = -1.0450598E-02;
				d8 = -5.1920577E-04;
				d9 =  0.0000000E00;
			}
			else if(Voltage <= 20.644) {
				d0 =  0.000000E00;
				d1 =  2.508355E01;
				d2 =  7.860106E-02;
				d3 = -2.503131E-01;
				d4 =  8.315270E-02;
				d5 = -1.228034E-02;
				d6 =  9.804036E-04;
				d7 = -4.413030E-05;
				d8 =  1.057734E-06;
				d9 = -1.052755E-08;
			}
			else {
				d0 = -1.318058E02;
				d1 =  4.830222E01;
				d2 = -1.646031E00;
				d3 =  5.464731E-02;
				d4 = -9.650715E-04;
				d5 =  8.802193E-06;
				d6 = -3.110810E-08;
				d7 =  0.000000E00;
				d8 =  0.000000E00;
				d9 =  0.000000E00;
			}

	        Temperature = d0
        			+ d1 * (double)(Voltage)
                    + d2 * (double)(pow(Voltage,2))   
                    + d3 * (double)(pow(Voltage,3))   
                    + d4 * (double)(pow(Voltage,4))   
                    + d5 * (double)(pow(Voltage,5))   
                    + d6 * (double)(pow(Voltage,6))   
                    + d7 * (double)(pow(Voltage,7))   
                    + d8 * (double)(pow(Voltage,8))   
                    + d9 * (double)(pow(Voltage,9));   
			break;

		  case TYPE_T:
        /* Convert Voltage to Temperature - Using T Type Formula */
			if(Voltage < 0.00) {
		        d0 =  0.0000000E00;
		        d1 =  2.5949192E01;
		        d2 = -2.1316967E-01;
		        d3 =  7.9018692E-01;
		        d4 =  4.2527777E-01;
		        d5 =  1.3304473E-01;
		        d6 =  2.0241446E-02;
		        d7 =  1.2668171E-03;
			}
			else {
		        d0 =  0.000000E00;
		        d1 =  2.592800E01;
		        d2 = -7.602961E-01;
		        d3 =  4.637791E-02;
		        d4 = -2.165394E-03;
		        d5 =  6.048144E-05;
		        d6 = -7.293422E-07;
		        d7 =  0.000000E00;
			}

	        Temperature = d0
        			+ d1 * (double)(Voltage)
                    + d2 * (double)(pow(Voltage,2))   
                    + d3 * (double)(pow(Voltage,3))   
                    + d4 * (double)(pow(Voltage,4))   
                    + d5 * (double)(pow(Voltage,5))   
                    + d6 * (double)(pow(Voltage,6))   
                    + d7 * (double)(pow(Voltage,7));   
			break;

		  case TYPE_J:
        /* Convert Voltage to Temperature - Using J Type Formula */
			if(Voltage < 0.00) {

        		d0 =  0.0000000E00;
        		d1 =  1.9528268E01;
        		d2 = -1.2286185E00;
        		d3 = -1.0752178E00;
        		d4 = -5.9086933E-01;
        		d5 = -1.7256713E-01;
        		d6 = -2.8131513E-02;
        		d7 = -2.3963370E-03;
        		d8 = -8.3823321E-05;

			}
			else if(Voltage <= 42.919) {

				d0 =  0.000000E00; 
        		d1 =  1.978425E01;
        		d2 = -2.001204E-01;
        		d3 =  1.036969E-02; 
        		d4 = -2.549687E-04;
        		d5 =  3.585153E-06; 
        		d6 = -5.344285E-08;
        		d7 =  5.099890E-10; 
        		d8 =  0.000000E00; 

			}
			else {

        		d0 = -3.11358187E03;
        		d1 =  3.00543684E02;
        		d2 = -9.94773230E00;
        		d3 =  1.70276630E-01;
        		d4 = -1.43033468E-03;
        		d5 =  4.73886084E-06;
        		d6 =  0.00000000E00;
        		d7 =  0.00000000E00;
        		d8 =  0.00000000E00;

			}
	        Temperature = d0
        			+ d1 * (double)(Voltage)
                    + d2 * (double)(pow(Voltage,2))   
                    + d3 * (double)(pow(Voltage,3))   
                    + d4 * (double)(pow(Voltage,4))   
                    + d5 * (double)(pow(Voltage,5))   
                    + d6 * (double)(pow(Voltage,6))   
                    + d7 * (double)(pow(Voltage,7))   
                    + d8 * (double)(pow(Voltage,8));   
			break;
		}		


        printf("ADC B Channel %2d - Raw ADC - 0x%08x  Temp -  %1.1f°C\n",   (int)iChannelCount-7, 
                                                                            (int)ADC_Value,
                                                                            Temperature); 
    }
        

    /* Display - Calibration Values - Can be Monitored for Alarms */

    /* Read Raw Value from ADC A - Zero Raw Calibration */
   	IO8403.Hy8403_Read(ADC_A_ZERO_CAL,(ULONG *)&ADC_Value);
	printf("ADC A - Zero Raw Calibration - 0x%08x\n",(int)ADC_Value); 


    /* Read Raw Value from ADC A - Pos  Raw Calibration */
   	IO8403.Hy8403_Read(ADC_A_POS_CAL,(ULONG *)&ADC_Value);
	printf("ADC A - Pos  Raw Calibration - 0x%08x\n",(int)ADC_Value); 


    /* Read Raw Value from ADC A - Neg  Raw Calibration */
   	IO8403.Hy8403_Read(ADC_A_NEG_CAL,(ULONG *)&ADC_Value);
	printf("ADC A - Neg  Raw Calibration - 0x%08x\n",(int)ADC_Value); 


    /* Read Raw Value from ADC B - Zero Raw Calibration */
   	IO8403.Hy8403_Read(ADC_B_ZERO_CAL,(ULONG *)&ADC_Value);
	printf("ADC B - Zero Raw Calibration - 0x%08x\n",(int)ADC_Value); 


    /* Read Raw Value from ADC B - Pos  Raw Calibration */
   	IO8403.Hy8403_Read(ADC_B_POS_CAL,(ULONG *)&ADC_Value);
	printf("ADC B - Pos  Raw Calibration - 0x%08x\n",(int)ADC_Value); 


    /* Read Raw Value from ADC B - Neg  Raw Calibration */
   	IO8403.Hy8403_Read(ADC_B_NEG_CAL,(ULONG *)&ADC_Value);
	printf("ADC B - Neg  Raw Calibration - 0x%08x\n",(int)ADC_Value); 


//	printf(" 10 sec sleeping...\n");
//	sleep(10);
// 	IO8403.Hy8403_EnableAutoCal(-1);

//	printf(" 10 sec sleeping...\n");
//	sleep(10);
// 	IO8403.Hy8403_EnableAutoCal(-1);

//	printf(" 10 sec sleeping...\n");
//	sleep(10);
// 	IO8403.Hy8403_EnableAutoCal(1);


//	printf(" 50 sec sleeping...\n");
//	sleep(100);



        
#endif
/**************************************/


    } // end for loop

//	status = IO8403.Hy8403_dumpadc();

	printf("8403 test finished. Status = %d \n\n", (int)status);

//-----------C version--------
//DeleteCHy8403(IO1);
//-----------C version--------

	//return 0;
#endif


/**********************************************************/







//====================End of 8403 test example======================

//====================8005 test example=============================
//              Assume 8005 VME card is in slot 3                  =
//==================================================================
#ifdef TEST_8005
 
//-----------C++ version--------
	CHy8005 IO8005;
//-----------C++ version--------

//-----------C version--------
	//CHy8005 *IO1;
	//IO1 = CreateCHy8005();
	//IO1->Hy8005_Configure(...);
	//.....replace every . operator to ->
//-----------C version--------
    //configuration data for DO
    int debrate = 0;
    ULONG readVal;
    ULONG debmask = 0;
    int scanrate = 0;
    USHORT vectornum1 = 0;
    ULONG intmask = 0;
    int pwidth = 0;
    ULONG pmask = 0;
    int inhibit = 0;
	int IOport0 = 0;
	int IOport1 = 1;
	int IOport2 = 2;
	int IOport3 = 3;

#define OUTPUTS 1

#ifdef OUTPUTS
	printf("Initialise all ports as output.....\n");	

    int dir = 1;

	status = IO8005.Hy8005_Configure(vmeslot, IPslot, IOport0, debrate, debmask, 
                scanrate, dir, vectornum1, intmask, pwidth, pmask, inhibit);
	if (status)
	{
		printf("Port 1 Config error!  - return code = %d\n",(int)status);
		return 0;	
	}
	printf("Port 0 Initialisation Status = %d\n", (int)status);	

	status = IO8005.Hy8005_Configure(vmeslot, IPslot, IOport1, debrate, debmask, 
                scanrate, dir, vectornum1, intmask, pwidth, pmask, inhibit);
	if (status)
	{
		printf("Port 1 Config error!  - return code = %d\n",(int)status);
		return 0;	
	}
	printf("Port 1 Initialisation Status = %d\n", (int)status);	

	status = IO8005.Hy8005_Configure(vmeslot, IPslot, IOport2, debrate, debmask, 
                scanrate, dir, vectornum1, intmask, pwidth, pmask, inhibit);
	if (status)
	{
		printf("Port 2 Config error!  - return code = %d\n",(int)status);
		return 0;	
	}
	printf("Port 2 Initialisation Status = %d\n", (int)status);	

	status = IO8005.Hy8005_Configure(vmeslot, IPslot, IOport3, debrate, debmask, 
                scanrate, dir, vectornum1, intmask, pwidth, pmask, inhibit);
	if (status)
	{
		printf("Port 3 Config error!  - return code = %d\n",(int)status);
		return 0;	
	}
	printf("Port 3 Initialisation Status = %d\n", (int)status);	

	printf("-----Perform a series of writes and reads---\n");	
	for (i=0; i<2; i++)
	{
        if(i & 0x01)
    		lvalue = 0x50050005;
        else

    		lvalue = 0xa000a00a;

		printf("Set port 0 to 0x%X\n", (int)lvalue);
		status = IO8005.Hy8005_Write(IOport0, 0xFFFFFFFF, lvalue);
		if (status != 0) 
		{
			printf("Write error! %d\n", (int)status);
			return 0;
		}
		sleep(1);

		printf("Set port 1 to 0x%X\n", (int)lvalue);
		status = IO8005.Hy8005_Write(IOport1, 0xFFFFFFFF, lvalue);
		if (status != 0) 
		{
			printf("Write error! %d\n", (int)status);
			return 0;
		}
		sleep(1);

		printf("Set port 2 to 0x%X\n", (int)lvalue);
		status = IO8005.Hy8005_Write(IOport2, 0xFFFFFFFF, lvalue);
		if (status != 0) 
		{
			printf("Write error! %d\n", (int)status);
			return 0;
		}
		sleep(1);

		printf("Set port 3 to 0x%X\n", (int)lvalue);
		status = IO8005.Hy8005_Write(IOport3, 0xFFFFFFFF, lvalue);
		if (status != 0) 
		{
			printf("Write error! %d\n", (int)status);
			return 0;
		}
		sleep(1);

		printf("Read each value back and check\n");
		for(j=0;j<4;j++)
		{
			status = IO8005.Hy8005_Read(j, &readVal, 0xFFFFFFFF);
			if (status != 0) 
			{
				printf("Read Port%d error! %d\n", j,(int)status);
				return 0;
			}
			else if (readVal != lvalue)
			{
				printf("Read Port%d error! wrote 0x%08X  read 0x%08X\n", j, (int)lvalue,
				(int)readVal);
				return 0;
			}
			else
				printf(" ----> Read port%d value 0x%08X\n", j, (int)readVal);
		}

	}

	printf("Walking 1's on all ports\n");
	for(j=0;j<32;j++)
	{
		status = IO8005.Hy8005_Write(IOport0, 0xFFFFFFFF, 1<<j);
		status = IO8005.Hy8005_Write(IOport1, 0xFFFFFFFF, 1<<j);
		status = IO8005.Hy8005_Write(IOport2, 0xFFFFFFFF, 1<<j);
		status = IO8005.Hy8005_Write(IOport3, 0xFFFFFFFF, 1<<j);
		sleep(1);
    }

	status = IO8005.Hy8005_Write(IOport0, 0xFFFFFFFF, 0);
	status = IO8005.Hy8005_Write(IOport1, 0xFFFFFFFF, 0);
	status = IO8005.Hy8005_Write(IOport2, 0xFFFFFFFF, 0);
	status = IO8005.Hy8005_Write(IOport3, 0xFFFFFFFF, 0);

#else
    int jj;

	printf("Initialise all ports as input.....\n");	

    int dir = 0;

	status = IO8005.Hy8005_Configure(vmeslot, IPslot, IOport0, debrate, debmask, 
                scanrate, dir, vectornum1, intmask, pwidth, pmask, inhibit);
	if (status == C_ERR_NODEVICE)
	{
		printf("Driver error!\n");
		return 0;	
	}
	printf("Port 0 Initialisation Status = %d\n", (int)status);	

	status = IO8005.Hy8005_Configure(vmeslot, IPslot, IOport1, debrate, debmask, 
                scanrate, dir, vectornum1, intmask, pwidth, pmask, inhibit);
	if (status == C_ERR_NODEVICE)
	{
		printf("Driver error!\n");
		return 0;	
	}
	printf("Port 1 Initialisation Status = %d\n", (int)status);	

	status = IO8005.Hy8005_Configure(vmeslot, IPslot, IOport2, debrate, debmask, 
                scanrate, dir, vectornum1, intmask, pwidth, pmask, inhibit);
	if (status == C_ERR_NODEVICE)
	{
		printf("Driver error!\n");
		return 0;	
	}
	printf("Port 2 Initialisation Status = %d\n", (int)status);	

	status = IO8005.Hy8005_Configure(vmeslot, IPslot, IOport3, debrate, debmask, 
                scanrate, dir, vectornum1, intmask, pwidth, pmask, inhibit);
	if (status == C_ERR_NODEVICE)
	{
		printf("Driver error!\n");
		return 0;	
	}
	printf("Port 3 Initialisation Status = %d\n", (int)status);	

	printf("Read each value back and check\n");
    for(jj=0;jj<32;jj++)
    {
    	for(j=0;j<4;j++)
    	{
    		status = IO8005.Hy8005_Read(j, &readVal, 0xFFFFFFFF);
    		if (status != 0) 
    		{
    			printf("Read Port%d error! %d\n", j,(int)status);
    			return 0;
    		}
    		printf(" ----> Read port%d value 0x%08X\n", j, (int)readVal);
    	}
        sleep(1);
        printf("\n");
    }



#endif

	printf("\n");

	printf("8005 test finished. Status = %d \n\n", (int)status);

//-----------C version--------
//DeleteCHy8403(IO1);
//-----------C version--------

//return 0;
#endif
//====================End of 8005 test example======================

//====================8505 test example=============================
//              Assume 8505 IP is in slot C                        =
//==================================================================
#ifdef TEST_8505
 
//-----------C++ version--------
//	CHy85056 IO1;
//-----------C++ version--------

//-----------C version--------
	//CHy85056 *IO1;
	//IO1 = CreateCHy85056();
	//IO1->Hy85056_Configure(...);
	//.....replace every . operator to ->
//-----------C version--------
    //configuration data for DO
    IPslot = 'A';
    int debrate = 0;
    USHORT debmask = 0;
    int scanrate = 0;
    int dir = 0;
    USHORT vectornum1 = 0xAA;
    USHORT intmask = 0xFFFF;
    int pwidth = 0;
    USHORT pmask = 0;
    int clock = 0;
	USHORT ioValue;

	status = IO1.Hy85056_Configure(3, IPslot, debrate, debmask, 
                scanrate, dir, vectornum1, intmask, pwidth, pmask, clock);
	if (status == C_ERR_NODEVICE)
	{
		printf("Driver error!\n");
		return 0;	
	}
	printf("Initialisation Status = %d\n", status);	

	IO1.Hy85056_EnableInterrupt(0);
	IO1.ConnectInterrupt(3, 0, vectornum1, 7, &callback, &int_info);

/*	printf("Set every bit to 1...\n");
	status = IO1.Hy85056_Write(0xFFFF, 0xFFFF);
	if (status < 0) //(status != 0) 
	{
		printf("Write 0xFFFF error! %d\n", status);
		return 0;
	}
*/

//*******this bit is for testing input interrupt*****************
for(i=0; i<100; i++)
{
	status = IO1.Hy85056_Read(&ioValue, 0xFFFF);
	if (status < 0) //(status != 0) 
	{
		printf("Read IO error! %d\n", status);
		return 0;
	}		
	printf("IOs = %x\n", ioValue);
	sleep(1);
}

return 0;
//*******this bit is for testing input interrupt*****************

	sleep(2);

	printf("Clear every bit to 0...\n");
	status = IO1.Hy85056_Write(0xFFFF, 0x00);
	if (status < 0)// (status != 0) 
	{	
		printf("Write 0x0000 error! %d\n", status);
		return 0;
	}

	status = IO1.Hy85056_Read(&ioValue, 0xFFFF);
	if (status < 0) //(status != 0) 
	{
		printf("Read IO error! %d\n", status);
		return 0;
	}		
	printf("IOs = %x\n", ioValue);

	printf("Set bit 0 to bit 15 one by one by using word write function...\n");
	for (i = 0; i<16; i++)
	{
		value = 1 << i;
		status = IO1.Hy85056_Write(value, value);
		sleep(2);
		if (status >= 0) //(status == 0)
		{
			printf("8505 channel %d value %d mask %d\n", i, (int) value, (int) value);
		}else
		{
			printf("Write to 8505 error! %d \n", (int)status);
			return 0;
		}
	}

	printf("Clear bit 0 to bit 15 one by one by using bit write function...\n");
	for (i = 0; i<16; i++)
	{
		value = 1 << i;
		status = IO1.Hy85056_Write_Bit(i, 0);
		sleep(2);
		if (status >= 0)// (status == 0)
		{
			printf("8505 channel %d value %d \n", i, (int) value);
		}else
		{	
			printf("Write to 8505 error! %d \n", (int)status);
			return 0;
		}
	}


	printf("8505 test finished. Status = %d \n\n", (int)status);

//-----------C version--------
	//DeleteCHy85056(IO1);
//-----------C version--------

	//return 0;
#endif
//====================End of 8505 test example======================


//====================8506 test example=============================
//   Assume 8506 IP is in slot A, carrier card in VME slot 3 	   =
//	 The test uses 8506 port 0 as output and port 2	as input	   =
//   hence please wire port 0 pins back to port 2 pins 			   =
//==================================================================
#ifdef TEST_8506
 
//-----------C++ version--------
	CHy85056 IO8506;
//-----------C++ version--------

    //configuration data for DO
    IPslot = 'A';
    int debrate = 0;
    USHORT debmask = 0;
    int scanrate = 0;
    USHORT vectornum1 = 0;
    USHORT intmask = 0;
    int pwidth = 0;
    USHORT pmask = 0;
    int clock = 0;
	int IOport0 = 0;
	int IOport1 = 1;
	int IOport2 = 2;

	printf("Initialise port 0 as output.....");	
    int dir = 1;
	status = IO8506.Hy85056_Configure(vmeslot, IPslot, IOport0, debrate, debmask, 
                scanrate, dir, vectornum1, intmask, pwidth, pmask, clock);
	if (status == C_ERR_NODEVICE)
	{
		printf("Driver error!\n");
		return 0;	
	}
	printf("Success.\n");	
	
	printf("Initialise port 2 as input.....");	
	dir = 0;
	status = IO8506.Hy85056_Configure(vmeslot, IPslot, IOport2, debrate, debmask, 
                scanrate, dir, vectornum1, intmask, pwidth, pmask, clock);
	if (status == C_ERR_NODEVICE)
	{
		printf("Driver error!\n");
		return 0;	
	}
	printf("Success\n\n");	


	printf("-----Write and Read Test: Loop 10 times-----\n");	
	for (i=0; i<10; i++)
	{
        if(i & 0x01)
    		value = 0x5555;
        else
    		value = 0xaaaa;

		printf("Set port 0 to 0x%X\n", value);
		status = IO8506.Hy85056_Write(IOport0, 0xFFFF, value);
		if (status != 0) 
		{
			printf("Write 0xFFFF error! %d\n", status);
			return 0;
		}
		sleep(2);


        status = IO8506.Hy85056_Read(IOport2, &value, 0xFFFF);
		if (status != 0) 
		{
			printf("Write 0xFFFF error! %d\n", status);
			return 0;
		}
		printf(" ----> Read port 2 value 0x%02X\n", value);
	}
	printf("\n");
	return 0;



#endif
//====================End of 8505 / 8506 test example======================


//====================8411 test example=============================
//              Assume 8411 IP is in slot A                        =
//==================================================================
#ifdef TEST_8411

//-----------C++ version--------
	CHy8411 ADC2;
//-----------C++ version--------

//-----------C version--------
	//CHy8411 *ADC2;
	//ADC2 = CreateCHy8411();
	//ADC2->Hy8411_Configure(...);
	//.....replace every . operator to ->
//-----------C version--------

    //configuration data for ADC
    char adcIPslot1 = 'B';
    int adcClockRate1 = 15;

	status = ADC2.Hy8411_Configure(vmeslot, adcIPslot1, adcClockRate1);
	if (status == C_ERR_NODEVICE)
	{
		printf("Driver error!\n");
		return 0;	
	}else
		printf("Configure IP card successfully. Return value = %d \n",(int)status);

	for (i = 0; i<16; i++)
	{
		for(j=0; j<16;j++)
		{	
			status = ADC2.Hy8411_Read(j, &value);
			if (status == 0)
			{
				printf("Channel %d value = 0x%X\n", j, (int) value);
			}else
			{
				printf("Read channel %d error! %d \n", j, (int)status);
				return 0;
			}
		} 
		printf("\n");
		getchar();
		//sleep(3);
	}

	printf("8411 Test finished. Return value = %d \n\n", (int)status);

//-----------C version--------
	//DeleteCHy8411(ADC2);
//-----------C version--------

	//return 0;
#endif
//====================End of 8411 test example======================



//====================8413 test example=============================
//              Assume 8413 IP is in slot A                        =
//==================================================================
#ifdef TEST_8413

//-----------C++ version--------
	CHy8413 ADC2;
//-----------C++ version--------

//-----------C version--------
	//CHy8413 *ADC2;
	//ADC2 = CreateCHy8413();
	//ADC2->Hy8413_Configure(...);
	//.....replace every . operator to ->
//-----------C version--------

    //configuration data for ADC
    char adcIPslot1 = 'A';
    int adcClockRate1 = 100000;

    /*
    long CHy8413::Hy8413_Configure(	int vmeslot, int ipslot, int vectornum,
	                    		int itrState, int aitype, int clocksource,
	                    		int clockrate, int inhibit, int samples,
			                    int spacing,  int trigger)
    */


	status = ADC2.Hy8413_Configure(vmeslot,adcIPslot1,0, 0,0,0, adcClockRate1,0,0, 0,0);
	if (status == C_ERR_NODEVICE)
	{
		printf("Driver error!\n");
		return 0;	
	}
    else
    {
		printf("Configure IP card successfully. Return value = %d \n",(int)status);
    }

	for (i = 0; i<16; i++)
	{
        // Display All Channels Include 16 - OV Ref and 17 - 2.5V Ref 
		for(j=0; j<18;j++)
		{	
			status = ADC2.Hy8413_Read(j, &value);
			if (status == 0)
			{
				printf("Channel %d value = 0x%X\n", j, (int) value);
			}
            else
			{
				printf("Read channel %d error! %d \n", j, (int)status);
				return 0;
			}
		} 
		printf("\n");
		getchar();
		//sleep(3);
	}

	printf("8413 Test finished. Return value = %d \n\n", (int)status);

//-----------C version--------
	//DeleteCHy8413(ADC2);
//-----------C version--------

	//return 0;
#endif
//====================End of 8413 test example======================




//====================8512 test example=============================
//              Assume 8512 IP is in slot D                        =
//==================================================================
#ifdef TEST_8512

//-----------C++ version--------
	CHy8512 SCALER1;
//-----------C++ version--------

//-----------C version--------
	//CHy8512 *SCALER1;
	//SCALER1 = CreateCHy8512();
	//SCALER1->Hy8512_Configure(...);
	//.....replace every . operator to ->
//-----------C version--------

    //configuration data for ADC
    char scalerIPslot1     = 'D';
    USHORT scaler_arm   = 0xFFFF;   // Arm / Enable all Scalers
    USHORT clock_source = 0x0000;   // Clock External
    ULONG scalers[16];


    // Test 1 - Read All Scalers at Once at 1 Hz Clocked from Internal 10MHz Clock
    // --------------------------------------------------------------------------- 
/*
    clock_source = 0xFFFF;          // Clock Internal 10MHz
	status = SCALER1.Hy8512_Configure(vmeslot, scalerIPslot1, scaler_arm, clock_source, 0, 0);

	if (status == C_ERR_NODEVICE)
	{
		printf("Driver error!\n");
		return 0;	
	}
    else
    {
		printf("Configure IP card successfully. Return value = %d \n",(int)status);
    }

    for (i = 0; i < 5; i++)
	{
	    // Read all Scalers Clearing Each Time
        status = SCALER1.Hy8512_Read(scalers, 1);
		if (status == 0)
		{
    		for(j = 0; j < 16;j++)
    		{	
				printf("Channel %d value = %d\n", j, (int)(scalers[j]) );
			}
		} 
        else
    	{
			printf("Read channel %d error! %d \n", j, (int)status);
			return 0;
		}
        // Once a second
        sleep(1);
    }   
*/
    // ----------------- End of Test 1 ----------------- 

    // Test 2 - Read All Scalers at Once at 1 Hz. Scalers 0,1,7,15 Clocked Externally Others from Internal 10MHz Clock
    // --------------------------------------------------------------------------------------------------------------- 
/*
    clock_source = 0x7F7C;          // Scalers 0,1,7,15 Clocked Externally Others from Internal 10MHz Clock
	status = SCALER1.Hy8512_Configure(vmeslot, scalerIPslot1, scaler_arm, clock_source, 0, 0);

	if (status == C_ERR_NODEVICE)
	{
		printf("Driver error!\n");
		return 0;	
	}
    else
    {
		printf("Configure IP card successfully. Return value = %d \n",(int)status);
    }

    for (i = 0; i < 100; i++)
	{
	    // Read all Scalers Clearing Each Time
        status = SCALER1.Hy8512_Read(scalers, 1);
		if (status == 0)
		{
    		for(j = 0; j < 16;j++)
    		{	
				printf("Channel %d value = %d\n", j, (int)(scalers[j]) );
			}
		} 
        else
    	{
			printf("Read channel %d error! %d \n", j, (int)status);
			return 0;
		}
        // Once a second
        sleep(1);
    }   
*/
    // ----------------- End of Test 2 ----------------- 

    // Test 3 - Write a Known Pattern to All Scalers and Read All Back to Confirm OK
    // ----------------------------------------------------------------------------- 
/*
    scaler_arm = 0x0000;          // Dis-arm All Scalers
	status = SCALER1.Hy8512_Configure(vmeslot, scalerIPslot1, scaler_arm, clock_source, 0, 0);

	if (status == C_ERR_NODEVICE)
	{
		printf("Driver error!\n");
		return 0;	
	}
    else
    {
		printf("Configure IP card successfully. Return value = %d \n",(int)status);
    }

    // Write All Scalers (16 Writes)
    for(j = 0; j < 16;j++)
	{	
        status = SCALER1.Hy8512_Write(j, ((ULONG)(j+1)*65536 + j));
	}

    // Read all Scalers Clearing Each Time
    status = SCALER1.Hy8512_Read(scalers, 0);
    if (status == 0)
	{
   		for(j = 0; j < 16;j++)
   		{	
			printf("Channel %d\tvalue = %d\tHigh = 0x%04X\tLow = 0x%04X\n", 
                    j, 
                    (int)(scalers[j]), 
                    (int)(scalers[j] >> 16),
                    (int)(scalers[j] &  0x0000FFFF) );
		}
	} 
    else
   	{
		printf("Read channel %d error! %d \n", j, (int)status);
		return 0;
	}

    // ----------------- End of Test 3 ----------------- 
*/
    // Test 4 - Write a 1,000,000 to first and last (0,15) Scalers, then Clock externally and Read Back to Confirm OK
    // -------------------------------------------------------------------------------------------------------------- 

/*    scaler_arm = 0xFFFF;          // Arm All Scalers
    clock_source = 0x0000;        // Clock All Externally  
	status = SCALER1.Hy8512_Configure(vmeslot, scalerIPslot1, scaler_arm, clock_source, 0, 0);

	if (status == C_ERR_NODEVICE)
	{
		printf("Driver error!\n");
		return 0;	
	}
    else
    {
		printf("Configure IP card successfully. Return value = %d \n",(int)status);
    }


	// Write Scalers 0 and 15
    status = SCALER1.Hy8512_Write(0, 1000000);
    status = SCALER1.Hy8512_Write(15, 1000000);

	for(j = 0; j < 5;j++)
	{	
        // Read First Scaler (No. 00)
        status = SCALER1.Hy8512_Read(0, &scalers[0], 0);
		if (status == 0)
		{
            printf("Scaler 00 = %d\n", (int)(scalers[0]) );
		} 
        else
    	{
			printf("Read channel 00 error! %d \n",(int)status);
			return 0;
		}

        // Read Last Scaler (No. 15)
        status = SCALER1.Hy8512_Read(15, &scalers[0], 0);
		if (status == 0)
		{
            printf("Scaler 15 = %d\n", (int)(scalers[0]) );
		} 
        else
    	{
			printf("Read channel 15 error! %d \n",(int)status);
			return 0;
		}

        // Once a second
        sleep(1);
	}

    // ----------------- End of Test 4 ----------------- 

*/

    scaler_arm = 0xFFFF;          // Arm All Scalers
    clock_source = 0x0000;        // Clock All Externally  
	status = SCALER1.Hy8512_Configure(vmeslot, scalerIPslot1, scaler_arm, clock_source, 0, 0);

	if (status == C_ERR_NODEVICE)
	{
		printf("Driver error!\n");
		return 0;	
	}
    else
    {
		printf("Configure IP card successfully. Return value = %d \n",(int)status);
    }

    USHORT myRegVal;
    USHORT myReg2Val;
	status = SCALER1.Hy8512_Read_Reg(2, &myRegVal);
	if (status == 0)
	{
		printf("Reg 2 = %x\n",myRegVal);
	}
	else
	{
		printf("Error! %x\n",(int)status);
		return 0;	
	}
	status = SCALER1.Hy8512_Read_Reg(0, &myRegVal);
	printf("Reg 0 = %x  %d\n",(int)myRegVal,(int)status);
	status = SCALER1.Hy8512_Read_Reg(2, &myReg2Val);
	printf("Reg 2 = %x  %d\n",(int)myReg2Val,(int)status);
	myRegVal=0xab;
	status = SCALER1.Hy8512_Write_Reg(2, myRegVal);
	status = SCALER1.Hy8512_Read_Reg(4, &myRegVal);
	printf("Reg 4 = %x  %d\n",(int)myRegVal,(int)status);
	status = SCALER1.Hy8512_Read_Reg(6, &myRegVal);
	printf("Reg 6 = %x  %d\n",(int)myRegVal,(int)status);
	status = SCALER1.Hy8512_Read_Reg(8, &myRegVal);
	printf("Reg 8 = %x  %d\n",(int)myRegVal,(int)status);
	status = SCALER1.Hy8512_Read_Reg(1, &myRegVal);
	printf("Reg 1 = %x  %d\n",(int)myRegVal,(int)status);

	status = SCALER1.Hy8512_Read_Reg(2, &myRegVal);
	printf("Reg 2 = %x  %d\n",(int)myRegVal,(int)status);
	status = SCALER1.Hy8512_Write_Reg(2, myReg2Val);
	status = SCALER1.Hy8512_Read_Reg(2, &myRegVal);
	printf("Reg 2 = %x  %d\n",(int)myRegVal,(int)status);
	printf("\n");

	myRegVal=0x0f;
	status = SCALER1.Hy8512_Write_Reg(2, myRegVal);
	status = SCALER1.Hy8512_Read_Reg(2, &myRegVal);
	printf("Reg 2 = %x  %d\n",(int)myRegVal,(int)status);
	myRegVal=0x60;
	status = SCALER1.Hy8512_ArmEnable(myRegVal);
	printf("Arm Enable 0x60  status = %d\n",(int)status);
	status = SCALER1.Hy8512_Read_Reg(2, &myRegVal);
	printf("Reg 2 = %x  %d\n",(int)myRegVal,(int)status);
	myRegVal=0x06;
	status = SCALER1.Hy8512_ArmClear(myRegVal);
	printf("Arm Clear 0x06  status = %d\n",(int)status);
	status = SCALER1.Hy8512_Read_Reg(2, &myRegVal);
	printf("Reg 2 = %x  %d\n",(int)myRegVal,(int)status);
	printf("\n");

	myRegVal=0x6f;
	status = SCALER1.Hy8512_Write_Reg(4, myRegVal);
	status = SCALER1.Hy8512_Read_Reg(4, &myRegVal);
	printf("Reg 4 = %x  %d\n",(int)myRegVal,(int)status);
	myRegVal=0x06;
	status = SCALER1.Hy8512_OverflowClear(myRegVal);
	printf("Overflow Clear 0x06  status = %d\n",(int)status);
	status = SCALER1.Hy8512_Read_Reg(4, &myRegVal);
	printf("Reg 4 = %x  %d\n",(int)myRegVal,(int)status);
	printf("\n");


// Zero all the channels
// .. then update each channel in turn checking the value of the all the rest
// .. this is to ensure that no overwriting of adjacemt or other channels is taking place


	// Read all Scalers and zero after reading
    status = SCALER1.Hy8512_Read(scalers, 1);
	if (status == 0)
	{
    	for(j = 0; j < 16;j++)
    	{	
			printf("Channel %d value = %d\n", j, (int)(scalers[j]) );
		}
    }
	else
    {
		printf("Read channel %d error! %d \n", j, (int)status);
		return 0;
	}
	printf("\n");

    status = SCALER1.Hy8512_Read(scalers, 1);
	if (status == 0)
	if (status == 0)
	{
    	for(j = 0; j < 16;j++)
    	{	
			printf("Channel %d value = %d\n", j, (int)(scalers[j]) );
		}
    }
	else
    {
		printf("Read channel %d error! %x \n", j, (int)status);
		return 0;
	}
	printf("\n");

// now set each scaler in turn and read the rest


    for (i = 0; i < 16; i++)
	{
        // Write to Scaler (No. i)
		scalers[i]=0x55555555;
        status = SCALER1.Hy8512_Write(i, scalers[i]);
		if (status != 0)
		{
			printf("Write channel %d error! %d \n", i, (int)status);
			return 0;
		} 

	    status = SCALER1.Hy8512_Read(scalers, 1);
		if (status == 0)
		{
	    	for(j = 0; j < 16;j++)
	    	{	
				printf("Channel %d value = %x\n", j, (int)(scalers[j]) );
			}
		}
        else
    	{
			printf("Read error! %d \n",(int)status);
			return 0;
		}
        
		printf("\n");
//		getchar();
		
        // Once a second
        //sleep(1);
	}

	printf("8512 Test finished. Return value = %d \n\n", (int)status);

//-----------C version--------
	//DeleteCHy8512(SCALER1);
//-----------C version--------

	//return 0;
#endif
//====================End of 8512 test example======================





//====================85156 test example=============================
//              Assume 85156 IP is in slot E                        =
//===================================================================
#ifdef TEST_85156

	//assume 8515 IP is in slot E

//-----------C++ version--------
	CHy85156 COM1;
	CHy85156 COM2;
//-----------C++ version--------

//-----------C version--------
	//CHy85156 *COM1;	
	//COM1 = CreateCHy86156();
	//COM1->Hy85156_Configure(...);
	//.....replace every . operator to ->
//-----------C version--------

    //configuration data for DAC
    IPslot = 'B';
    char vector = 0;
    int delay = 0;
    int halfduplex = 0;
    int delay485 = 0;
	vmeslot = 3;
	UCHAR my_word[] = "This is a test from concurrent.\n\a";
	UCHAR word[64];
	
    int port = 0;
    ULONG baud = 19200;
    char parity = 'N';
    int stopbits = 1;
    int databits = 8;
    char flowctl = 0;
	long number;

	//generic settings for the 8515
	status = COM1.Hy85156_Configure(vmeslot, IPslot, vector, delay, halfduplex, delay485);
	if (status == C_ERR_NODEVICE)
	{
		printf("Configure error! %d\n", (int) status);
		return 0;	
	}else
		printf("Configure IP card successfully. Return value = %d \n",(int)status);

	//setup com port 5 which is actually number 6 on the terminal (start from 0)
	port = 0;
	status = COM1.Hy85156_ComsSetup(port, baud, parity, stopbits, databits, flowctl);
	if (status != 0)
	{
		printf("COMSetup error! %d\n", (int) errno);
		return 0;	
	}else
		printf("Setup COM port successfully. Return value = %d \n",(int)status);

	//send a string out
	number = strlen((char*)my_word);
	status = COM1.Hy85156_Write(port, my_word, (int) number);
	if (status > 0)
	{
		printf("Sent %d characters out port %d.\n", (int) status, port);
	}else
	{
		printf("Sending characters error! %d \n", (int)status);
		return 0;
	}

	//polling the receive buffer for text until receiving a carriage return -- not very good example though
	printf("Received string (hit return to end): \n");
	usleep(500000);
	number = COM1.Hy85156_Read(port, word);
	if(number > 0)
	{
		word[number] = 0;
		printf("Received ->%s\n", word);
	}else
		printf("Errors%d\n", errno);

/*	for (i=0; i<64; i++) word[i] = 0;
    do 
	{
		do
		{
			number = COM1.Hy85156_Read(port, word);
            if (number < 0) 
            {
                printf("Error %d", number);
            }
		}while(number<=0);
	
		word[number] = 0;							//terminate the string
//        printf("Number=%d, word %c\n", number, word);
        printf("%s\n", word);
	}while(word[0]!='\n');
*/
	printf("\n8515 Test finished.\n");
//-----------C version--------
	//DeleteCHy85156(COM1);
//-----------C version--------
	return 0;
#endif
//=================End of 85156 test example========================




//====================8601 test example=============================
//              Assume 8601 IP is in slot D                        =
//==================================================================
#ifdef TEST_8601

	unsigned long position;
	long steps;

//-----------C++ version--------
	CHy8601 motor1;
//-----------C++ version--------

//-----------C version--------
	//CHy8601 *motor1;
	//motor1 = CreateCHy8601();
	//motor1->Hy8601_Configure(...);
	//.....replace every . operator to ->
//-----------C version--------

    //configuration data for DAC
    IPslot = 'D';

    int axis = 0;
    int startspeed = 100;
    int maxspeed = 400;
    int ramp = 64;

	status = motor1.Hy8601_Configure(vmeslot, IPslot);
	if (status == C_ERR_NODEVICE)
	{
		printf("Driver error!\n");
		return 0;	
	}else
		printf("Configure IP card successfully. Return value = %d. \n",(int)status);

	status = motor1.Hy8601_MotorSetup(axis, startspeed, maxspeed, ramp);
	if (status == C_ERR_NODEVICE)
	{
		printf("Setup motor axis 0 error!\n");
		return 0;	
	}else
		printf("Motor axis 0 setup OK. Return value = %d. \n",(int)status);

	steps = -200;
	printf("Move negative steps %d ....", (int) steps);
	status = motor1.Hy8601_Move(axis, steps);
	sleep(4);

	status = motor1.Hy8601_ReadEncoder(axis, &position);
	if (status == 0)
		printf("Absolute postion = %d \n", (int) position);

	steps = 200;
	printf("Move positive steps %d ....", (int) steps);
	status = motor1.Hy8601_Move(axis, steps);
	sleep(4);

	status = motor1.Hy8601_ReadEncoder(axis, &position);
	if (status == 0)
		printf("Absolute postion = %d \n", (int) position);

	sleep(1);	

	status = motor1.Hy8601_Status(axis, 0xFFFF, &value);
	if (status == 0)
		printf("Current CSR status = 0x%x \n", value);

	printf("8601 Test finished. Return value = %d \n\n", (int)status);

//-----------C version--------
	//DeleteCHy8601(motor1);
//-----------C version--------

	//return 0;
#endif
//====================End of 8601 test example======================
	

}
