/********************************************************************************/
/*    H   H Y   Y TTTTT EEEEE  CCC       HYTEC ELECTRONICS LTD                  */
/*    H   H  Y Y    T   E     C          5 Cradock Road,                        */
/*    HHHHH   Y     T   EEE   C          Reading, Berks.    Tel: 0118 9757770	*/
/*    H   H   Y     T   E     C          RG2 0JT            Fax: 0118 9757566	*/
/*    H   H   Y     T   EEEEE  CCC       Web: www.hytec-electronics.co.uk       */
/********************************************************************************/
/*                                                                              */
/*			Linux PROJECT	--- Hytec Sky Scanner Testing Program				*/
/*                                                                              */
/********************************************************************************/
/*  Source file name      :-      TipTest.c										*/
/*  Definition header file:		  HYTECIPAPI.h									*/
/*								  Hy8402.h										*/
/*								  Hy85056.h										*/
/*								  Hy85156.h										*/
/*								  HyTip120.h									*/
/*  Platform			  :		  Linux, Windows XP, Windows 2000               */
/*  Initial creation date :-      01-Feb-2010                                   */
/*                                                                              */
/*  Original Developers   :-      Jim S Chen                                    */
/********************************************************************************/
/*                                                                              */
/*  Description :-                                                              */
/*                                                                              */
/*  This is for testing Hytec IP card API library for Sky Scanner job. It 		*/
/*	inlcudes testing IP cards such as 8402, 8505/8506, 8515/8516 and Tip120		*/
/*                                                                              */
/*  Uncomment each macro define on the top that associates to the particular 	*/
/*	IP card to run the test.													*/
/*																				*/
/*  (C)2010 Hytec Electronics Ltd.												*/
/*                                                                              */
/********************************************************************************/
/*                                                                              */
/*  Modification record :-                                                      */
/*                                                                              */
/*  Date modified |     Person    |   Description of modification               */
/* ---------------|---------------|-------------------------------------------- */
/*    17/01/10    |	  Jim S Chen  |	  Initial copy								*/
/*                |               |                                             */
/*                                                                              */
/********************************************************************************/

//#define TEST_ID
#define TEST_TIP120

#include "../include/HytecIPAPI.h"
#include "../include/HyTip120.h"

#include <pthread.h>
#include <math.h>

// global because its function need to be called by interrupt callback function which is run in another thread
CHyTip120 tip120;
EN_VME_INT_INFO int_info;					//interrupt structure that holds the interrupt information
void printStatus(int status);

//CHy85056 IO1;

// interrupt callback function
void callback(void *drv)
{
	int i;
	int status;
	int intreason;
	int tipInterruptedFlag = 0;

//	printf("\n************Interrupted**************\n");

	EN_VME_INT_INFO *d = (EN_VME_INT_INFO *)drv;
//	printf("number of interrupt=%d\n", d->numOfInts);
	if (d->numOfInts > 0)
	{
		for(i=0; i<(int)d->numOfInts; i++)
		{
			switch(d->vectors[i])
			{
				case 0x88:															//the interrupt is from tip120
					status = tip120.HyTip120_INTReason(0, &intreason);				//get interrupt reason 
					if (status !=0)
					{
						printf("Get interrupt reason error. %x \n",status);
					}else
					{
//						printf("Interrupt reason. %x \n",intreason);
						tipInterruptedFlag = 1;											//rise the flag to tell to clear interrupt at the end of this ISR

						//input 1 (negative limit switch hit) interrupt
						if (intreason & INT_INPUT1)				
						{
							printf("Negative limit is reached.\n");
							status = tip120.HyTip120_Stop(0, STOP_ABRUPTLY);		//stop motor immediately
							if (status !=0) printf("Stop motor error. %x \n",status);
						}

						//input 2 (home limit switch hit) interrupt
						if (intreason & INT_INPUT2)				
						{
							printf("Home limit switch hit.\n");
							status = tip120.HyTip120_Stop(0, STOP_ABRUPTLY);		//stop motor immediately
							if (status !=0) printf("Stop motor error. %x \n",status);
							status = tip120.HyTip120_DefineHome(0);					//define home
							if (status !=0) printf("Define home error. %x \n",status);
						}

						//input 3 (positive limit switch hit) interrupt
						if (intreason & INT_INPUT3)				
						{
							printf("Positive limit is reached.\n");
							status = tip120.HyTip120_Stop(0, STOP_ABRUPTLY);		//stop motor immediately
							if (status !=0) printf("Stop motor error. %x \n",status);
						}

						//excessive error interrupt
						if (intreason & INT_POSITIONERROR)				
						{
							printf("Excessive position error and motor stopped.\n");
						}

						//index interrupt
						if (intreason & INT_INDEXPULSE)				
						{
							printf("Index passed.\n");
						}

						//trajectory completed interrupt
						if (intreason & INT_TRAJECTORYCOMPLETE)	
						{
//							printf("Motion finished.\n");
						}
					}
					break;
				case 0xAA:
					break;
			}
		}
		if(tipInterruptedFlag)
		{
			status = tip120.HyTip120_ResetInterrupt(0, intreason);
			if (status !=0) printf("Reset interrupt error. %x \n",status);
		}
	}	
}


int main(void)
{
	int i=0;
	int vmeslot=3;        

	printf("\nTesting Program starts...... \n\n");

//====================TIP120 test example===========================
//              Assume TIP120 IP is in slot D                     ||
//==================================================================

#ifdef TEST_TIP120
 
	int status=0;
	char IPslot='D';
	int intLevel = 6;
	int tip120vector = 0x88;		//vector for tip120 IP card
	int channel = 0;
    int degree;
	ULONG fbdata;
	int INTMask = INT_INDEXPULSE | INT_TRAJECTORYCOMPLETE | 
				  INT_INPUT1 | INT_INPUT2 | INT_INPUT3 | INT_POSITIONERROR;

	printf("Configure TIP120 INT level=%d, Vector=%x\n", intLevel, tip120vector);
	status = tip120.HyTip120_Configure(vmeslot, IPslot, intLevel, tip120vector);
	if (status !=0)
	{
		printf("Configuration error! -- %d\n", status);
		return 0;	
	}

	printf("Set up motor channel %d for position control.\n", channel);
	printf("Generate index, trajectory completion, input 1~3 and position error interrupts.\n");
	printf("Set 70 degree as the position error threshold. If error, stop the motor.\n");
	printf("Use default PID parameter settings.\n");

	status = tip120.HyTip120_MotorSetup(channel, 0, INTMask, 0, 70, 1, &callback, &int_info);
	if (status !=0)
	{
		printf("Setup error! -- %d\n", status);
		return 0;	
	}

	// enable interrupt
	printf("Enable interrupt.\n\n");
	status = tip120.HyTip120_EnableInterrupt(channel);

	// move to 60 degree
	degree =40;
	printf("Command motor to move t0 60 degree by using 60rpm and 200 acceleration......\n\n");
	
	sleep(3);

	status = tip120.HyTip120_Move(channel, 60, 200, degree);
	if (status !=0)
	{
		printf("Move error! -- %d\n", status);
		return 0;	
	}
	
	//display status and so on for 5 times
	for(i=0;i<5;i++)
	{
		printf("i=%d, *****Status*****\n", i);
		status = tip120.HyTip120_ReadData(channel, &fbdata, DATA_STATUS);
		if (status !=0)
		{
			printf("Read status error. %x \n",status);
			break;	
		}
		printStatus((int)fbdata);
		printf("****************\n\n");
	
		printf("Desired position......");
		status = tip120.HyTip120_ReadData(channel, &fbdata, DATA_DESIRED_POSITION);
		if (status !=0)
		{
			printf("Read desired position error. %d \n",status);
			break;	
		}
		printf("%d\n",(int)fbdata);
		printf("Real position......");
		status = tip120.HyTip120_ReadData(channel, &fbdata, DATA_REAL_POSITION);
		if (status !=0)
		{  
			printf("Read real position error. %x \n",status);
			break;	
		}
		printf("%d\n",(int)fbdata);
		printf("Desired velocity......");
		status = tip120.HyTip120_ReadData(channel, &fbdata, DATA_DESIRED_VELOCITY);
		if (status !=0)
		{
			printf("Read desired velocity error. %d \n",status);
			break;	
		}
		printf("%d\n",(int)fbdata);
		
		printf("Real velocity......");
		status = tip120.HyTip120_ReadData(channel, &fbdata, DATA_REAL_VELOCITY);
		if (status !=0)
		{  
			printf("Read real velocity error. %x \n",status);
			break;	
		}
		printf("%d\n\n",(int)fbdata);

		usleep(1000000);							//wait 1 second
	}

	printf("TIP120 Test finished. \n\n");

#endif
//==================End of Tip120 test example=========================



//===============Read Identities from IOC Box example===============
#ifdef TEST_ID

	USHORT id_buffer[14];
	USHORT carrier;
	USHORT CSR;
	int j=0;

    //ioc.IOC9010SetCarrierFanSpeed(1,1);
	status = tip120.IOC9010GetCarrier(vmeslot, &carrier);

    printf("Carrier Card is %04X\n",carrier);

    if (carrier == 0x8002)
    {
        printf("Detect IP card installed.....\n\n");
        for (j=0;j<4;j++)
        {
            status = tip120.IOC9010GetConfig(vmeslot, j, 14, id_buffer);
            if (status == C_ERR_NODEVICE)
            {
                printf("Device open failed!");
                break;
            }
            else
            {
                printf("Slot %c IP identity: ", (char) (j+0x41));
                for (i=0;i<14;i++)
    				printf("%X ",id_buffer[i]);
                printf("\n");
            }
	   }
    }
    else
    {
        //printf("This Carrier Has NO IP Cards !\n"); 

        printf("Detect IP card installed.....\n\n");
        for (j=0;j<6;j++)
        {
            status = tip120.IOC9010GetConfig(vmeslot, j, 14, id_buffer);
            if (status == C_ERR_NODEVICE)
            {
                printf("Device open failed!");
                break;
            }
            else
            {
                printf("Slot %c IP identity: ", (char) (j+0x41));
                for (i=0;i<14;i++)
    				printf("%X ",id_buffer[i]);
                printf("\n");
            }
	   }
    }	
	printf("End of program.\n\n");

#endif
//===============Read Identities from IOC Box example===============

	return 0;
}

void printStatus(int status)
{
	if(status & STATUS_HOST_INTERRUPT) printf("Host interrupted.\n");
	if(status & STATUS_ACCELERATION_LOADED) printf("Acceleration loaded.\n");
	if(status & STATUS_UDF_EXECUTED) printf("UDF command executed.\n");
	if(status & STATUS_FORWARD_DIRECTION) 
		printf("Forward direction.\n");
	else
		printf("Reverse direction.\n");
	if(status & STATUS_VELOCITY_MODE) 
		printf("Velocity mode.\n");
	else
		printf("Position mode.\n");
	if(status & STATUS_ON_TARGET) printf("Position on target.\n");
	if(status & STATUS_OFF_ONERROR) printf("Stop motor if position error.\n");
	if(status & STATUS_8BIT_OUTPUT) 
		printf("8 bit output.\n");
	else
		printf("12 bit output.\n");
	if(status & STATUS_MOTOR_OFF) printf("Motor is off.\n");
	if(status & STATUS_BREAKPOINT) printf("Breakpoint interrupted.\n");
	if(status & STATUS_POSITIONERROR) printf("Position error interrupted.\n");
	if(status & STATUS_WRAPAROUND) printf("Wrap around interrupted.\n");
	if(status & STATUS_INDEXPULSE) printf("Index interrupted.\n");
	if(status & STATUS_TRAJECTORYCOMPLETE) printf("Trajectory completion interrupted.\n");
	if(status & STATUS_COMMANDERROR) printf("Command error interrupted.\n");
}
