/********************************************************************************/
/*    H   H Y   Y TTTTT EEEEE  CCC       HYTEC ELECTRONICS LTD                  */
/*    H   H  Y Y    T   E     C          5 Cradock Road,                        */
/*    HHHHH   Y     T   EEE   C          Reading, Berks.    Tel: 0118 9757770	*/
/*    H   H   Y     T   E     C          RG2 0JT            Fax: 0118 9757566	*/
/*    H   H   Y     T   EEEEE  CCC       Web: www.hytec-electronics.co.uk       */
/********************************************************************************/
/*                                                                              */
/*			IOC9010 PROJECT	--- Hytec IP Card API Library Testing Program		*/
/*                                                                              */
/********************************************************************************/
/*  Source file name      :-      demo_use.c									*/
/*  Definition header file:		  HYTECIPAPI.h									*/
/*								  Hy8402.h										*/
/*								  Hy8413.h										*/
/*  Platform			  :		  Linux, Windows XP, Windows 2000               */
/*  Initial creation date :-      17-FEB-2010                                   */
/*                                                                              */
/*  Original Developers   :-      Jim S Chen                                    */
/********************************************************************************/
/*                                                                              */
/*  Description :-                                                              */
/*                                                                              */
/*  This is for testing Hytec device driver and API function library. This is 	*/
/*	particularly designed for testing 8402, 8413 IP modules.					*/
/*                                                                              */
/*  Uncomment each macro define on the top that associates to the particular 	*/
/*	IP card to run the test.													*/
/*																				*/
/*  (C)2010 Hytec Electronics Ltd.												*/
/*                                                                              */
/********************************************************************************/
/*                                                                              */
/*  Modification record :-                                                      */
/*                                                                              */
/*  Date modified |     Person    |   Description of modification               */
/* ---------------|---------------|-------------------------------------------- */
/*    22/02/10    |	  Jim S Chen  |	  Initial copy								*/
/*                |               |                                             */
/*                                                                              */
/********************************************************************************/

//#define TEST_8402  
#define TEST_8413

#include "../include/HytecIPAPI.h"
#include "../include/Hy8005.h"  
#include "../include/Hy8402.h"
#include "../include/Hy8403.h"
#include "../include/Hy8404.h"
#include "../include/Hy8408.h"
#include "../include/Hy8411.h"
#include "../include/Hy8413.h"
#include "../include/Hy85056.h"
#include "../include/Hy8512.h"
#include "../include/Hy85156.h"
#include "../include/Hy8601.h"

#include <pthread.h>
#include <math.h>
#include <sys/time.h>

int main(void)  
{
	int i, j;
	USHORT value;	
	ULONG lvalue;	
	long status;
	int vmeslot = 3;        //ignored by IOC9010
	char IPslot;

	i = 0;					//to make the compiler happy
	j = 0;
	IPslot = 0;
	value = 0;
	lvalue = 0;
	status = 0;

	printf("\nTesting Program starts...... \n\n");

//====================8402 test example=============================
//              Assume 8402 IP is in slot B                        =
//==================================================================
#ifdef TEST_8402
	unsigned short int readback;
 
//-----------C++ version--------
	CHy8402 DAC2;
//-----------C++ version-------- 

//-----------C version--------
	//CHy8402 *DAC1;
	//DAC1 = CreateCHy8402();
	//DAC1->Hy8402_Configure(...);
	//.....replace every . operator to ->
//-----------C version--------
   
    //configuration data for DAC
    char dacIPslot1 = 'A';
    int vectornum2 = 0;
    int doram1 = 0;
    int clockSource1 = 0;
    int dacClockRate1 = 13;
    int inhibit1 = 0;

	status = DAC2.Hy8402_Configure(vmeslot, dacIPslot1, vectornum2, 
                doram1, clockSource1, dacClockRate1, inhibit1);
	if (status == C_ERR_NODEVICE)
	{
		printf("Driver error!\n");
		return 0;	
	}else
		printf("Configure IP card on slot %c successfully. Return value = %d \n", dacIPslot1, (int)status);

	//loop 10 times to update ADC channel 0 from 0 to 65530 with interval of 6553
	for (i = 0; i<11; i++)
	{
		status = DAC2.Hy8402_Write(0, i*6553);
		sleep(2);
		status = DAC2.IOC9010IPRead(0, 1, 0x10, &readback);
		if (status == 0)
		{
			printf("Output %d to DAC channel %d. Read back = %d \n", i*6553, 0, readback);
		}else

		{
			printf("Write error! %d \n", (int)status);
			return 0;
		} 
	}

	printf("8402 Test finished. Return value = %d \n\n", (int)status);

//-----------C version--------
	//DeleteCHy8402(DAC2);
//-----------C version--------

	//return 0;
#endif
//==================End of 8402 test example=========================



//====================8413 test example=============================
//              Assume 8413 IP is in slot A                        =
//==================================================================
#ifdef TEST_8413

//-----------C++ version--------
	CHy8413 ADC2;
//-----------C++ version--------

//-----------C version--------
	//CHy8413 *ADC2;
	//ADC2 = CreateCHy8413();
	//ADC2->Hy8413_Configure(...);
	//.....replace every . operator to ->
//-----------C version--------
  
    //configuration data for ADC
    char adcIPslot1 = 'B';
    int adcClockRate1 = 100000;

	status = ADC2.Hy8413_Configure(vmeslot,adcIPslot1,0, 0,0,0, adcClockRate1,0,0, 0,0);
	if (status == C_ERR_NODEVICE)
	{
		printf("Driver error!\n");
		return 0;	
	}
    else
    {
		printf("Configure IP card successfully. Return value = %d \n",(int)status);
    }

    j = 0;
    timespec ts1, ts2;

    clock_gettime(CLOCK_REALTIME, &ts1);            //save time tick before acquisition

	for (i = 0; i<50000; i++)
	{
        // Read all 16 ADC channels 
		for(j=0; j<16;j++)
    	{	
			status = ADC2.Hy8413_Read(j, &value);
			if (status != 0)
			{
				printf("Read channel %d error! %d \n", j, (int)status);
				return 0;
			}
		} 
	}

    clock_gettime(CLOCK_REALTIME, &ts2);            //save time tick after acquisition
    long int sdiff = ts2.tv_sec - ts1.tv_sec;
    long int ndiff = ts2.tv_nsec - ts1.tv_nsec;
    double diff = ((double)sdiff * 1000000000 + (double)ndiff) / 10;    //calculate the difference
    printf("Time taken for 5000 samples: %f ms\n", diff / 1000000);

	printf("8413 Test finished. Return value = %d \n\n", (int)status);

//-----------C version--------
	//DeleteCHy8413(ADC2);
//-----------C version--------

#endif
//====================End of 8413 test example======================

    return 0;
}
