/********************************************************************************/
/*    H   H Y   Y TTTTT EEEEE  CCC       HYTEC ELECTRONICS LTD                  */
/*    H   H  Y Y    T   E     C          5 Cradock Road,                        */
/*    HHHHH   Y     T   EEE   C          Reading, Berks.    Tel: 0118 9757770   */
/*    H   H   Y     T   E     C          RG2 0JT            Fax: 0118 9757566   */
/*    H   H   Y     T   EEEEE  CCC       Web: www.hytec-electronics.co.uk       */
/********************************************************************************/
/*                                                                              */
/* linux-driver.h - Header for Hytec Linux driver for IOC 9010.                 */
/*                                                                              */
/* (C)2006 Hytec Electronics Ltd.                                               */
/*                                                                              */
/********************************************************************************/
/*                                                                              */
/*  Revision history: (comment and initial revisions)                           */
/*                                                                              */
/*  vers.       revised         modified by                                     */
/*  -----       -----------     -------------------------                       */
/*  See Above.  24-FEB-2006		D. Nineham, Hytec Electronics Ltd.				*/
/*                              Initial version.                                */
/*                                                                              */
/********************************************************************************/


/*******************************************************************************
*
* COMPILATION OPTIONS
*
*   __DEBUG__
*       Define this if you want diagnostics output from the driver as it runs.
*       While very useful for debugging (hence the name), this takes up a lot of
*       CPU when running live.
*
*   __DIRECT_IO__
*       Define this if you want to include the direct I/O routines in the
*       driver. These routines access the FIFO directly and are no longer for
*       general use.
*
*******************************************************************************/

#include <linux/ioctl.h>

/*---- definitions -------------------------------------------------*/


/* Register Offsets from Base */
#define REG_CSR					0x0000
#define REG_SWITCH				0x0002
#define REG_LCD_CONTROL			0x0004
#define REG_LCD_DATA			0x0006
#define REG_IP_INT				0x0008
#define REG_IP_ERROR_INT		0x000A
#define REG_IP_INT_MASK			0x000C
#define REG_IP_ERROR_MASK		0x000E
#define REG_IP_CLOCK			0x0010

#define REG_FANS_1_2			0x0012
#define REG_FANS_3_4			0x0014
#define REG_FANS_5_6			0x0016
#define REG_FANS_CONTROL		0x0018
#define REG_TEMPERATURES		0x001A
#define REG_CONFIG_2			0x001C


/* CSR Register Bit Map */
#define CSR_PMC_PRESENT			0x0001
#define CSR_TIMEOUT_INT_CLEAR	0x0002
#define CSR_SPARE_2				0x0004
#define CSR_FAN_1_OK			0x0008
#define CSR_FAN_2_OK			0x0010
#define CSR_FAN_3_OK			0x0020
#define CSR_FAN_4_OK			0x0040
#define CSR_FAN_5_OK			0x0080
#define CSR_INPUT_1				0x0100
#define CSR_INPUT_2				0x0200
#define CSR_INPUT_3				0x0400
#define CSR_SPARE_3				0x0800
#define CSR_SPARE_4				0x1000
#define CSR_SPARE_5				0x2000
#define CSR_SPARE_6				0x4000
#define CSR_SPARE_7				0x8000


/* SWITCH Register Bit Map */
#define SWITCH_UP				0x0001
#define SWITCH_OK				0x0002
#define SWITCH_DOWN				0x0004
#define SWITCH_RESET			0x0008
#define SWITCH_SPARE1			0x0010
#define SWITCH_SPARE2			0x0020
#define SWITCH_SPARE3			0x0040
#define SWITCH_SPARE4			0x0080
#define SWITCH_SW0				0x0100
#define SWITCH_SW1				0x0200
#define SWITCH_SW2				0x0400
#define SWITCH_SW3				0x0800
#define SWITCH_SW4				0x1000
#define SWITCH_SW5				0x2000
#define SWITCH_SW6				0x4000
#define SWITCH_SW7				0x8000


/* LCD Control Register Bit Map */
#define LCD_CONTROL_0			0x0001
#define LCD_CONTROL_1			0x0002
#define LCD_CONTROL_2			0x0004
#define LCD_CONTROL_3			0x0008
#define LCD_CONTROL_4			0x0010
#define LCD_CONTROL_5			0x0020
#define LCD_CONTROL_6			0x0040
#define LCD_CONTROL_7			0x0080
#define LCD_CONTROL_8			0x0100
#define LCD_CONTROL_9			0x0200
#define LCD_CONTROL_10			0x0400
#define LCD_CONTROL_11			0x0800
#define LCD_CONTROL_12			0x1000
#define LCD_CONTROL_13			0x2000
#define LCD_CONTROL_14			0x4000
#define LCD_CONTROL_15			0x8000


/* LCD Data Register Bit Map */
#define LCD_DATA_0				0x0001
#define LCD_DATA_1				0x0002
#define LCD_DATA_2				0x0004
#define LCD_DATA_3				0x0008
#define LCD_DATA_4				0x0010
#define LCD_DATA_5				0x0020
#define LCD_DATA_6				0x0040
#define LCD_DATA_7				0x0080
#define LCD_DATA_8				0x0100
#define LCD_DATA_9				0x0200
#define LCD_DATA_10				0x0400
#define LCD_DATA_11				0x0800
#define LCD_DATA_12				0x1000
#define LCD_DATA_13				0x2000
#define LCD_DATA_14				0x4000
#define LCD_DATA_15				0x8000


/* IP Interrupt Register Bit Map */
#define IP_INT_A_IRQ0			0x0001
#define IP_INT_A_IRQ1			0x0002
#define IP_INT_B_IRQ0			0x0004
#define IP_INT_B_IRQ1			0x0008
#define IP_INT_C_IRQ0			0x0010
#define IP_INT_C_IRQ1			0x0020
#define IP_INT_D_IRQ0			0x0040
#define IP_INT_D_IRQ1			0x0080
#define IP_INT_E_IRQ0			0x0100
#define IP_INT_E_IRQ1			0x0200
#define IP_INT_F_IRQ0			0x0400
#define IP_INT_F_IRQ1			0x0800
#define IP_INT_SPARE1			0x1000
#define IP_INT_SPARE2			0x2000
#define IP_INT_SPARE3			0x4000
#define IP_INT_SPARE4			0x8000


/* IP Error Interrupt Register Bit Map */
#define IP_ERROR_INT_A			0x0001
#define IP_ERROR_INT_B			0x0002
#define IP_ERROR_INT_C			0x0004
#define IP_ERROR_INT_D			0x0008
#define IP_ERROR_INT_E			0x0010
#define IP_ERROR_INT_F			0x0020
#define IP_ERROR_SPARE0			0x0040
#define IP_ERROR_SPARE1			0x0080
#define IP_ERROR_SPARE2			0x0100
#define IP_ERROR_SPARE3			0x0200
#define IP_ERROR_SPARE4			0x0400
#define IP_ERROR_SPARE5			0x0800
#define IP_ERROR_SPARE6			0x1000
#define IP_ERROR_SPARE7			0x2000
#define IP_ERROR_SPARE8			0x4000
#define IP_ERROR_SPARE9			0x8000



/* IP Interrupt Mask Register Bit Map */
#define IP_INT_MASK_A_IRQ0		0x0001
#define IP_INT_MASK_A_IRQ1		0x0002
#define IP_INT_MASK_B_IRQ0		0x0004
#define IP_INT_MASK_B_IRQ1		0x0008
#define IP_INT_MASK_C_IRQ0		0x0010
#define IP_INT_MASK_C_IRQ1		0x0020
#define IP_INT_MASK_D_IRQ0		0x0040
#define IP_INT_MASK_D_IRQ1		0x0080
#define IP_INT_MASK_E_IRQ0		0x0100
#define IP_INT_MASK_E_IRQ1		0x0200
#define IP_INT_MASK_F_IRQ0		0x0400
#define IP_INT_MASK_F_IRQ1		0x0800
#define IP_INT_MASK_SPARE1		0x1000
#define IP_INT_MASK_SPARE2		0x2000
#define IP_INT_MASK_SPARE3		0x4000
#define IP_INT_MASK_SPARE4		0x8000


/* IP Error Interrupt Mask Register Bit Map */
#define IP_ERROR_MASK_INT_A		0x0001
#define IP_ERROR_MASK_INT_B		0x0002
#define IP_ERROR_MASK_INT_C		0x0004
#define IP_ERROR_MASK_INT_D		0x0008
#define IP_ERROR_MASK_INT_E		0x0010
#define IP_ERROR_MASK_INT_F		0x0020
#define IP_ERROR_MASK_SPARE0	0x0040
#define IP_ERROR_MASK_SPARE1	0x0080
#define IP_ERROR_MASK_SPARE2	0x0100
#define IP_ERROR_MASK_SPARE3	0x0200
#define IP_ERROR_MASK_SPARE4	0x0400
#define IP_ERROR_MASK_SPARE5	0x0800
#define IP_ERROR_MASK_SPARE6	0x1000
#define IP_ERROR_MASK_SPARE7	0x2000
#define IP_ERROR_MASK_SPARE8	0x4000
#define IP_ERROR_MASK_SPARE9    0x8000


/* IP Clock Register Bit Map */
#define CLK_IP_A_8MHZ			0x0000
#define CLK_IP_A_32MHZ			0x0001
#define CLK_IP_B_8MHZ			0x0000
#define CLK_IP_B_32MHZ			0x0002
#define CLK_IP_C_8MHZ			0x0000
#define CLK_IP_C_32MHZ			0x0004
#define CLK_IP_D_8MHZ			0x0000
#define CLK_IP_D_32MHZ			0x0008
#define CLK_IP_E_8MHZ			0x0000
#define CLK_IP_E_32MHZ			0x0010
#define CLK_IP_F_8MHZ			0x0000
#define CLK_IP_F_32MHZ			0x0020
#define CLK_IP_SPARE1			0x0040
#define CLK_IP_SPARE2			0x0080
#define CLK_IP_SPARE3			0x0100
#define CLK_IP_SPARE4			0x0200
#define CLK_IP_SPARE5			0x0400
#define CLK_IP_SPARE6			0x0800
#define CLK_IP_SPARE7			0x1000
#define CLK_IP_SPARE8			0x2000
#define CLK_IP_SPARE9			0x4000
#define CLK_IP_SPARE10			0x8000


/* 9010 Maps all access to the IP Cards as Memory starting at Base Address 3 */
#define IP_A_MEMORY_BASE_ADDR	0x000000
#define IP_B_MEMORY_BASE_ADDR	0x200000
#define IP_C_MEMORY_BASE_ADDR	0x400000
#define IP_D_MEMORY_BASE_ADDR	0x600000
#define IP_E_MEMORY_BASE_ADDR	0x800000
#define IP_F_MEMORY_BASE_ADDR	0xA00000
#define IP_IO_BASE_ADDR			0xE00000


/* The IP I/O Address Space is sub-divided by the following offsets...*/
#define IP_A_IO_BASE_ADDR		0x000
#define IP_A_ID_BASE_ADDR		0x080
#define IP_B_IO_BASE_ADDR		0x100
#define IP_B_ID_BASE_ADDR		0x180
#define IP_C_IO_BASE_ADDR		0x200
#define IP_C_ID_BASE_ADDR		0x280
#define IP_D_IO_BASE_ADDR		0x300
#define IP_D_ID_BASE_ADDR		0x380
#define IP_E_IO_BASE_ADDR		0x400
#define IP_E_ID_BASE_ADDR		0x480
#define IP_F_IO_BASE_ADDR		0x500
#define IP_F_ID_BASE_ADDR		0x580
#define IP_A_INT_BASE_ADDR		0x800
#define IP_B_INT_BASE_ADDR		0x900
#define IP_C_INT_BASE_ADDR		0xA00
#define IP_D_INT_BASE_ADDR		0xB00
#define IP_E_INT_BASE_ADDR		0xC00
#define IP_F_INT_BASE_ADDR		0xD00

/* The IP Card's ID PROM Follows the 'VITA4 ' Standard */
/* The Below Defines are the offset from each IP Cards */
#define ID_ASCII_VI				0
#define ID_ASCII_TA				2
#define ID_ASCII_4				4
#define ID_HYTEC_ID_HIGH		6		
#define ID_HYTEC_ID_LOW			8
#define ID_MODEL_NUMBER			10
#define ID_REVISION				12		

/* IOCTL operation codes */

#define HYTEC_IOC_MAGIC             'H'

#define OP_GET_CONFIG	            _IO( HYTEC_IOC_MAGIC, 1)
#define OP_GENERAL_READ				_IOR(HYTEC_IOC_MAGIC, 2 , int)
#define OP_GENERAL_READ_BLOCK		_IOR(HYTEC_IOC_MAGIC, 3 , int)
#define OP_GENERAL_WRITE_BLOCK		_IOW(HYTEC_IOC_MAGIC, 4 , int)
#define OP_CARRIER_READ_BLOCK		_IOR(HYTEC_IOC_MAGIC, 5 , int)
#define OP_CARRIER_WRITE_BLOCK		_IOW(HYTEC_IOC_MAGIC, 6 , int)
#define OP_GENERAL_MEM_READ_BLOCK	_IOR(HYTEC_IOC_MAGIC, 7 , int)
#define OP_GENERAL_MEM_WRITE_BLOCK	_IOW(HYTEC_IOC_MAGIC, 8 , int)
#define OP_GENERAL_VME_READ_BLOCK	_IOR(HYTEC_IOC_MAGIC, 9 , int)
#define OP_GENERAL_VME_WRITE_BLOCK	_IOW(HYTEC_IOC_MAGIC, 10, int)
#define OP_ENABLE_INTERRUPT         _IO( HYTEC_IOC_MAGIC, 11)
#define OP_GENERAL_VME_READ_BLOCK_32    _IOR(HYTEC_IOC_MAGIC, 12, int)
#define OP_GENERAL_VME_WRITE_BLOCK_32   _IOW(HYTEC_IOC_MAGIC, 13, int)
#define OP_GET_3331_CSR	            _IO( HYTEC_IOC_MAGIC, 14)
#define OP_GET_5331_CSR	            _IO( HYTEC_IOC_MAGIC, 14) /* Compatibility only */
#define OP_3331_RESET	            _IO( HYTEC_IOC_MAGIC, 15)



typedef struct
{
	unsigned short	sHardwareID;	
} IOCTL_CONFIG;


typedef struct
{
	unsigned long	sData;	
  	unsigned long	lAddress;
	unsigned long 	lLength;
} IOCTL_BUF;


typedef struct
{
	unsigned long	sData;	
  	unsigned long	lSlot;
  	unsigned long	lSite;
  	unsigned long	lMemAccess;
  	unsigned long	lAddress;
	unsigned long 	lLength;
} IOCTL_VME_BUF;



