/********************************************************************************/
/*    H   H Y   Y TTTTT EEEEE  CCC       HYTEC ELECTRONICS LTD                  */
/*    H   H  Y Y    T   E     C          5 Cradock Road,                        */
/*    HHHHH   Y     T   EEE   C          Reading, Berks.    Tel: 0118 9757770	*/
/*    H   H   Y     T   E     C          RG2 0JT            Fax: 0118 9757566	*/
/*    H   H   Y     T   EEEEE  CCC       Web: www.hytec-electronics.co.uk       */
/********************************************************************************/
/*                                                                              */
/*				IOC9010 PROJECT	--- Hytec IP Card API Functions 				*/
/*							for Windows and Linux Platforms	    				*/
/*                                IP Card Type: 8403                            */
/********************************************************************************/
/*  Source file name      :-      Hy8403.h										*/
/*  Definition header file:		  HYTECIPAPI.h									*/
/*  Platform			  :		  Linux, Windows XP, Windows 2000               */
/*  Initial creation date :-      10-JUL-2008                                   */
/*                                                                              */
/*  Original Developers   :-                                                    */
/********************************************************************************/
/*                                                                              */
/*  Description :-                                                              */
/*                                                                              */
/*  This header file is designed for Hytec API function library of 8403 IP card.*/
/*  It defines the class prototype, error code, masks and registers of 8403.    */
/*																				*/
/*  The design of this interface is cross platforms, which means the code 		*/
/*	can be used in operating systems like Windows, Linux etc without changes.	*/
/*                                                                              */
/*  The language used is C++ but the final product -- the Dynamic Link library	*/
/*	(dll) or the shared library can be used by either C or C++ applications.	*/
/*																				*/
/*  (C)2006 Hytec Electronics Ltd.												*/
/*                                                                              */
/********************************************************************************/
/*                                                                              */
/*  Modification record :-                                                      */
/*                                                                              */
/*  Date modified |     Person    |   Description of modification | Version     */
/* ---------------|---------------|-------------------------------------------- */
/*                |	              |	  Initial copy				  |	0.0.1		*/
/*                |               |                                             */
/*                                                                              */
/********************************************************************************/
#include "HytecIPAPI.h"

#if PLATFORM == 1

#define HYTECAPI_EXPORTS 1
#ifdef HYTECAPI_EXPORTS
#define HYTECAPI_API __declspec(dllexport)
#else
#define HYTECAPI_API __declspec(dllimport)
#endif
#include "C:\Hytec\IOC9010\IOCGeneric\IOCGeneric\IOCGeneric.h"

#else

#include "IOCGeneric.h"

#endif


//error code definition
#define NOT_8403_IP -8403001
#define CHANNEL_INVALID_8403 -8403002
#define TYPE_INVALID_8403 -8403003
#define GAIN_INVALID_8403 -8403004
#define Hy8403_NOT_CONFIGURED -8403014

//signature definition
#define SIGNATURE_8403 0x8403

//register address defines
#define CSR_8403    0x0000
#define CONPTR_8403 0x0002
#define NUMCON_8403 0x0004
#define CLKRAT_8403 0x0006
#define VECTOR_8403 0x0008
#define ADCCON_8403 0x000A
#define ADCDAT_8403 0x000C
#define ACR_8403    0x0E
#define DACOUT_8403 0x10


#define PGAMUX_8403  0x20

#define PGAMUX_8403_ADCA_1  (PGAMUX_8403)
#define PGAMUX_8403_ADCB_1  (PGAMUX_8403 + 2)
#define PGAMUX_8403_ADCA_2  (PGAMUX_8403 + 4)
#define PGAMUX_8403_ADCB_2  (PGAMUX_8403 + 6)
#define PGAMUX_8403_ADCA_3  (PGAMUX_8403 + 8)
#define PGAMUX_8403_ADCB_3  (PGAMUX_8403 + 10)
#define PGAMUX_8403_ADCA_4  (PGAMUX_8403 + 12)
#define PGAMUX_8403_ADCB_4  (PGAMUX_8403 + 14)
#define PGAMUX_8403_ADCA_5  (PGAMUX_8403 + 16)
#define PGAMUX_8403_ADCB_5  (PGAMUX_8403 + 18)
#define PGAMUX_8403_ADCA_6  (PGAMUX_8403 + 20)
#define PGAMUX_8403_ADCB_6  (PGAMUX_8403 + 22)
#define PGAMUX_8403_ADCA_7  (PGAMUX_8403 + 24)
#define PGAMUX_8403_ADCB_7  (PGAMUX_8403 + 26)
#define PGAMUX_8403_ADCA_8  (PGAMUX_8403 + 28)
#define PGAMUX_8403_ADCB_8  (PGAMUX_8403 + 30)


#define THERMOCOUPLE_8403_CAL_A PGAMUX_8403_ADCA_8
#define THERMOCOUPLE_8403_CAL_B PGAMUX_8403_ADCB_7
#define THERMOCOUPLE_8403_PTR   PGAMUX_8403_ADCB_8

/* Defines Required for Calibrating Thermocouples */
#define CALBITS_8403	7110893
/* Number of Samples over which Average is taken */
#define AVERAGE_SIZE_8403 5

/* 
    REG_PGAMUX registers are 16 registers of 16 bits each.
    Each register describes the ADC multiplexer setting
    for the given channel.
*/

#define ADCBASE_8403 0x40
/*
    the ADC data registers are 16 channels of 32 bits
    starting from REG_ADCBASE (0x40 to 0x7E).
    chan 1 lo
    chan 1 hi
    chan 2 lo
    chan 2 hi
    ...
    chan 16 lo
    chan 16 hi
*/


//register mask defines
//CSR
#define CSR_8403_RDY  0x0001
#define CSR_8403_F    0x0002
#define CSR_8403_CC   0x0004
#define CSR_8403_MIS  0x0008
#define CSR_8403_MII  0x0010
#define CSR_8403_EII  0x0020
#define CSR_8403_ECAL 0x0040
#define CSR_8403_1M   0x0080
#define CSR_8403_HE   0x0100
#define CSR_8403_FE   0x0200
#define CSR_8403_EE   0x0400
#define CSR_8403_ET   0x0800
#define CSR_8403_XC   0x1000
#define CSR_8403_ST   0x2000
#define CSR_8403_EX   0x4000
#define CSR_8403_ARM  0x8000

/* define the opcodes that can be written to the ADC control
*  register REG_ADCCON */

#define CON_8403_ADCA           0x0000
#define CON_8403_ADCB           0x8000

#define CON_8403_WREG8          0x0001
#define CON_8403_WREG88         0x0101
#define CON_8403_RREG8          0x0002
#define CON_8403_RREG88         0x0102
#define CON_8403_RDAT           0x0013
#define CON_8403_CALOG          0x0004
#define CON_8403_CALO           0x0014
#define CON_8403_CALG           0x0024
#define CON_8403_SCALO          0x0034
#define CON_8403_SCALG          0x0044
#define CON_8403_SYNC           0x00C4
#define CON_8403_SLEEP          0x00D4
#define CON_8403_RESET          0x00E4
#define CON_8403_REG2RAM        0x00A5
#define CON_8403_RAM2REG        0x00A6

/*
    Each register in the ADC has a number [0..15].
    These numbers are used when issuing commands to the
    ADC (read, write, etc.)
*/
#define ADC_8403_SETUP  (0x00)
#define ADC_8403_MUX    (0x01)
#define ADC_8403_ACR    (0x02)
#define ADC_8403_IDAC1  (0x03)
#define ADC_8403_IDAC2  (0x04)
#define ADC_8403_ODAC   (0x05)
#define ADC_8403_DIO    (0x06)
#define ADC_8403_DIR    (0x07)
#define ADC_8403_DEC0   (0x08)
#define ADC_8403_DEC1   (0x09)
#define ADC_8403_OCRO   (0x0A)
#define ADC_8403_OCR1   (0x0B)
#define ADC_8403_OCR2   (0x0C)
#define ADC_8403_FSR0   (0x0D)
#define ADC_8403_FSR1   (0x0E)
#define ADC_8403_FSR2   (0x0F)

#define NUMADCREG  0x10


/* ADC SETUP Register Bit Defines */
/* ------------------------------ */
#define ADC_8403_SETUP_BORD	0x01	/* Bit Order 0 - MSB First, 1 - LSB First */
#define ADC_8403_SETUP_BUFEN	0x02	/* Buffer Enabled */ 
#define ADC_8403_SETUP_REFHI	0x04	/* Internal Ref 0 - 1.25V, 1 - 2.5V */ 
#define ADC_8403_SETUP_REFEN	0x08	/* Internal Ref 0 - Disabled, 1 - Enabled */
#define ADC_8403_SETUP_SPEED	0x10	/* Speed 0 - FMOD = 128, 1 - FMOD = 256 */
#define ADC_8403_SETUP_ID1	0x20	/* Factory Set Bits */
#define ADC_8403_SETUP_ID2	0x40
#define ADC_8403_SETUP_ID3	0x80


/* ADC ODAC Register Bit Defines */
/* ----------------------------- */
#define ADC_8403_ODAC_OSET0	0x01	/*  */
#define ADC_8403_ODAC_OSET1	0x02	/*  */ 
#define ADC_8403_ODAC_OSET2	0x04	/*  */ 
#define ADC_8403_ODAC_OSET3	0x08	/*  */
#define ADC_8403_ODAC_OSET4	0x10	/*  */
#define ADC_8403_ODAC_OSET5	0x20	/*  */
#define ADC_8403_ODAC_OSET6	0x40	/*  */
#define ADC_8403_ODAC_SIGN	0x80	/*  */

#define ADC_8403_ODAC_OFFSET_VOLTAGE	0xFF


/* ADC DEC0 Register Bit Defines */
/* ----------------------------- */
#define ADC_8403_DEC0_DEC00	0x01	/*  */
#define ADC_8403_DEC0_DEC01	0x02	/*  */ 
#define ADC_8403_DEC0_DEC02	0x04	/*  */ 
#define ADC_8403_DEC0_DEC03	0x08	/*  */
#define ADC_8403_DEC0_DEC04	0x10	/*  */
#define ADC_8403_DEC0_DEC05	0x20	/*  */
#define ADC_8403_DEC0_DEC06	0x40
#define ADC_8403_DEC0_DEC07	0x80


/* ADC DEC1 Register Bit Defines */
/* ----------------------------- */
#define ADC_8403_DEC1_DEC08	0x01	/*  */
#define ADC_8403_DEC1_DEC09	0x02	/*  */ 
#define ADC_8403_DEC1_DEC10	0x04	/*  */ 
#define ADC_8403_DEC1_RESERV	0x08	/* Reserved */
#define ADC_8403_DEC1_SMODE0	0x10	/* Setting Mode Bit 0 */
#define ADC_8403_DEC1_SMODE1	0x20	/* Setting Mode Bit 1 */
#define ADC_8403_DEC1_UB	0x40	/* 0 - Bipolar, 1 - Unipolar */
#define ADC_8403_DEC1_DRDY	0x80	/* Data Ready - Read Only Bit */ 

/* Setting Modes */
#define ADC_8403_SETTING_AUTO	0
#define ADC_8403_SETTING_FAST	ADC_8403_DEC1_SMODE0
#define ADC_8403_SETTING_SINC2	ADC_8403_DEC1_SMODE1
#define ADC_8403_SETTING_SINC3	(ADC_8403_DEC1_SMODE0 | ADC_8403_DEC1_SMODE1)

/* 
	ADC1216 Sampling Freq - This 11 Bit Value is loaded into the ADC1216 Registers 
	DEC1 (3 MSBs) and DEC0 (Lower 8 LSBs).
*/
#define ADC_8403_SAMPLE_FREQ		0x0300 
#define ADC_8403_SAMPLE_FREQ_MSB 	((ADC_8403_SAMPLE_FREQ >> 8) & 0x07) 
#define ADC_8403_SAMPLE_FREQ_LSB 	(ADC_8403_SAMPLE_FREQ & 0x00FF) 


/* these are predefined values for ADC registers */
#define MUX_AINCOM 0x08

/* the setup register */
#define STP_BUF_EN 0x02	/* Enable Buffer                          */
#define STP_REF_HI 0x04	/* 0 - 1.25V, 1 - 2.5V Internal Reference */ 
#define STP_REF_EN 0x08 /* Enable Internal Voltage Reference      */

/* the mode and decimation register */
#define MDEC_UB    0x40


#define WAIT4RDY_8403  data=0; \
  while(!data & CSR_8403_RDY) {  \
  status = IOC9010IPRead(initVMESlot, initIPSlot, CSR_8403, &data); \
            usleep(1); } \
            usleep(50000);

// This class is exported from the HYTECAPI.dll
#if PLATFORM == 1									//windows
class HYTECAPI_API CHy8403 : public CIOCGeneric
#else											//linux
class CHy8403 : public CIOCGeneric
#endif
{
private:
	long m_ErrorCode;  // global error code holder
	int initIPSlot;    // remember the IP slot when configured
	int initVMESlot;   // remember the VME slot when configured
    int initType;      // remember the type of inputs we are reading
    pthread_t readThread;
    int ThreadGo;      // use to control thread 0=stop

    void create_thread();   // creates a new thread and starts it
    friend void* proxy_function(void*);

    long ADCDATA[16];   // Private Array to hold the 16 channels of ADC data

/*	Array of Values for ADC PGA MUX */
    static short PGAMuxSettings[16][6];
    static int ADCChannelMapping[16][6];

    void Hy8403_Wait_Till_Ready();

/*----------------------------------------------------------------------------------*/
/*Function:     Periodically read the ADC Data Registers into local variables      	*/
/*----------------------------------------------------------------------------------*/
/*Description:	Private routine to read ADC Data Registers							*/
/*Prototype:	void Hy8403_timedReadThread()          	                            */
/*Arguments		ptr - pointer to thread argument                                    */
/*Return:	                                                						*/
/*----------------------------------------------------------------------------------*/
	void Hy8403_timedReadThread();


/*----------------------------------------------------------------------------------*/
/*Function:     Read the specified ADC Register                     				*/
/*----------------------------------------------------------------------------------*/
/*Description:	Private routine to read ADC Registers								*/
/*Prototype:	long Hy8403_rdadc(short adcsel, short adcreg)	                    */
/*Arguments		adcsel  - which ADC to read                              			*/
/*                      - 0x0000 = ADC A                                            */
/*                      - 0x8000 = ADC B                                            */
/*				adcreg  - which ADC register to read                       			*/
/*Return:		The contents of the requested register.      						*/
/*----------------------------------------------------------------------------------*/
	USHORT Hy8403_rdadc(short adcsel, short adcreg);

/*----------------------------------------------------------------------------------*/
/*Function:     Update the specified ADC Register with the passed value				*/
/*----------------------------------------------------------------------------------*/
/*Description:	Private routine to update ADC Registers								*/
/*Prototype:	long Hy8403_wradc(short adcsel, short adcreg, char val)	            */
/*Arguments		adcsel  - which ADC to read                              			*/
/*                      - 0x0000 = ADC A                                            */
/*                      - 0x8000 = ADC B                                            */
/*				adcreg  - which ADC register to read                       			*/
/*				val  - value to update ADC register with                   			*/
/*Return:		status code, 0 = successful. negative = errors						*/
/*----------------------------------------------------------------------------------*/
	long Hy8403_wradc(short adcsel, short adcreg, char val);

public:
	CHy8403(void);
	CHy8403(int vmeslot, char ipslot);
	~CHy8403(void);

	long GetErrorCode()
	{
		return m_ErrorCode;
	}
/*----------------------------------------------------------------------------------*/
/*Function:     Dump contents opf ADC registers     	                            */
/*----------------------------------------------------------------------------------*/
/*Description:	Private routine to read ADC Registers    							*/
/*Prototype:	long dumpadc()                 	                                    */
/*Arguments		none                                                                */
/*Return:	                                                						*/
/*----------------------------------------------------------------------------------*/
    long CHy8403::Hy8403_dumpadc();
	

/*----------------------------------------------------------------------------------*/
/*Function:		Configure 8403 IP card												*/
/*----------------------------------------------------------------------------------*/
/*Description:	8403 IP configuration routine										*/
/*				according to the settings.											*/
/*Prototype:	long Hy8403_Configure(int vmeslot, char ipslot, int type, int gain) */
/*Arguments:	vmeslot = VME slot number											*/
/*				ipslot = IP card slot number 0 ~ 3 for VME, 0 ~ 5 for IOC			*/
/*				type = type of input                                                */
/*                  0 = Dif	- Differential                                          */
/*                  1 = S/E	- Single Ended                                          */
/*                  2 = PT100	- PT-100 (original)                                 */
/*                  3 = TC	- Thermocouples                                         */
/*                  4 = PTB	- PT-100 with 8212 Transistion Card                     */
/*                  5 = TCB	- Thermocouples with 8212 Transistion Card              */
/*              gain = programmable gain - 0=1, 1=2, 2=4, 3=8, 4=16 ....7=128       */
/*Return:		status code, 0 = successful. negative = errors						*/
/*----------------------------------------------------------------------------------*/
	long Hy8403_Configure(int vmeslot, char ipslot, int type, int gain);

/*----------------------------------------------------------------------------------*/
/*Function:		Read 1 ADC channel of 8403											*/
/*----------------------------------------------------------------------------------*/
/*Description:	Read one channel data from 8403 ADC IP card							*/
/*Prototype:	long Hy8403_Read(int ADC_ch, ULONG *value)							*/
/*Arguements:	ADC_ch = ADC input channel 0 ~ 15									*/
/*				*value = pointer to the 16bit value variable						*/
/*Return:		0 is successfull, negative please see error define					*/
/*----------------------------------------------------------------------------------*/
	long Hy8403_Read(int ADC_ch, ULONG *value);
};
/*	Array of Values for ADC PGA MUX */
short CHy8403::PGAMuxSettings[16][6]={
/* Dif   S/E  PT100 TC    PTB   TCB */
{ 0x01, 0x08, 0x01, 0x08, 0x01, 0x01 },	/* ADC A - Channel 0 */
{ 0x01, 0x08, 0x01, 0x08, 0x01, 0x01 },	/* ADC B - Channel 0 */
{ 0x23, 0x18, 0x12, 0x18, 0x23, 0x23 },	/* ADC A - Channel 1 */
{ 0x23, 0x18, 0x12, 0x18, 0x23, 0x23 },	/* ADC B - Channel 1 */
{ 0x45, 0x28, 0x23, 0x28, 0x45, 0x45 },	/* ADC A - Channel 2 */
{ 0x45, 0x28, 0x23, 0x28, 0x45, 0x45 },	/* ADC B - Channel 2 */
{ 0x67, 0x38, 0x34, 0x38, 0x67, 0x67 },	/* ADC A - Channel 3 */
{ 0x67, 0x38, 0x34, 0x38, 0x67, 0x67 },	/* ADC B - Channel 3 */
{ 0x00, 0x48, 0x45, 0x48, 0x01, 0x01 },	/* ADC A - Channel 4 */
{ 0x00, 0x48, 0x45, 0x48, 0x01, 0x01 },	/* ADC B - Channel 4 */
{ 0x00, 0x58, 0x56, 0x58, 0x23, 0x23 },	/* ADC A - Channel 5 */
{ 0x00, 0x58, 0x56, 0x58, 0x23, 0x23 },	/* ADC B - Channel 5 */
{ 0x00, 0x68, 0x67, 0x68, 0x45, 0x45 },	/* ADC A - Channel 6 */
{ 0x00, 0x68, 0x67, 0x68, 0x45, 0x45 },	/* ADC B - Channel 6 */
{ 0x00, 0x78, 0x78, 0x78, 0x67, 0x67 },	/* ADC A - Channel 7 */
{ 0x00, 0x78, 0x78, 0x78, 0x67, 0x67}};	/* ADC B - Channel 7 */

int CHy8403::ADCChannelMapping[16][6]={
/* Dif   S/E  PT100 TC    PTB   TCB */
{ 0x40, 0x40, 0x40, 0x40, 0x40, 0x40 },	/* ADC A - Channel 0 */
{ 0x48, 0x48, 0x48, 0x48, 0x44, 0x44 },	/* ADC A - Channel 1 */
{ 0x50, 0x50, 0x50, 0x50, 0x48, 0x48 },	/* ADC A - Channel 2 */
{ 0x58, 0x58, 0x58, 0x58, 0x4C, 0x4C },	/* ADC A - Channel 3 */
{ 0x60, 0x60, 0x60, 0x60, 0x50, 0x50 },	/* ADC A - Channel 4 */
{ 0x68, 0x68, 0x68, 0x68, 0x54, 0x54 },	/* ADC A - Channel 5 */
{ 0x70, 0x70, 0x70, 0x70, 0x58, 0x58 },	/* ADC A - Channel 6 */
{ 0x78, 0x78, 0x78, 0x78, 0x58, 0x58 },	/* ADC A - Channel 7 */
{ 0x44, 0x44, 0x44, 0x44, 0x5C, 0x5C },	/* ADC B - Channel 0 */
{ 0x4C, 0x4C, 0x4C, 0x4C, 0x40, 0x40 },	/* ADC B - Channel 1 */
{ 0x54, 0x54, 0x54, 0x54, 0x44, 0x44 },	/* ADC B - Channel 2 */
{ 0x5C, 0x5C, 0x5C, 0x5C, 0x48, 0x48 },	/* ADC B - Channel 3 */
{ 0x64, 0x64, 0x64, 0x64, 0x4C, 0x4C },	/* ADC B - Channel 4 */
{ 0x6C, 0x6C, 0x6C, 0x6C, 0x50, 0x50 },	/* ADC B - Channel 5 */
{ 0x74, 0x74, 0x74, 0x74, 0x54, 0x54 },	/* ADC B - Channel 6 */
{ 0x7C, 0x7C, 0x7C, 0x7C, 0x5C, 0x5C}}; /* ADC B - Channel 7 */


#if PLATFORM == 1									//windows
extern "C" HYTECAPI_API CHy8403* CreateCHy8403();
extern "C" HYTECAPI_API void DeleteCHy8403( CHy8403* instance );
#else											//linux
extern "C" CHy8403* CreateCHy8403();
extern "C" void DeleteCHy8403( CHy8403* instance );
#endif

