/********************************************************************************/
/*    H   H Y   Y TTTTT EEEEE  CCC       HYTEC ELECTRONICS LTD                  */
/*    H   H  Y Y    T   E     C          5 Cradock Road,                        */
/*    HHHHH   Y     T   EEE   C          Reading, Berks.    Tel: 0118 9757770	*/
/*    H   H   Y     T   E     C          RG2 0JT            Fax: 0118 9757566	*/
/*    H   H   Y     T   EEEEE  CCC       Web: www.hytec-electronics.co.uk       */
/********************************************************************************/
/*                                                                              */
/*				IOC9010 PROJECT	--- Hytec IP Card API Functions 				*/
/*							for Windows and Linux Platforms	    				*/
/*                                IP Card Type: 8404                            */
/********************************************************************************/
/*  Source file name      :-      Hy8404.h										*/
/*  Definition header file:		  HYTECIPAPI.h									*/
/*  Platform			  :		  Linux, Windows XP, Windows 2000               */
/*  Initial creation date :-      03-AUG-2008                                   */
/*                                                                              */
/*  Original Developers   :-      Darrell Nineham                               */
/********************************************************************************/
/*                                                                              */
/*  Description :-                                                              */
/*                                                                              */
/*  This header file is designed for Hytec API function library of 8404 IP card.*/
/*  It defines the class prototype, error code, masks and registers of 8404.    */
/*																				*/
/*  The design of this interface is cross platforms, which means the code 		*/
/*	can be used in operating systems like Windows, Linux etc without changes.	*/
/*                                                                              */
/*  The language used is C++ but the final product -- the Dynamic Link library	*/
/*	(dll) or the shared library can be used by either C or C++ applications.	*/
/*																				*/
/*  (C)2008 Hytec Electronics Ltd.												*/
/*                                                                              */
/********************************************************************************/
/*                                                                              */
/*  Modification record :-                                                      */
/*                                                                              */
/*  Date modified |     Person    |   Description of modification | Version     */
/* ---------------|---------------|-------------------------------------------- */
/*    03/08/08    |	     DAN      |	  Initial copy				  |	0.0.1		*/
/*                |               |                                             */
/*                                                                              */
/********************************************************************************/
#include "HytecIPAPI.h"

#if PLATFORM == 1

#define HYTECAPI_EXPORTS 1
#ifdef HYTECAPI_EXPORTS
#define HYTECAPI_API __declspec(dllexport)
#else
#define HYTECAPI_API __declspec(dllimport)
#endif
#include "C:\Hytec\IOC9010\IOCGeneric\IOCGeneric\IOCGeneric.h"

#else

#include "IOCGeneric.h"

#endif


//error code definition
#define NOT_8404_IP             -8404001
#define CLOCKRATE_INVALID_8404  -8404012
#define CHANNEL_INVALID_8404    -8404013
#define Hy8404_NOT_CONFIGURED   -8404014

//mask definition
#define ARMDAC_8404         0x8000
#define EXT_TRG_8404        0x4000
#define SOFT_TRG_8404       0x2000
#define CLOCKSOURCE_8404    0x1000
#define EXT_INHIBIT_8404    0x0800
#define MEMORY_INT_8404     0x0400
#define FF_INT_8404         0x0200
#define HF_INT_8404         0x0100
#define SPARE_1_8404        0x0080
#define SPARE_2_8404        0x0040
#define MEMORY_1MB_8404     0x0020
#define MEM_INHIBIT_8404    0x0010
#define CONTINOUS_8404      0x0008
#define CONV_8404           0x0004
#define FULL_8404           0x0002
#define HALF_FULL_8404      0x0001

#define CHANNEL_MEMORY_BLOCK_8404 0x08000


//signature definition
#define SIGNATURE_8404      0x8404

//register address defines
#define CSR_8404 			0x0000
#define MEMORY_8404			0x0002
#define UPDATES_8404		0x0004
#define CLOCK_RATE_8404 	0x0006
#define VECTOR_8404 		0x0008
#define TRIM_ID_8404 		0x000A
#define SPARE_8404		    0x000C
#define UPDATES2_8404		0x000E
#define FIRST_CHANNEL_8404 	0x0010
#define FIRST_CH_CMD_8404 	0x0030

//DAC Range Commands
#define RANGE_5V_UNIPOLAR_8404  0x80
#define RANGE_10V_UNIPOLAR_8404 0x90
#define RANGE_5V_BIPOLAR_8404   0xA0
#define RANGE_10V_BIPOLAR_8404  0xB0
#define RANGE_2V5_BIPOLAR_8404  0xC0
#define RANGE_2V5_TO_7V5_8404   0xD0



// This class is exported from the HYTECAPI.dll
#if PLATFORM == 1									//windows
class HYTECAPI_API CHy8404 : public CIOCGeneric
#else											//linux
class CHy8404 : public CIOCGeneric
#endif
{
private:
	long m_ErrorCode;							// global error code holder
	int initIPSlot;								// remember the IP slot when configures
	int initVMESlot;

public:
	CHy8404(void);
	CHy8404(int vmeslot, char ipslot);
	~CHy8404(void);

	long GetErrorCode()
	{
		return m_ErrorCode;
	}
	
/*----------------------------------------------------------------------------------*/
/*Function:		Configure 8404 IP card												*/
/*----------------------------------------------------------------------------------*/
/*Description:	8404 IP configuration routine										*/
/*				according to the settings.											*/
/*Prototype:	long Hy8404_Configure(int vmeslot, char ipslot, int vectornum,		*/
/*				int doram, int clockSource, int clockRate, int inhibit)				*/
/*Arguements:	vmeslot = VME slot number											*/
/*				ipslot = IP card slot number 0 ~ 3 for VME, 0 ~ 5 for IOC			*/
/*				vectornum = interrupt vector number									*/
/*				doram = 0, output with register, = 1, output with RAM @ClockRate	*/
/*				clockSource = 0, internal, = 1, external							*/
/*				clockRate = 0 ~ 13, clock rate for output if selected. see manual	*/
/*				inhibit = 0, inhibit disabled, = 1, inhibit enabled					*/
/*Return:		status code, 0 = successful. negative = errors						*/
/*----------------------------------------------------------------------------------*/
	long Hy8404_Configure(int vmeslot, char ipslot, int vectornum, int doram, 
							   int clockSource, int clockRate, int inhibit);
/*----------------------------------------------------------------------------------*/
/*Function:		Write One Channel													*/
/*----------------------------------------------------------------------------------*/
/*Description:	Output data to a DAC channel of 8404								*/
/*Prototype:	long Hy8404_Write(USHORT DAC_ch, USHORT value)						*/
/*Arguements:	ADC_ch = DAC output channel 0 ~ 15									*/
/*				value = unsigned 16bit value										*/
/*				range = unsigned 16bit indicating which voltage range to use        */
/*Return:		0 is successfull, negative please see error definition				*/
/*----------------------------------------------------------------------------------*/
	long Hy8404_Write(int DAC_ch, USHORT value, USHORT range);

/*--------------------------------------------------------------------------------------*/
/*Function:		Write Waveform Data		 				    							*/
/*--------------------------------------------------------------------------------------*/
/*Description:	Output the whole waveform data (64k byte) to a DAC channel memory area	*/
/*Prototype:	long Hy8402_Write(int DAC_ch, USHORT *value)	     					*/
/*Arguements:	ADC_ch = DAC output channel 0 ~ 15					    				*/
/*				*value = pointer to a 32k word array				    				*/
/*				range = unsigned 16bit indicating which voltage range to use            */
/*Return:		0 is successfull, negative please see error definition	    			*/
/*--------------------------------------------------------------------------------------*/
    long Hy8404_Write(int DAC_ch, USHORT *value, USHORT range);


};
#if PLATFORM == 1									//windows
extern "C" HYTECAPI_API CHy8404* CreateCHy8404();
extern "C" HYTECAPI_API void DeleteCHy8404( CHy8404* instance );
#else											//linux
extern "C" CHy8404* CreateCHy8404();
extern "C" void DeleteCHy8404( CHy8404* instance );
#endif

