/******************************************************************************
*        ____
*      _-    -_    _------_
*     -        -_--        -_
*          _--------------_   -_
* 	_---              -_
*      _-                 ----___
*                         ---___
*      Copyright (c) 2000-2001 Vista Control Systems, Inc.
*      All rights reserved
*      176 Central Park Square * Los Alamos, N.M. 87544 * 505-662-2484
*
*      Filename: vhytec/bittest.c
*
**********************************************************************/

/*
 * Program to research the best way of testing if a binary value has changed.
 * Author: Eugene Zharkov
 * Started: 15-Dec-2000
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <vsys_version.h>
#include <vsys_errors.h>
#include <vsys_time.h>
#include <vsys_cli.h>

static char programName[] = "bittest";
static char commandName[] = "bittest";
static char versionString[] = VSYS_VERSION_STRING ".1";

typedef int bool;

static struct {
  bool banner;
  unsigned count;		/* Repetition count. */
  unsigned numChanges;		/* Every numChanges's bytes is not 0. */
  unsigned numInt32s;
  unsigned testNumber;
  unsigned logTotal;
  unsigned logCount;
  unsigned buffer[2048];
} data;

/*
 *********************************************************************
 */

#define RRR() rrr (__LINE__)

static void rrr (int line) {
  printf ("vhytec/server: error at %d\n", line);
  exit (EXIT_FAILURE);
}

/*
 *********************************************************************
 */

static void logValue (int value) {
  data.logCount ++;
  data.logTotal += value;
}

/*
 *********************************************************************
 */

static void test1 (void) {

  unsigned i;

  for (i = 0; i < data.count; i++) {
    VUINT32 *p32 = data.buffer;
    unsigned i32;
    for (i32 = 0; i32 < data.numInt32s; i32++) {
      if (p32[i32] != 0) {
	VUINT8 *p8 = (VUINT8*) &p32[i32];
	unsigned i8;
	for (i8 = 0; i8 < 4; i8++, p8++) {
	  if (*p8 != 0) {
	    unsigned i1 = 0;
	    for (i1 = 0; i1 < 8; i1++) {
	      VUINT8 mask = 1 << i1;
	      int value = (*p8 & mask) == 0 ? 0 : 1;
	      logValue (value);
	    }
	  }
	}
      }
    }
  }
}

/*
 *********************************************************************
 */

static void test2 (void) {

  unsigned i;

  for (i = 0; i < data.count; i++) {
    VUINT8 *p8 = (VUINT8*) data.buffer;
    unsigned i8;
    for (i8 = 0; i8 < data.numInt32s * 4; i8++, p8++) {
      if (*p8 != 0) {
	unsigned i1 = 0;
	for (i1 = 0; i1 < 8; i1++) {
	  VUINT8 mask = 1 << i1;
	  int value = (*p8 & mask) == 0 ? 0 : 1;
	  logValue (value);
	}
      }
    }
  }
}

/*
 *********************************************************************
 */

static void prepareDataBuffer (void) {

  VUINT8 *ptr = (VUINT8*) data.buffer;
  unsigned i;

  if (data.numInt32s * 4 >= sizeof data.buffer) RRR ();
  for (i = 0; i < data.numInt32s * 4; i++) {
    ptr[i] = (i % data.numChanges) == 0 ? 1 : 0;
  }
}

/*
 *********************************************************************
 */

int main (int argc, char *argv[]) {

  vtime startTime, endTime, elapsedTime;
  long oper = VSYS_SECS;
  vcli_t * cli;
  double elapsed;
  int status;

  if ((cli = vcliInit ()) == NULL) RRR ();

  if (vcliSetTitle (cli, programName) != VCLI_OK ||
      vcliSetVersion (cli, versionString) != VCLI_OK ||
      vcliSetCommandName (cli, commandName) != VCLI_OK ||
      vcliEnableSortedHelp (cli) != VCLI_OK ||
      vcliVparse
      (cli,
       argc, argv,
       "q banner default 1", &data.banner,
       "q change $uint default 4", &data.numChanges,
       "q count $uint default 1000", &data.count,
       "q num32 $uint default 1000", &data.numInt32s,
       "q test $int default 1", &data.testNumber,
       NULL) != VCLI_OK) {
    vcliPerror (cli, NULL);
    RRR ();
  }

  if (data.banner)
    printf ("Vsystem %s Version %s\n", programName, versionString);

  prepareDataBuffer ();
  printf ("test = %d\n", data.testNumber);
  printf ("Num ints = %d, num changes = %d, count = %d\n",
	  data.numInt32s, data.numChanges, data.count);
  vsys_get_time (&startTime);

  switch (data.testNumber) {
  case 1: test1 (); break;
  case 2: test2 (); break;
  default: RRR ();
  }

  vsys_get_time (&endTime);
  vsys_subtime (&endTime, &startTime, &elapsedTime, &status);
  vsys_conv_to_double (&oper, &elapsed, &elapsedTime, &status);
  printf ("Log total = %u, count = %u\n", data.logTotal, data.logCount);
  printf ("Elapsed time = %g seconds\n", elapsed);
  
  return EXIT_SUCCESS;
}
