/******************************************************************************
*        ____
*      _-    -_    _------_
*     -        -_--        -_
*          _--------------_   -_
* 	_---              -_
*      _-                 ----___
*                         ---___
*      Copyright (c) 2003-2004 Vista Control Systems, Inc.
*      All rights reserved
*      176 Central Park Square * Los Alamos, N.M. 87544 * 505-662-2484
*
*      Filename: vhytec/scanner_lib.h
*
**********************************************************************/

#ifndef scanner_lib_h_LOADED
#define scanner_lib_h_LOADED

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <verr.h>
#include <vthread.h>
#include <vsys_cli.h>
#include <vsys_time.h>
#include <vsys_alloc.h>
#include <vsys_routines.h>

#include <vdb_routines.h>
#include <vdb_errors.h>
#include <vdb_class.h>

#define VSCANNER_SUCCESS 0
#define VSCANNER_FAILURE (-1)

extern "C" {

/* typedef enum { false, true } bool; */
typedef int int01;

/* routines returning void */

void vscannerBanner (void);
void vscannerExitFailure (void);
void vscannerFatalInterr_LINE (char* line);
void vscannerFatalVdbChixError_LINE (vista_chix* c, char* line);
void vscannerFatalVdbStatusError_LN (vista_chix* c, int status, char* line);
void vscannerIgnoreSigfpe (void);
void vscannerInitLogMutex (void);
void vscannerInitStatusChannel (void);
void vscannerLogMsg (char* format, ...);
void vscannerLogMsgNoprint (char* format, ...);
void vscannerLogFatal (char* format, ...);
void vscannerSetEventQueueSize (void);
void vscannerSetNameAndVersion (char* name, char* version);

/* routines returning status */

int vscannerDefineStandardVcliGroup (vcli_t* cli);
int vscannerExitSuccess (void);
int vscannerLogError (char* format, ...);

/* routines returning an int value (but not status) */

int vscannerDbChannelType (vista_chix* chix);
int vscannerDbIn (vista_chix* chix);

/* ... */

int01 vscannerGetNoline (void);

/* ... */
}
#define vscannerFatalInterr() vscannerFatalInterr_LINE (VERR__FILE__LINE__)
#define vscannerFatalVdbChixError(c)				\
  vscannerFatalVdbChixError_LINE (c, VERR__FILE__LINE__)
#define vscannerFatalVdbStatusError(c, s)			\
  vscannerFatalVdbStatusError_LN (c, s, VERR__FILE__LINE__)

#define VSCANNER_TIME_CMP(t1, t2) (		\
  (t1).t_sec > (t2).t_sec ? 1 :			\
  (t1).t_sec < (t2).t_sec ? -1 :		\
  (t1).t_nsec > (t2).t_nsec ? 1 :		\
  (t1).t_nsec == (t2).t_nsec ? 0 : -1		\
)

#define VSCANNER_TIME_SUB(t1, t2, t3) (				\
  (t1).t_nsec < (t2).t_nsec ?					\
   ((t3).t_sec = (t1).t_sec - (t2).t_sec - 1,			\
    (t3).t_nsec =  (t1).t_nsec + 1000000000 - (t2).t_nsec) :	\
   ((t3).t_sec = (t1).t_sec - (t2).t_sec,			\
    (t3).t_nsec = (t1).t_nsec - (t2).t_nsec)			\
)

#define VSCANNER_TIME_ADD(t1, t2, t3) do {	\
  (t3).t_sec = (t1).t_sec + (t2).t_sec;		\
  (t3).t_nsec = (t1).t_nsec + (t2).t_nsec;	\
  if ((t3).t_nsec >= 1000000000) {		\
    (t3).t_sec += 1;				\
    (t3).t_nsec -= 1000000000;			\
  }						\
} while (0)

#define VSCANNER_TIME_ADD_SEC(t1, sec) do {	\
  (t1).t_sec = sec;				\
} while (0)

#endif
