/******************************************************************************
*        ____
*      _-    -_    _------_
*     -        -_--        -_
*          _--------------_   -_
* 	_---              -_
*      _-                 ----___
*                         ---___
*      Copyright (c) 2001-2004 Vista Control Systems, Inc.
*      All rights reserved
*      176 Central Park Square * Los Alamos, N.M. 87544 * 505-662-2484
*
*      Filename: vhytec/slio.c
*
**********************************************************************/

/*
 * Serial Line I/O.
 */

/*
 * TODO:
 *
 * - modem control on all UNIXes and VMS
 * - set select timeout depending on the line speed
 */

#include <stdio.h>
#include <stdlib.h>

#ifdef UNIX /* fixme5: remove when select is replaced with VTIME */
#include <sys/time.h>
#endif

#ifdef UNIX
#include <errno.h>
#include <fcntl.h>
#include <termios.h>
#include <stropts.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <sys/types.h>
#endif

#ifdef VMS
#include <descrip.h>
#include <iodef.h>
#include <iosbdef.h>
#include <ssdef.h>
#include <starlet.h>
#include <stsdef.h>
#include <ttdef.h>
#endif
extern "C" {
#include <vsys_internal.h>
#include "internal.h"
}
/* I use "#if bob" to mark old Bob Knight's stuff. Make sure that bob
   is not defined accidentally elsewhere. */
#ifdef bob
error, should not be defined;
#endif

#include "slio.h"

#define SUCCESS SL_OK
#define FAILURE (-1)

/*
 *********************************************************************
 */

static struct {
  int baudSym;
  int baud;
} baudSymTable[] = {
#ifdef WINNT
  { CBR_110, 110 },
  { CBR_300, 300 },
  { CBR_600, 600 },
  { CBR_1200, 1200 },
  { CBR_2400, 2400 },
  { CBR_4800, 4800 },
  { CBR_9600, 9600 },
  { CBR_14400, 14400 },
  { CBR_19200, 19200 },
  { CBR_38400, 38400 },
  { CBR_56000, 56000 },
  { CBR_57600, 57600 },
  { CBR_115200, 115200 },
  { CBR_128000, 128000 },
  { CBR_256000, 256000 }
#elif defined VMS
  { TT$C_BAUD_50, 50 },
  { TT$C_BAUD_75, 75 },
  { TT$C_BAUD_110, 110 },
  { TT$C_BAUD_134, 134 },
  { TT$C_BAUD_150, 150 },
  { TT$C_BAUD_300, 300 },
  { TT$C_BAUD_600, 600 },
  { TT$C_BAUD_1200, 1200 },
  { TT$C_BAUD_1800, 1800 },
  { TT$C_BAUD_2000, 2000 },
  { TT$C_BAUD_2400, 2400 },
  { TT$C_BAUD_3600, 3600 },
  { TT$C_BAUD_4800, 4800 },
  { TT$C_BAUD_7200, 7200 },
  { TT$C_BAUD_9600, 9600 },
  { TT$C_BAUD_19200, 19200 },
  { TT$C_BAUD_38400, 38400 },
  { TT$C_BAUD_57600, 57600 },
  { TT$C_BAUD_76800, 76800 },
  { TT$C_BAUD_115200, 115200 }
#elif defined UNIX
  { B50, 50 },
  { B75, 75 },
  { B134, 134 },
  { B150, 150 },
  { B200, 200 },
  { B300, 300 },
  { B600, 600 },
  { B1200, 1200 },
  { B1800, 1800 },
  { B2400, 2400 },
  { B4800, 4800 },
  { B9600, 9600 },
  { B19200, 19200 },
  { B38400, 38400 },
#ifdef B57600
  { B57600, 57600 },
#endif
#ifdef B76800
  { B76800, 76800 },
#endif
#ifdef B115200
  { B115200, 115200 },
#endif
#ifdef B153600
  { B153600, 153600 },
#endif
#ifdef B230400
  { B230400, 230400 },
#endif
#ifdef B307200
  { B307200, 307200 },
#endif
#ifdef B460800
  { B460800, 460800 },
#endif
#ifdef B500000
  { B500000, 500000 },
#endif
#ifdef B576000
  { B576000, 576000 },
#endif
#ifdef B921600
  { B921600, 921600 },
#endif
#ifdef B1000000
  { B1000000, 1000000 },
#endif
#ifdef B1152000
  { B1152000, 1152000 },
#endif
#ifdef B1500000
  { B1500000, 1500000 },
#endif
#ifdef B2000000
  { B2000000, 2000000 },
#endif
#ifdef B2500000
  { B2500000, 2500000 },
#endif
#ifdef B3000000
  { B3000000, 3000000 },
#endif
#ifdef B3500000
  { B3500000, 3500000 },
#endif
#ifdef B4000000
  { B4000000, 4000000 },
#endif
  { B110, 110 } /* We need something without a comma at the end, i.e.
		   something that is defined on all platforms. */
#else
  what else ?;
#endif
};

/*
 *********************************************************************
 */

#ifdef WINNT
static int winntError (sl_t *h, char *routine) {
  /* fixme4: convert code to string */
  int code = GetLastError ();
  sprintf (h->errstr, "VMBSL_ERROR, error %d from routine %s", code, routine);
  return FAILURE;
}
#endif

#ifdef UNIX
static int unixError (sl_t *h, char *routine) {
  /* fixme4: convert code to string */
  int code = errno;
  (void) sprintf (h->errstr, "VMBSL_ERROR, error %d from routine %s",
		  code, routine);
  return FAILURE;
}
#endif

#ifdef VMS
static int vmsError (sl_t *h, char *routine, int status) {
  /* fixme4: convert code to string */
  sprintf (h->errstr, "VMBSL_ERROR, error %%x%x from routine %s",
	   status, routine);
  return FAILURE;
}
#endif

/*
 *********************************************************************
 */

#define errEOF(h) stringError (h, "VMBSL_EOF, unexpected eof of data")
#define errINVDEV(h) stringError (h, "VMBSL_NAME, invalid device name")
#define errTIMEOUT(h) stringError (h, "VMBSL_TIMEOUT, connection timed out")

static int stringError (sl_t *h, char *string) {
  (void) vsys_strncpy (h->errstr, string, sizeof h->errstr);
  return FAILURE;
}

/*
 *********************************************************************
 */

#define errINVARG(h) errInvarg (h, __LINE__)

static int errInvarg (sl_t *h, int line) {
  (void) sprintf (h->errstr, "VMBSL_INVARG, Invalid argument (%d)", line);
  return FAILURE;
}

/*
 *********************************************************************
 */

#define errINTERR(h) errInterr (h, __LINE__)
#define errFIXME(h) errInterr (h, __LINE__)

static int errInterr (sl_t *h, int line) {
  (void) sprintf (h->errstr, "VMBSL_INTERR, Internal error %d", line);
  return FAILURE;
}

/*
 *********************************************************************
 */

#define errBAUDSYM(h, sym) errBaudSym (h, sym)

static int errBaudSym (sl_t *h, int sym) {
  (void) sprintf (h->errstr, "VMBSL_BAUDSYM, Unsupported baud rate value %d",
		  sym);
  return FAILURE;
}

/*
 *********************************************************************
 */

static int baudSymToBaud (sl_t *h, int baudSym, int *baud) {
  unsigned i;
  for (i = 0; i < VSYS_ARRAY_SIZE (baudSymTable); i++) {
    if (baudSymTable[i].baudSym == baudSym) {
      *baud = baudSymTable[i].baud;
      return SUCCESS;
    }
  }
  return errBAUDSYM (h, baudSym);
}

/*
 *********************************************************************
 */

static int baudToBaudSym (sl_t *h, int baud, int *baudSym) {
  unsigned i;
  for (i = 0; i < VSYS_ARRAY_SIZE (baudSymTable); i++) {
    if (baudSymTable[i].baud == baud) {
      *baudSym = baudSymTable[i].baudSym;
      return SUCCESS;
    }
  }
  return errINVARG (h);
}
      
/*
 *********************************************************************
 */

#ifdef WINNT
static int getIntervalTimeout (sl_t *h, int baudSym, vtime *time) {
  int baud;
  if (baudSymToBaud (h, baudSym, &baud) != SUCCESS)
    return FAILURE;
  else {
    double tmo = (1.0 / (baud / 10.0)) * 10.0; 
    time->t_sec = (unsigned) tmo;
    time->t_nsec = (unsigned) ((tmo - time->t_sec) * 1000000000) % 999999999;
    return SUCCESS;
  }
}
#endif

/*
 *********************************************************************
 */

int slRead (sl_t *h, void *reply, unsigned len, unsigned *bytesRead) {

#if bob
  if (vp->serial.delays.turnaround)
    vsys_wait(vp->serial.delays.turnaround);
#endif

  {
#if defined VMS

    unsigned terminators[2] = {0, 0}; /* No terminators */
    unsigned timeout = h->timeout;
    int status;
    iosb iosb;

    if (h->timeoutFlag)
      return errTIMEOUT (h);

    /* fixme2: old hytec uses IO$_READPROMPT, maybe I should do the same? */

    status = sys$qiow (h->ef, h->chan, 
		       IO$_READLBLK | IO$M_NOECHO | IO$M_NOFILTR |
		       IO$M_PURGE | IO$M_NOFORMAT | IO$M_TIMED,
		       &iosb, 0, 0,
		       reply, len, timeout, terminators, 0, 0);

    if (status != SS$_NORMAL)
      return vmsError (h, "sys$qiow", status);

    if (iosb.iosb$w_status != SS$_NORMAL) {
      if (iosb.iosb$w_status != SS$_TIMEOUT)
	return vmsError (h, "sys$qiow", iosb.iosb$w_status);
      else if (iosb.iosb$w_bcnt == 0)
	return errTIMEOUT (h);
      else {
	*bytesRead = iosb.iosb$w_bcnt;
	h->timeoutFlag = 1;
      }
    }
    else {
      if (iosb.iosb$w_bcnt != len)
	return errINTERR (h);
      *bytesRead = len;
    }

#elif defined WINNT

    if (! ReadFile (h->file, reply, len, bytesRead, NULL))
      return winntError (h, "ReadFile");
    if (*bytesRead == 0) return errTIMEOUT (h);

#elif defined UNIX

    struct timeval timeout;
    int fd = h->fd;
    int status;
    int nread;

    timeout.tv_sec = h->timeout;
    timeout.tv_usec = 0;
    FD_SET ((unsigned) fd, &h->reads);
    status = select (fd + 1, &h->reads, NULL, NULL,  &timeout);
    if (status < 0) return unixError (h, "select");
    if (status == 0) return errTIMEOUT (h);
    if (! FD_ISSET(fd, &h->reads)) return errINTERR (h);
    nread = read (fd, reply, len);
    if (nread < 0) return unixError (h, "read");
    if (nread == 0) return errEOF (h);
    *bytesRead = nread;

#else
    what else ?;
#endif
  }

  return SUCCESS;
}

/*
 *********************************************************************
 */

void slFlushAll(sl_t *h) {
  unsigned bytesRead = 1;
  char reply[200];
  int status = 1;
#if defined WINNT
  COMMTIMEOUTS timeouts;
  COMMTIMEOUTS old_timeouts;



  GetCommTimeouts(h->file, &old_timeouts);

  memset (&timeouts, 0, sizeof timeouts);
  timeouts.ReadIntervalTimeout = 1;
  timeouts.ReadTotalTimeoutConstant = 1;
  if (! SetCommTimeouts (h->file, &timeouts))
    winntError (h, "SetCommTimeouts");

  while(status && bytesRead != 0) {
/*     vsys_msg_printf(0, "Into function"); */
    status = ReadFile (h->file, reply, 1, &bytesRead, NULL);
/*     vsys_msg_printf(0, "Out of function with reply %x %d", reply[0], bytesRead); */
  }
  if (! SetCommTimeouts (h->file, &old_timeouts))
    winntError (h, "SetCommTimeouts");


#endif
}

int slWriteAll (sl_t *h, void *msg, unsigned length) {

  {
#ifdef VMS

    int status;
    iosb iosb;

#if bob
    union ttdef ttymode[4];
    if (s->delays.rts) {
      memset(ttymode, 0, sizeof(ttymode));
      ttymode[1].tt$v_ds_rts = 1;
      cc = sys$qiow(h->ef, vp->serial.fd,
		    IO$_SETMODE | IO$M_SET_MODEM | IO$M_MAINT, 
		    ttymode, 0, 0, 0, 0);
      vsys_wait(s->delays.rts);
    }
#endif
    h->timeoutFlag = 0;
    status = sys$qiow (h->ef, h->chan, 
		       IO$_WRITELBLK | IO$M_BREAKTHRU | IO$M_NOFORMAT,
		       &iosb, 0, 0, msg, length, 0, 0, 0, 0);

    if (status != SS$_NORMAL)
      return vmsError (h, "sys$qiow", status);

    if (iosb.iosb$w_status != SS$_NORMAL)
      return vmsError (h, "sys$qiow", iosb.iosb$w_status);

    if (iosb.iosb$w_bcnt != length)
      return errINTERR (h);
#if bob
    if (s->delays.rts) {
      memset(ttymode, 0, sizeof(ttymode));
      ttymode[0].tt$v_ds_rts = 1;
      cc = sys$qiow(h->ef, vp->serial.fd,
		    IO$_SETMODE | IO$M_SET_MODEM | IO$M_MAINT, 
		    ttymode, 0, 0, 0, 0);
    }
#endif

#elif defined WINNT

    unsigned nwritten;

#if bob
    if (s->delays.rts) {
      EscapeCommFunction(vp->serial.port, SETRTS);
      Sleep(vp->serial.delays.rts);
    }
#endif

    if (WriteFile (h->file, msg, length, &nwritten, NULL) == 0)
      return winntError (h, "WriteFile");

    if (length != nwritten)
      return errINTERR (h);

#if bob
    if (s->delays.rts)
      EscapeCommFunction(vp->serial.port, CLRRTS);
#endif

#elif defined UNIX

    int fd = h->fd;

#if bob
    if (s->delays.rts) {
      int status;
      ioctl(fd, TIOCMGET, &status);
      status |= TIOCM_RTS;
      ioctl(fd, TIOCMSET, &status);
      vsys_wait(s->delays.rts);
    }
#endif

    (void) tcflush (fd, TCIFLUSH); /* Flush input side */
    while (length) {
      int n = write (fd, msg, length);
      if (n <= 0) {
	if (n < 0) return unixError (h, "write");
	return errINTERR (h);
      }
      msg = (VINT8*) msg + n;
      length -= n;
    }
#if 0
    if (0) tcflush (fd, TCOFLUSH); /* FIXME: (JEA) I commented this because
				      otherwise data get lost (on solaris,
				      at least). */
#endif
#if bob
    if (s->delays.rts) {
      int status;
      ioctl(fd, TIOCMGET, &status);
      status &= ~TIOCM_RTS;
      ioctl(fd, TIOCMSET, &status);
    }
#endif

#else
    what else ?;
#endif
  }

  return SUCCESS;
}

/*
 *********************************************************************
 */

int slOpen (sl_t *h, char *name) {

  if (h->opened)
    (void) slClose (h);

  if (h->delays.rts != 0 || h->delays.turnaround != 0)
    return errFIXME (h);

  {
#if defined VMS

    $DESCRIPTOR (desc, "");
    int status;
  
    desc.dsc$a_pointer = name;
    desc.dsc$w_length= strlen (name);
    
    status = sys$assign (&desc, &h->chan, 0, 0);
    if (! (status & STS$M_SUCCESS))
      return vmsError (h, "sys$assign", status);

    status = lib$get_ef (&h->ef);
    if (! (status & STS$M_SUCCESS)) {
      sys$dassgn (h->chan);
      return vmsError (h, "lib$get_ef", status);
    }

#elif defined WINNT

    COMMTIMEOUTS timeouts;
    vtime timeout;
    DCB dcb;

    if ((h->file = CreateFile (name, GENERIC_READ | GENERIC_WRITE,
			       0, 0, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL,
			       0)) == INVALID_HANDLE_VALUE)
      return winntError (h, "CreateFile");

    h->opened = 1;

    if (! GetCommState (h->file, &dcb))
      return winntError (h, "GetCommState");

    if (h->baud)
      dcb.BaudRate = h->baudSym;

    dcb.ByteSize = 8;

    switch (h->parity) {
    case SL_PARITY_EVEN:
      dcb.Parity = EVENPARITY;
      break;
    case SL_PARITY_ODD:
      dcb.Parity = ODDPARITY;      
      break;
    case SL_PARITY_NONE:
      dcb.Parity = NOPARITY;      
      break;
    }

    switch (h->stop) {
    case 1:
      dcb.StopBits = ONESTOPBIT;
      break;
    case 2:
      dcb.StopBits = TWOSTOPBITS;
      break;
    }

    if (! SetCommState (h->file, &dcb))
      return winntError (h, "SetCommState");

    if (getIntervalTimeout (h, dcb.BaudRate, &timeout) != SUCCESS)
      return FAILURE;

    memset (&timeouts, 0, sizeof timeouts);
    timeouts.ReadIntervalTimeout = (timeout.t_sec * 1000 +
				    (timeout.t_nsec / 1000000));
    timeouts.ReadTotalTimeoutConstant = h->timeout * 1000;
    if (! SetCommTimeouts (h->file, &timeouts))
      return winntError (h, "SetCommTimeouts");

#if bob
    EscapeCommFunction(s->port, SETDTR);
#endif

#elif defined UNIX

    struct termios t;
    int fd;

    if (strncmp (name, "/dev/", 5) != 0)
      return errINVDEV (h);
    fd = h->fd = open (name, O_RDWR | O_NOCTTY, 0);
    if (fd <= 0)
      return unixError (h, "open");
    if (ioctl (fd, TIOCEXCL, 0) != 0)
      return unixError (h, "TIOCEXCL");
    if (tcgetattr (fd, &t) < 0)
      return unixError (h, "tcgetattr");
    t.c_iflag &= ~(IGNBRK | BRKINT | PARMRK | ISTRIP | INLCR |
			     IGNCR | ICRNL | IXON);
    t.c_oflag &= ~OPOST;
    t.c_lflag &= ~(ECHO | ECHONL | ICANON | ISIG | IEXTEN);
    t.c_cflag &= ~(CSIZE | PARENB);
    t.c_cflag |= CS8;
#if bob /* FIXME: I am not sure FNDELAY is a good idea */
    if (fcntl (fd, F_SETFL, FNDELAY))
      return unixError (h, "fcntl");
#endif
    
    switch (h->parity) {
    case SL_PARITY_EVEN:
      t.c_cflag &= ~PARODD;
      t.c_cflag |= PARENB;
      break;
    case SL_PARITY_ODD:
      t.c_cflag |= PARODD | PARENB;
      break;
    case SL_PARITY_NONE:
      t.c_cflag &= ~PARENB;
      break;
    }
    
    switch (h->stop) {
    case 1:
      t.c_cflag &= ~CSTOPB;
      break;
    case 2:
      t.c_cflag |= CSTOPB;
      break;
    }
    
    if (h->baud != 0) {
      (void) printf ("bs = %d\n", h->baudSym);
      if (cfsetospeed (&t, h->baudSym) < 0)
	return unixError (h, "cfsetospeed");
      if (cfsetispeed (&t, h->baudSym) < 0)
	return unixError (h, "cfsetispeed");
    }

    if (tcflush (fd, TCIFLUSH) < 0)
      return unixError (h, "tcflush");
    if (tcsetattr (fd, TCSANOW, &t) < 0)
      return unixError (h, "tcsetattr");

#else

    not implemented yet;

#endif
  }

  return SUCCESS;
}

/*
 *********************************************************************
 */

int slClose (sl_t *h) {
  if (h->opened) {
#ifdef WINNT
    CloseHandle (h->file);
#elif defined UNIX
    (void) close (h->fd);
#else
    lib$free_ef (&h->ef);
    sys$dassgn (h->chan);
#endif
    h->opened = 0;
  }
  return SUCCESS;
}

/*
 *********************************************************************
 */

int slSetBaud (sl_t *h, int baud) {
  int baudSym;
  if (baudToBaudSym (h, baud, &baudSym) != SUCCESS)
    return FAILURE;
  else {
    h->baud = baud;
    h->baudSym = baudSym;
    return SUCCESS;
  }
}

/*
 *********************************************************************
 */

int slSetDebug (sl_t *h, int value) {
  h->debug = value;
  return SUCCESS;
}

/*
 *********************************************************************
 */

int slSetParity (sl_t *h, int value) {
  h->parity = value;
  return SUCCESS;
}

/*
 *********************************************************************
 */

int slSetRts (sl_t *h, int value) {
  h->delays.rts = value;
  return SUCCESS;
}

/*
 *********************************************************************
 */

int slSetStop (sl_t *h, int value) {
  h->stop = value;
  return SUCCESS;
}

/*
 *********************************************************************
 */

int slSetTurnaround (sl_t *h, int value) {
  h->delays.turnaround = value;
  return SUCCESS;
}

/*
 *********************************************************************
 */

int slSetTimeout (sl_t *h, unsigned value) {
  h->timeout = value;
  return SUCCESS;
}

/*
 *********************************************************************
 */

int slGetBaud (sl_t *h, int *value) {
#ifdef WINNT
  DCB dcb;
  if (! GetCommState (h->file, &dcb))
    return winntError (h, "GetCommState");
  if (baudSymToBaud (h, dcb.BaudRate, value) != SUCCESS)
    return FAILURE;
#elif defined VMS
  return errFIXME (h);
#elif defined UNIX
  struct termios s;
  int ispeed, ospeed;
  if (tcgetattr (h->fd, &s) != 0)
    return unixError (h, "tcgetattr");
  ispeed = cfgetispeed (&s);
  ospeed = cfgetospeed (&s);
  if (ispeed != ospeed)
    *value = 0;
  else if (baudSymToBaud (h, ispeed, value) != SUCCESS)
    return FAILURE;
#else
  what else ?;
#endif
  return SUCCESS;
}

/*
 *********************************************************************
 */

int slGetParity (sl_t *h, int *value) {
#ifdef WINNT
  DCB dcb;
  if (! GetCommState (h->file, &dcb))
    return winntError (h, "GetCommState");
  switch (dcb.Parity) {
  case EVENPARITY: *value = SL_PARITY_EVEN; break;
  case ODDPARITY: *value = SL_PARITY_ODD; break;
  case NOPARITY: *value = SL_PARITY_NONE; break;
  default: *value = 0; break;
  }
#elif defined VMS
  return errFIXME (h);
#elif defined UNIX
  struct termios s;
  if (tcgetattr (h->fd, &s) != 0)
    return unixError (h, "tcgetattr");
  if (s.c_cflag & PARENB)
    *value = s.c_cflag & PARODD ? SL_PARITY_ODD : SL_PARITY_EVEN;
  else
    *value = SL_PARITY_NONE;
#else
  what else ?;
#endif
  return SUCCESS;
}

/*
 *********************************************************************
 */

int slGetStop (sl_t *h, int *value) {
#ifdef WINNT
  DCB dcb;
  if (! GetCommState (h->file, &dcb))
    return winntError (h, "GetCommState");
  switch (dcb.StopBits) {
  case ONESTOPBIT: *value = 1; break;
  case TWOSTOPBITS: *value = 2; break;
  default: *value = 0; break;
  }
#elif defined VMS
  return errFIXME (h);
#elif defined UNIX
  struct termios s;
  if (tcgetattr (h->fd, &s) != 0)
    return unixError (h, "tcgetattr");
  *value = s.c_cflag & CSTOPB ? 2 : 1;
#else
  what else ?;
#endif
  return SUCCESS;
}

/*
 *********************************************************************
 */

char * slStrerror (sl_t *h) {
  return h->errstr;
}

/*
 *********************************************************************
 */

sl_t* slInit (void) {

  sl_t* h = (sl_t*)VMEM_MALLOC0 (sizeof *h);

  /* Timeouts used to be hardcoded. When I made them configurable, I kept
     the defaults. Where do these defaults come from - I have no idea!
     (EZ 7-Oct-2004). */

#ifdef VMS
  h->timeout = 2;
#elif defined WINNT
  h->timeout = 4;
#else
  h->timeout = 5;
#endif

  return h;
}

/*
 *********************************************************************
 */

void slDestroy (sl_t *h) {
  if (h != NULL) {
    (void) slClose (h);
    VMEM_FREE (h);
  }
}
