/********************************************************************************/
/*    H   H Y   Y TTTTT EEEEE  CCC       HYTEC ELECTRONICS LTD                  */
/*    H   H  Y Y    T   E     C          5 Cradock Road,                        */
/*    HHHHH   Y     T   EEE   C          Reading, Berks.    Tel: 0118 9757770	*/
/*    H   H   Y     T   E     C          RG2 0JT            Fax: 0118 9757566	*/
/*    H   H   Y     T   EEEEE  CCC       Web: www.hytec-electronics.co.uk       */
/********************************************************************************/
/*                                                                              */
/*			Linux PROJECT	--- Hytec Sky Scanner Testing Program				*/
/*                                                                              */
/********************************************************************************/
/*  Source file name      :-      TipTest.c										*/
/*  Definition header file:		  HYTECIPAPI.h									*/
/*								  Hy8402.h										*/
/*								  Hy85056.h										*/
/*								  Hy85156.h										*/
/*								  HyTip120.h									*/
/*  Platform			  :		  Linux, Windows XP, Windows 2000               */
/*  Initial creation date :-      01-Feb-2010                                   */
/*                                                                              */
/*  Original Developers   :-      Jim S Chen                                    */
/********************************************************************************/
/*                                                                              */
/*  Description :-                                                              */
/*                                                                              */
/*  This is for testing Hytec IP card API library for Sky Scanner job. It 		*/
/*	inlcudes testing IP cards such as 8402, 8505/8506, 8515/8516 and Tip120		*/
/*                                                                              */
/*  Uncomment each macro define on the top that associates to the particular 	*/
/*	IP card to run the test.													*/
/*																				*/
/*  (C)2010 Hytec Electronics Ltd.												*/
/*                                                                              */
/********************************************************************************/
/*                                                                              */
/*  Modification record :-                                                      */
/*                                                                              */
/*  Date modified |     Person    |   Description of modification               */
/* ---------------|---------------|-------------------------------------------- */
/*    17/01/10    |	  Jim S Chen  |	  Initial copy								*/
/*                |               |                                             */
/*                                                                              */
/********************************************************************************/

//#define TEST_ID
#define TEST_TIP120

#include "../include/HytecIPAPI.h"
#include "../include/HyTip120.h"
#include "../include/Hy85056.h"
#include "../include/Hy85156.h"
#include "../include/Hy8402.h"

#include <pthread.h>
#include <math.h>

#define AZIMUTH 0
#define ELEVATION 1

// global because its function need to be called by interrupt callback function which is run in another thread
CHyTip120 tip120;							//global motor controller
CHy8402 DAC2;								//global DAC for azimuth reference
CHy85056 DIO1;								//global digital controller
CHy85156 COM1;								//global serial controller
int serialPort = 0;							//serial port number
int channel = 0;		//axis
EN_VME_INT_INFO int_info;					//interrupt structure that holds the interrupt information

// local functions
void printTip120Status(int status);			//print status in readable text
int readStatus(int times, int channel);
void displaymenu(void);
int isNumber(char n);
int FetchNumber(char *cstart);

// interrupt callback function for all IP cards
void callback(void *drv)
{
	int i;
	int status;
	int intreason;
	int whichInterrupt = 0;
	USHORT data;

//	printf("\n************Interrupted**************\n");

	EN_VME_INT_INFO *d = (EN_VME_INT_INFO *)drv;
//	printf("number of interrupt=%d\n", d->numOfInts);
	if (d->numOfInts > 0)
	{
		for(i=0; i<(int)d->numOfInts; i++)
		{
//			printf("vector=%x\n", d->vectors[i]);
			switch(d->vectors[i])
			{
				case 0x88:															//the interrupt is from tip120
				case 0x89:															//the interrupt is from tip120
                    if(d->vectors[i] == 0x88)
                        channel = AZIMUTH;
                    else
                        channel = ELEVATION;
//					printf("vector=%x\n", d->vectors[i]);
					status = tip120.HyTip120_INTReason(channel, &intreason);				//get interrupt reason 
					if (status !=0)
					{
						printf("Get interrupt reason error. %x \n",status);
					}else
					{
						printf("Reason=%x\n",intreason);
						whichInterrupt |= 0x01;											//rise the flag to tell to clear interrupt at the end of this ISR

						//input 1 (negative limit switch hit) interrupt
						if (intreason & INT_INPUT1)				
						{
							printf("Home limit switch is hit.\n");
							status = tip120.HyTip120_Stop(channel, STOP_SMOOTHLY);		//stop motor immediately
							if (status !=0) printf("Stop motor error. %x \n",status);
							status = tip120.HyTip120_DefineHome(channel);					//define home
							if (status !=0) printf("Define home error. %x \n",status);
						}

						//input 2 (home limit switch hit) interrupt
						if (intreason & INT_INPUT2)				
						{
							printf("SW_B.\n");
							status = tip120.HyTip120_Stop(channel, STOP_SMOOTHLY);		//stop motor immediately
							if (status !=0) printf("Stop motor error. %x \n",status);
						}

						//input 3 (positive limit switch hit) interrupt
						if (intreason & INT_INPUT3)				
						{
							printf("SW_U.\n");
							status = tip120.HyTip120_Stop(channel, STOP_SMOOTHLY);		//stop motor immediately
							if (status !=0) printf("Stop motor error. %x \n",status);
						}

						//excessive error interrupt
						if (intreason & INT_POSITIONERROR)				
						{
							//status = tip120.HyTip120_Stop(channel, STOP_SMOOTHLY);		//stop motor immediately
							printf("Excessive position error and motor stopped.\n");
						}

						//index interrupt
						if (intreason & INT_INDEXPULSE)				
						{
							if(channel == AZIMUTH)
								tip120.HyTip120_SetIndex(AZIMUTH);
							printf("Index passed.\n");
						}

						//trajectory completed interrupt
						if (intreason & INT_TRAJECTORYCOMPLETE)	
						{
//							printf("Motion finished.\n");
						}
					}
					break;
				case 0x18:						// 8505 interrupt
					whichInterrupt |= 0x02;
					status = DIO1.Hy85056_Read(&data, 0xFFFF);				//read all IOs
					if (status !=0)
					{
						printf("Get digital IO error. %d \n",status);
					}else
					{
						printf("8505 Input value=%x \n", data);
					}
					break;
				case 0x15:						// 8515 interrupt
					whichInterrupt |= 0x04;
					status = COM1.Hy85156_INTReason(&serialPort, &intreason);
					if (status !=0)
					{
						printf("Get intreason error. %d \n",status);
					}else
					{
						if(intreason & INTERRUPT_REASON_LASER_READY)
					    	printf("8515 Laser ready!. reason=%x port=%d\n", intreason, serialPort);
					}
					break;
			}
		}
		if(whichInterrupt & 0x01)
		{
            intreason = 0x77E;
			status = tip120.HyTip120_ResetInterrupt(channel, intreason);
		}
		if(whichInterrupt & 0x02)
		{
			status = DIO1.Hy85056_ResetInterrupt();					//reset interrupt
			if (status !=0) printf("Reset interrupt error. %x \n",status);
		}
		if(whichInterrupt & 0x04)
		{
			status = COM1.Hy85156_ResetInterrupt(serialPort, intreason);	//reset interrupt
			if (status !=0) printf("Reset interrupt error. %x \n",status);
		}
	}else
    {
            channel = AZIMUTH;
            intreason = 0x77E;
			status = tip120.HyTip120_ResetInterrupt(channel, intreason);
            channel = ELEVATION;
			status = tip120.HyTip120_ResetInterrupt(channel, intreason);
    }	
}


int main(void)
{
	int i=0, j, k;
	int vmeslot=3;        
	int status=0;

	printf("\nTesting Program starts...... \n\n");

//====================TIP120 test example===========================
//              Assume Carrier card is in VME slot 3			  ||
//					   Serial 8515 IP is in slot A                ||
//					   Digital 8505 IP is in slot B               ||
//					   DAC 8402 IP is in slot C                   ||
//					   TIP120 IP is in slot D                     ||
//==================================================================

#ifdef TEST_TIP120
 
	char IPslot='D';
	int intLevel = 6;
	int vector = 0x88;		//vector for tip120 IP card
	int mode = 1;			//mode=0 for position. =1 for speed control

	printf("************* Motion General Configuration *****************\n");
	printf("- vmeslot=%d, ipslot=%c, INT level=%d, Vector=%x\n", vmeslot, IPslot, intLevel, vector);
	status = tip120.HyTip120_Configure(vmeslot, IPslot, intLevel, vector, &callback, &int_info);
	if (status !=0)
	{
		printf("Configuration error! -- %d\n", status);
		return 0;	
	}
	printf("\n");

	printf("******************* Azimuth Configuration *****************\n");
	int INTMask = INT_INDEXPULSE;
	channel = AZIMUTH;

	printf("- Set up azimuth to speed control.\n");
	printf("- Set up index interrupt.\n");
	status = tip120.HyTip120_MotorSetup(AZIMUTH, mode, INTMask, 0, 0, 0);
	if (status !=0)
	{
		printf("Setup azimuth error! -- %d\n", status);
		return 0;	
	}
	printf("- Enable interrupt.\n");
	status = tip120.HyTip120_EnableInterrupt(AZIMUTH);
	if (status !=0)
	{
		printf("Enable interrupt failed! -- %d\n", status);
	}
	printf("- Enable amplifier.\n");
	 status = tip120.HyTip120_EnableDriver(AZIMUTH, 1);
	 if (status !=0)
	 {
		 printf("Enable amplifier failed! -- %d\n", status);
	 }
	printf("\n");

	printf("******************* Elevation Configuration *****************\n");
	INTMask = INT_INDEXPULSE | INT_INPUT1 | INT_INPUT2 | INT_INPUT3 | INT_TRAJECTORYCOMPLETE;
	channel = ELEVATION;
	mode = 0;			//mode=0 for position. =1 for speed control

	printf("- Set up elevation to position mode.\n");
	printf("- Set up index, both limit switch interrupts.\n");
	status = tip120.HyTip120_MotorSetup(ELEVATION, mode, INTMask, 0x07, 70, 0);
	if (status !=0)
	{
		printf("Setup elevation error! -- %d\n", status);
		return 0;	
	}
	printf("- Enable interrupt.\n");
	status = tip120.HyTip120_EnableInterrupt(ELEVATION);
	if (status !=0)
	{
		printf("Enable interrupt failed! -- %d\n", status);
	}
	printf("- Enable amplifier.\n");
	status = tip120.HyTip120_EnableDriver(ELEVATION, 1);
	if (status !=0)
	{
		printf("Enable amplifier failed! -- %d\n", status);
	}
	printf("\n");

	usleep(500000);		/* rest for a while */

/*
	printf("******************* Hy8505 Configuration *****************\n\n");
    int debrate = 0;
    USHORT debmask = 0xFFFF;
    int scanrate = 0;
    USHORT intmask = 0xFFFF;			//allow interrupt for all
    int pwidth = 0;
    USHORT pmask = 0;
    int clock = 0;
    int dir = 0x00;						//as inputs and invert
	IPslot = 'B';
	vector = 0x18;		//vector for digital 8505 IP card
	status = DIO1.Hy85056_Configure(vmeslot, IPslot, debrate, debmask, 
                scanrate, dir, vector, intmask, pwidth, pmask, clock,
				intLevel, &callback, &int_info);
	if (status !=0)
	{
		printf("Configuration error! -- %d\n", status);
		return 0;	
	}
	status = DIO1.Hy85056_EnableInterrupt();

	printf("************** Hy8505 Configuration Completed************\n\n");

	usleep(500000);
*/


	printf("**************** Laser Control Configuration 8515************\n");
    int delay = 0;
    ULONG baud = 9600;
    char parity = 'N'; 
    int stopbits = 1;
    int databits = 8;
    char flowctl = 0;
	IPslot = 'A';
	vector = 0x15;		//vector for serial 8515 IP card
	printf("- vmeslot=%d, ipslot=%c, INT level=%d, Vector=%x\n", vmeslot, IPslot, intLevel, vector);
	status = COM1.Hy85156_Configure(vmeslot, IPslot, vector, delay, intLevel, &callback, &int_info);
//	status = COM1.Hy85156_Configure(vmeslot, IPslot, vector, delay, intLevel, NULL, NULL);
	if (status < 0)
	{
		printf("Configure 8515 for COM1 error! %d\n", (int) status);
		return 0;	
	}

	//setup com1 port 0 
	printf("- baud=%d, parity=%c, stop=%d, data=%d, flowctl=%d\n", (int)baud, parity, stopbits, databits, flowctl);
	status = COM1.Hy85156_ComsSetup(serialPort, baud, parity, stopbits, databits, flowctl);
	if (status < 0)
	{
		printf("Setup COM1 error! %d\n", (int) status);
		return 0;	
	}
    
	printf("- Enable interrupt.\n");
    status = COM1.Hy85156_EnableInterrupt(serialPort, ENABLE_MODEM_INT_85156, 0, 0);
	if (status < 0)
	{
		printf("Enable interrupt error! %d\n", (int) status);
		return 0;	
	}
	printf("\n");


	printf("******************* DAC8402 Configuration *****************\n");
    char dacIPslot1 = 'C';
    int vectornum2 = 0;
    int doram1 = 0;
    int clockSource1 = 0;
    int dacClockRate1 = 13;
    int inhibit1 = 0;

	printf("- vmeslot=%d, ipslot=%c, INT level=%d, Vector=%x\n", vmeslot, dacIPslot1, intLevel, vectornum2);
	status = DAC2.Hy8402_Configure(vmeslot, dacIPslot1, vectornum2, 
                doram1, clockSource1, dacClockRate1, inhibit1);
	if (status == C_ERR_NODEVICE)
	{
		printf("Driver error!\n");
		return 0;	
	}
	printf("\n");


	char buffer[255];
	char rcvBuf[255];
	int degree, direction;

	status = DAC2.Hy8402_Write(0, 0x8000);
	status = DAC2.Hy8402_Write(1, 0x8000);
	status = DAC2.Hy8402_Write(2, 0x8000);
	status = DAC2.Hy8402_Write(3, 0x8000);	


	// THE BIG LOOP
	displaymenu();
	do
	{
 		scanf("%s",buffer);
		i = strlen((char*)buffer);
		buffer[i] = 0x0d;
		buffer[i+1] = 0;

		switch (buffer[0])
		{
		case 0x31:			//strat azimuth
			direction = FetchNumber(&buffer[2]);
			if(direction != -1)
			{
			     status = tip120.HyTip120_Run(AZIMUTH, 40.0, 0.1, direction);
			     tip120.HyTip120_SetIndex(AZIMUTH);
			     if (status < 0)
			     {
				        printf("Start azimuth error! %d\n", (int) status);
						return 0;	
			     }
            }else
			{
				printf("Input format error! %s\n", buffer);				
			}
			break;
		case 0x32:			//stop azimuth
			status = tip120.HyTip120_Stop(AZIMUTH, 0);
			status = tip120.HyTip120_EnableDriver(AZIMUTH, 0);
			if (status < 0)
			{
				printf("Stop azimuth error! %d\n", (int) status);
				return 0;	
			}
			break;
		case 0x33:			//Move elevation absolute
			degree = FetchNumber(&buffer[2]);
			if(degree != -1)
			{
				status = tip120.HyTip120_Move(ELEVATION, 15.0, 0.1, degree);
				if (status < 0)
				{
					printf("Start azimuth error! %d\n", (int) status);
					return 0;	
				}
		        tip120.HyTip120_SetIndex(ELEVATION);
//				readStatus(10, ELEVATION);			
			}else
			{
				printf("Input format error! %s\n", buffer);				
			}
			break;
		case 0x34:			//fire laser
    		status = COM1.Hy85156_FireLaser(serialPort, 1000);
			if (status < 0)
			{
				printf("Fire Laser error! %d\n", (int) status);
				return 0;	
			}
	        printf("Laser trigger sent!\n");
			break;
		case 0x35:			//send command to LRF
			COM1.Hy85156_Flush(serialPort, 1, 1);
			usleep(10000);
			status = COM1.Hy85156_Write(serialPort, (UCHAR*)&buffer[2], strlen((char*)buffer) - 2);
//		printf("Sent=%s\n length=%d\n", buffer, status);
			rcvBuf[0] = 0;
			k = 0;
			for(i=0; i<5000000; i++)
			{
			    status = COM1.Hy85156_Read(serialPort, (UCHAR*)buffer);
				if(status > 0)
				{
//				printf("number=%d str=%s\n", status, buffer);
					for(j=0; j<status;j++)
						rcvBuf[k++] = buffer[j];
					if(rcvBuf[k-1] == 0x0d)
					{
						rcvBuf[k]=0;
			       		printf("Received=%d: %s\n", k-1, rcvBuf);
						break;
					}
			    }	
			}
			break;
		case 0x36:			//Quit
			status = DAC2.Hy8402_Write(0, 0x8000);
			status = tip120.HyTip120_DisableInterrupt(AZIMUTH);      //these disable must be before de-energising the motors
			status = tip120.HyTip120_DisableInterrupt(ELEVATION);
//			printf("after disable interrupt\n");
			status = tip120.HyTip120_Stop(AZIMUTH, 0);
			status = tip120.HyTip120_Stop(ELEVATION, 0);
			status = tip120.HyTip120_EnableDriver(AZIMUTH, 0);
			status = tip120.HyTip120_EnableDriver(ELEVATION, 0);
//			printf("after disable motors\n");
			return 0;
			break;
		case 0x37:			//Refresh menu
			displaymenu();
			break;
		case 0x38:			//Read status n times
			degree = FetchNumber(&buffer[2]);
			if(degree != -1)
			{
				readStatus(degree, ELEVATION);			
				if (status < 0)
				{
					printf("Read status error! %d\n", (int) status);
					return 0;	
				}
			}else
			{
				printf("Input format error! %s\n", buffer);				
			}
			break;
		}
	}while(1);


//	readStatus(20 , channel);

	printf("TIP120 Test finished. \n\n");

#endif
//==================End of Tip120 test example=========================

//===============Read Identities from IOC Box example===============
#ifdef TEST_ID

	USHORT id_buffer[14];
	USHORT carrier;
	USHORT CSR;
	int j=0;

    //ioc.IOC9010SetCarrierFanSpeed(1,1);
	status = tip120.IOC9010GetCarrier(vmeslot, &carrier);

    printf("Carrier Card is %04X\n",carrier);

    if (carrier == 0x8002)
    {
        printf("Detect IP card installed.....\n\n");
        for (j=0;j<4;j++)
        {
            status = tip120.IOC9010GetConfig(vmeslot, j, 14, id_buffer);
            if (status == C_ERR_NODEVICE)
            {
                printf("Device open failed!");
                break;
            }
            else
            {
                printf("Slot %c IP identity: ", (char) (j+0x41));
                for (i=0;i<14;i++)
    				printf("%X ",id_buffer[i]);
                printf("\n");
            }
	   }
    }
    else
    {
        //printf("This Carrier Has NO IP Cards !\n"); 

        printf("Detect IP card installed.....\n\n");
        for (j=0;j<6;j++)
        {
            status = tip120.IOC9010GetConfig(vmeslot, j, 14, id_buffer);
            if (status == C_ERR_NODEVICE)
            {
                printf("Device open failed!");
                break;
            }
            else
            {
                printf("Slot %c IP identity: ", (char) (j+0x41));
                for (i=0;i<14;i++)
    				printf("%X ",id_buffer[i]);
                printf("\n");
            }
	   }
    }	
	printf("End of program.\n\n");

#endif
//===============Read Identities from IOC Box example===============

	return 0;
}


int readStatus(int times, int channel)
{
	int i;
	ULONG fbdata;
	int status;
	
	//display status and so on for many times
	for(i=0;i<times;i++)
	{
		printf("i=%d, *****Status*****\n", i);
		status = tip120.HyTip120_ReadData(channel, &fbdata, DATA_STATUS);
		if (status !=0)
		{
			printf("Read status error. %x \n",status);
			break;	
		}
		printTip120Status((int)fbdata);
		printf("****************\n\n");
	
		printf("Desired position......");
		status = tip120.HyTip120_ReadData(channel, &fbdata, DATA_DESIRED_POSITION);
		if (status !=0)
		{
			printf("Read desired position error. %d \n",status);
			break;	
		}
		printf("%d\n",(int)fbdata);
		printf("Real position......");
		status = tip120.HyTip120_ReadData(channel, &fbdata, DATA_REAL_POSITION);
		if (status !=0)
		{  
			printf("Read real position error. %x \n",status);
			break;	
		}
		printf("%d\n",(int)fbdata);
		printf("Desired velocity......");
		status = tip120.HyTip120_ReadData(channel, &fbdata, DATA_DESIRED_VELOCITY);
		if (status !=0)
		{
			printf("Read desired velocity error. %d \n",status);
			break;	
		}
		printf("%d\n",(int)fbdata);
		
		printf("Real velocity......");
		status = tip120.HyTip120_ReadData(channel, &fbdata, DATA_REAL_VELOCITY);
		if (status !=0)
		{  
			printf("Read real velocity error. %x \n",status);
			break;	
		}
		printf("%d\n\n",(int)fbdata);

		printf("Real index register......");
		status = tip120.HyTip120_ReadData(channel, &fbdata, DATA_INDEX_POSITION);
		if (status !=0)
		{  
			printf("Read index position error. %x \n",status);
			break;	
		}
		printf("%d\n\n",(int)fbdata);
		usleep(1000000);							//wait 1 mini-second
	}
	return 1;
}
void printTip120Status(int status)
{
	printf("Status=%x\n", status);
	if(status & STATUS_HOST_INTERRUPT) printf("Host interrupted.\n");
	if(status & STATUS_ACCELERATION_LOADED) printf("Acceleration loaded.\n");
	if(status & STATUS_UDF_EXECUTED) printf("UDF command executed.\n");
	if(status & STATUS_FORWARD_DIRECTION) 
		printf("Forward direction.\n");
	else
		printf("Reverse direction.\n");
	if(status & STATUS_VELOCITY_MODE) 
		printf("Velocity mode.\n");
	else
		printf("Position mode.\n");
	if(status & STATUS_ON_TARGET) printf("Position on target.\n");
	if(status & STATUS_OFF_ONERROR) printf("Stop motor if position error.\n");
	if(status & STATUS_8BIT_OUTPUT) 
		printf("8 bit output.\n");
	else
		printf("12 bit output.\n");
	if(status & STATUS_MOTOR_OFF) printf("Motor is off.\n");
	if(status & STATUS_BREAKPOINT) printf("Breakpoint interrupted.\n");
	if(status & STATUS_POSITIONERROR) printf("Position error interrupted.\n");
	if(status & STATUS_WRAPAROUND) printf("Wrap around interrupted.\n");
	if(status & STATUS_INDEXPULSE) printf("Index interrupted.\n");
	if(status & STATUS_TRAJECTORYCOMPLETE) printf("Trajectory completion interrupted.\n");
	if(status & STATUS_COMMANDERROR) printf("Command error interrupted.\n");
}

void displaymenu(void)
{
	printf("*****************Skyscanner Commands****************\n");
	printf("  Note:-for	command 2,4,6, just type the number	    \n");
	printf("  		and return									\n");
	printf("  	   -for	command 1, type the command number 	    \n");
	printf("  	    followed by '=' and direction number. '0'	\n");
	printf("  	    goes one way and '1' goes the other.        \n");
	printf("  	   -for	command 3, type the command number 		\n");
	printf("  	    followed by '=' and the number of degree	\n");
	printf("  	    in absolute position and return.			\n");
	printf("  	   -for	LRF command 5, type 5 followed by '=' 	\n");
	printf("  	    and the command string then return			\n");
	printf("====================================================\n");
	printf("    												\n");
	printf("    1. Start azimuth								\n");
	printf("    2. Stop azimuth 								\n");
	printf("    3. Move elevation (e.g. 3=5, 3=20 and/or 3=15)	\n");
	printf("    4. Fire laser									\n");
	printf("    5. Send command to LRF	(e.g. 6=$LRFT )			\n");
	printf("    6. Quit											\n");
	printf("    7. Reload menu									\n");
	printf("    8. Read status. (e.g. 8=5, 5 times. To change the\n");
	printf("    	motor, change readStatus(degree, ELEVATION);\n");	
	printf("     	ELEVATION to AZIMUTH or vise versa			\n");
	printf("****************************************************\n");
}

int FetchNumber(char *cstart)
{
	int i, l, num=0, c;
	l=strlen(cstart) - 1;
	for(i=0; i<l; i++)
	{
		c = isNumber(*(cstart + i));
		if(c == -1)
			return c;
		num = num +  (int)pow(10, l - i - 1) * c;
	}
	return num;
}

int isNumber(char n)
{
	if(n > 0x39 || n < 0x30)
		return -1;
	else
		return n - 0x30;
}

