/********************************************************************************/
/*    H   H Y   Y TTTTT EEEEE  CCC       HYTEC ELECTRONICS LTD                  */
/*    H   H  Y Y    T   E     C          5 Cradock Road,                        */
/*    HHHHH   Y     T   EEE   C          Reading, Berks.    Tel: 0118 9757770	*/
/*    H   H   Y     T   E     C          RG2 0JT            Fax: 0118 9757566	*/
/*    H   H   Y     T   EEEEE  CCC       Web: www.hytec-electronics.co.uk       */
/********************************************************************************/
/*                                                                              */
/*			IOC9010 PROJECT	--- Hytec IP Card API Library Testing Program		*/
/*                                                                              */
/********************************************************************************/
/*  Source file name      :-      Test8506.c									*/
/*  Definition header file:		  HYTECIPAPI.h									*/
/*								  Hy85056.h										*/
/*  Platform			  :		  Linux, Windows XP, Windows 2000               */
/*  Initial creation date :-      25-NOV-2008                                   */
/*                                                                              */
/*  Original Developers   :-      Mark Woodward                                 */
/********************************************************************************/
/*                                                                              */
/*  Description :-                                                              */
/*                                                                              */
/*  This is a test program to try and reproduce a possible problem between the  */
/*  3331 controller and 8506 ip cards.                                       	*/
/*                                                                              */
/*																				*/
/*  (C)2008 Hytec Electronics Ltd.												*/
/*                                                                              */
/********************************************************************************/
/*                                                                              */
/*  Modification record :-                                                      */
/*                                                                              */
/*  Date modified |     Person    |   Description of modification               */
/* ---------------|---------------|-------------------------------------------- */
/*    25/11/08    |	    MW        |	  Initial copy								*/
/*                |               |                                             */
/*                                                                              */
/********************************************************************************/


#include "../include/HytecIPAPI.h"
#include "../include/Hy85056.h"

#include <pthread.h>
#include <math.h>
#include <sys/time.h>

float sns_inxform[9]; 
float position[3];    /* Transformed positions X,Y,Z*/
float sensor[3] = {1.01, 1.52, 0.45}; 

/* Note: each pointer points to a ROW */
  static float *sns_inxf[3] =  {
    &(sns_inxform[0]),
    &(sns_inxform[3]),
    &(sns_inxform[6])
  };

void init() {
  /* Note: Initialize coefficient matrix */
  sns_inxf[0][0] = 2.0 / 954.0;
  sns_inxf[0][1] = -1.0 / 954.0;
  sns_inxf[0][2] = 1.0 / 550.8;
  sns_inxf[1][0] = 0;
  sns_inxf[1][1] = 1.0 / 550.8;
  sns_inxf[1][2] = -1.0 / 550.8;       
  sns_inxf[2][0] = 1.0 / 3.0;
  sns_inxf[2][1] = 1.0 / 3.0;
  sns_inxf[2][2] = 1.0 / 3.0;
}

int mat_mult() {
  int i;
  float *m, *inval, *outval, accum;
  float dpos[3]; /* Track RMS speed */

  m = sns_inxform;
  outval = position;

  for (i=0; i<3; i++) {
    inval = sensor;
    accum  = (*m++) * (*inval++);
    accum += (*m++) * (*inval++);
    accum += (*m++) * (*inval);
    dpos[i] = accum - *outval;
    *outval++ = accum;
  }
  return 0;
}

int one_div() {
  double val;
  val = M_PI / 42.0;
  return 1;
}

int one_mult() {
  double val;
  val = M_PI * 42.0;
  return 1;
}



int vmeslot = 2;        //ignored by IOC9010
int TRUN[3];

int main(void)
{
	printf("\nIOC9010 floating calculation speed testing starts...... \n\n");


  int loop;
  init();
  int times = 100000000;

  /* simulate 5 KHz */
  timespec ts1, ts2;

  clock_gettime(CLOCK_REALTIME, &ts1);
  for (loop=0; loop<times; loop++) { 
    mat_mult();
    one_mult();
    one_div();
  }
  clock_gettime(CLOCK_REALTIME, &ts2);

  printf("Before: sec.nanosec=%d.%d\n", ts1.tv_sec,ts1.tv_nsec);
  printf("After: sec.nanosec=%d.%d\n", ts2.tv_sec,ts2.tv_nsec);
  if(ts2.tv_nsec < ts1.tv_nsec)
  {
	ts2.tv_sec--;
	ts2.tv_nsec += 1000000000;
  }
  long int sdiff = ts2.tv_sec - ts1.tv_sec;
  long int ndiff = ts2.tv_nsec - ts1.tv_nsec;
  double diff = (double)sdiff * 1000000000 + (double)ndiff;
  printf("Time taken for 100 million times: %f ns\n", diff);
  printf("Time taken: %f ns/loop\n", (double)diff/times);

    return(0);
}
