/********************************************************************************/
/*    H   H Y   Y TTTTT EEEEE  CCC       HYTEC ELECTRONICS LTD                  */
/*    H   H  Y Y    T   E     C          5 Cradock Road,                        */
/*    HHHHH   Y     T   EEE   C          Reading, Berks.    Tel: 0118 9757770	*/
/*    H   H   Y     T   E     C          RG2 0JT            Fax: 0118 9757566	*/
/*    H   H   Y     T   EEEEE  CCC       Web: www.hytec-electronics.co.uk       */
/********************************************************************************/
/*                                                                              */
/*				IOC9010 PROJECT	--- Hytec IP Card API Functions 				*/
/*							for Windows and Linux Platforms	    				*/
/*                                IP Card Type: 8408                            */
/********************************************************************************/
/*  Source file name      :-      Hy8408.h										*/
/*  Definition header file:		  HYTECIPAPI.h									*/
/*  Platform			  :		  Linux, Windows XP, Windows 2000               */
/*  Initial creation date :-      3-AUG-2008                                    */
/*                                                                              */
/*  Original Developers   :-      Darrell Nineham                               */
/********************************************************************************/
/*                                                                              */
/*  Description :-                                                              */
/*                                                                              */
/*  This header file is designed for Hytec API function library of 8408 IP card.*/
/*  It defines the class prototype, error code, masks and registers of 8408.    */
/*																				*/
/*  The design of this interface is cross platforms, which means the code 		*/
/*	can be used in operating systems like Windows, Linux etc without changes.	*/
/*                                                                              */
/*  The language used is C++ but the final product -- the Dynamic Link library	*/
/*	(dll) or the shared library can be used by either C or C++ applications.	*/
/*																				*/
/*  (C)2008 Hytec Electronics Ltd.												*/
/*                                                                              */
/********************************************************************************/
/*                                                                              */
/*  Modification record :-                                                      */
/*                                                                              */
/*  Date modified |     Person    |   Description of modification | Version     */
/* ---------------|---------------|-------------------------------------------- */
/*    03/08/08    |	    DAN       |	  Initial copy				  |	0.0.1		*/
/*    18/10/10    |   KJ          |   Added a support for than 1  |  1.0.0     */
/*    		      |   	          |   device                      |  		    */
/*                |               |                                             */
/*                                                                              */
/********************************************************************************/
#include "HytecIPAPI.h"

#if PLATFORM == 1

#define HYTECAPI_EXPORTS 1
#ifdef HYTECAPI_EXPORTS
#define HYTECAPI_API __declspec(dllexport)
#else
#define HYTECAPI_API __declspec(dllimport)
#endif
#include "C:\Hytec\IOC9010\IOCGeneric\IOCGeneric\IOCGeneric.h"

#else

#include "IOCGeneric.h"

#endif


//error code definition
#define NOT_8408_IP             -8408001
#define CLOCKRATE_INVALID_8408  -8408012
#define CHANNEL_INVALID_8408    -8408013
#define Hy8408_NOT_CONFIGURED   -8408014

//signature definition
#define SIGNATURE_8408      0x8408

//register address defines
#define CSR_8408 			0x0000
#define TRIGGER_LS_8408 	0x0002
#define TRIGGER_MS_8408 	0x0004
#define CLOCK_RATE_8408 	0x0006
#define VECTOR_8408			0x0008
#define FIFO_PRE_8408		0x000A
#define FIFO_POST_8408		0x000C
#define FIFO_COUNT_8408		0x000E
#define FIRST_CHANNEL_8408 	0x0010

//CSR mask definition
#define ARMDAC_8408         0x8000
#define EXT_TRG_8408        0x4000
#define SOFT_TRG_8408       0x2000
#define CLOCKSOURCE_8408    0x1000
#define ENABLE_GO_8408      0x0800
#define EXT_8408            0x0400
#define POST_FF_INT_8408    0x0200
#define PRE_FF_INT_8408     0x0100
#define DMA_8408            0x0080
#define AGO_8408            0x0040
#define IC_8408             0x0020
#define RESET_FIFO_8408     0x0010
#define SPARE_8408          0x0008
#define PRE_EMPTY_8408      0x0004
#define POST_FULL_8408      0x0002
#define PRE_FULL_8408       0x0001


// This class is exported from the HYTECAPI.dll
#if PLATFORM == 1									//windows
class HYTECAPI_API CHy8408 : public CIOCGeneric
#else											//linux
class CHy8408 : public CIOCGeneric
#endif
{
private:
	long m_ErrorCode;						// global error code holder
	int initIPSlot;								// remember the IP slot when configures
	int initVMESlot;

public:
	CHy8408(void);
	CHy8408(char *devName);
	CHy8408(int vmeslot, char ipslot);
	~CHy8408(void);

	long GetErrorCode()
	{
		return m_ErrorCode;
	}
	
/*----------------------------------------------------------------------------------*/
/*Function:		Configure 8408 IP card												*/
/*----------------------------------------------------------------------------------*/
/*Description:	8408 IP configuration routine										*/
/*				according to the settings.											*/
/*Prototype:	long Hy8408_Configure(int vmeslot, char ipslot, int clockRate)		*/
/*Arguements:	vmeslot = VME slot number											*/
/*				ipslot = IP card slot number 0 ~ 3 for VME, 0 ~ 5 for IOC			*/
/*				clockRate = 0 ~ 7, clock rate for output if selected. see manual	*/
/*Return:		status code, 0 = successful. negative = errors						*/
/*----------------------------------------------------------------------------------*/
	long Hy8408_Configure(int vmeslot, char ipslot, int clockRate);

/*----------------------------------------------------------------------------------*/
/*Function:		Read 1 ADC channel of 8408											*/
/*----------------------------------------------------------------------------------*/
/*Description:	Read one channel data from 8408 ADC IP card							*/
/*Prototype:	long Hy8408_Read(int ADC_ch, USHORT *value)							*/
/*Arguements:	ADC_ch = ADC input channel 0 ~ 15									*/
/*				*value = pointer to the 16bit value variable						*/
/*Return:		0 is successfull, negative please see error define					*/
/*----------------------------------------------------------------------------------*/
	long Hy8408_Read(int ADC_ch, USHORT *value);

/*----------------------------------------------------------------------------------*/
/*Function:		Read All channel of 8408 from the FIFO							    */
/*----------------------------------------------------------------------------------*/
/*Description:	Read all channel data from 8408 ADC IP card	FIFO					*/
/*Prototype:	long Hy8408_Read(int ADC_ch, USHORT *value)							*/
/*Arguements:	*value = pointer to the 16bit value array						    */
/*Return:		0 is successfull, negative please see error define					*/
/*----------------------------------------------------------------------------------*/
    long Hy8408_Read(USHORT *value);

};

#if PLATFORM == 1									//windows
extern "C" HYTECAPI_API CHy8408* CreateCHy8408();
extern "C" HYTECAPI_API void DeleteCHy8408( CHy8408* instance );
#else											//linux
extern "C" CHy8408* CreateCHy8408();
extern "C" void DeleteCHy8408( CHy8408* instance );
#endif

