/********************************************************************************/
/*    H   H Y   Y TTTTT EEEEE  CCC       HYTEC ELECTRONICS LTD                  */
/*    H   H  Y Y    T   E     C          5 Cradock Road,                        */
/*    HHHHH   Y     T   EEE   C          Reading, Berks.    Tel: 0118 9757770	*/
/*    H   H   Y     T   E     C          RG2 0JT            Fax: 0118 9757566	*/
/*    H   H   Y     T   EEEEE  CCC       Web: www.hytec-electronics.co.uk       */
/********************************************************************************/
/*                                                                              */
/*				IOC9010 PROJECT	--- Hytec IP Card API Functions 				*/
/*							for Windows and Linux Platforms	    				*/
/*                                IP Card Type: 8411                            */
/********************************************************************************/
/*  Source file name      :-      Hy8411.h										*/
/*  Definition header file:		  HYTECIPAPI.h									*/
/*  Platform			  :		  Linux, Windows XP, Windows 2000               */
/*  Initial creation date :-      11-AUG-2006                                   */
/*                                                                              */
/*  Original Developers   :-      Jim S Chen                                    */
/********************************************************************************/
/*                                                                              */
/*  Description :-                                                              */
/*                                                                              */
/*  This header file is designed for Hytec API function library of 8411 IP card.*/
/*  It defines the class prototype, error code, masks and registers of 8411.    */
/*																				*/
/*  The design of this interface is cross platforms, which means the code 		*/
/*	can be used in operating systems like Windows, Linux etc without changes.	*/
/*                                                                              */
/*  The language used is C++ but the final product -- the Dynamic Link library	*/
/*	(dll) or the shared library can be used by either C or C++ applications.	*/
/*																				*/
/*  (C)2006 Hytec Electronics Ltd.												*/
/*                                                                              */
/********************************************************************************/
/*                                                                              */
/*  Modification record :-                                                      */
/*                                                                              */
/*  Date modified |     Person    |   Description of modification | Version     */
/* ---------------|---------------|-------------------------------------------- */
/*    11/08/06    |	  Jim S Chen  |	  Initial copy				  |	0.0.1		*/
/*    18/10/10    |   KJ          |   Added a support for than 1  |  1.0.0     */
/*    		      |   	          |   device                      |  		    */
/*                |               |                                             */
/*                                                                              */
/********************************************************************************/
#include "HytecIPAPI.h"

#if PLATFORM == 1

#define HYTECAPI_EXPORTS 1
#ifdef HYTECAPI_EXPORTS
#define HYTECAPI_API __declspec(dllexport)
#else
#define HYTECAPI_API __declspec(dllimport)
#endif
#include "C:\Hytec\IOC9010\IOCGeneric\IOCGeneric\IOCGeneric.h"

#else

#include "IOCGeneric.h"

#endif


//error code definition
#define NOT_8411_IP -8411001
#define CLOCKRATE_INVALID_8411 -8411012
#define CHANNEL_INVALID_8411 -8411013
#define Hy8411_NOT_CONFIGURED -8411014

//signature definition
#define SIGNATURE_8411 0x8411

//register address defines
#define CSR_8411 0x0000
#define CLOCK_RATE_8411 0x0006
#define FIRST_CHANNEL_8411 0x0010

// This class is exported from the HYTECAPI.dll
#if PLATFORM == 1									//windows
class HYTECAPI_API CHy8411 : public CIOCGeneric
#else											//linux
class CHy8411 : public CIOCGeneric
#endif
{
private:
	long m_ErrorCode;						// global error code holder
	int initIPSlot;								// remember the IP slot when configures
	int initVMESlot;

public:
	CHy8411(void);
	CHy8411(char *devName);
	CHy8411(int vmeslot, char ipslot);
	~CHy8411(void);

	long GetErrorCode()
	{
		return m_ErrorCode;
	}
	
/*----------------------------------------------------------------------------------*/
/*Function:		Configure 8411 IP card												*/
/*----------------------------------------------------------------------------------*/
/*Description:	8411 IP configuration routine										*/
/*				according to the settings.											*/
/*Prototype:	long Hy8411_Configure(int vmeslot, char ipslot, int clockRate)		*/
/*Arguements:	vmeslot = VME slot number											*/
/*				ipslot = IP card slot number 0 ~ 3 for VME, 0 ~ 5 for IOC			*/
/*				clockRate = 0 ~ 7, clock rate for output if selected. see manual	*/
/*Return:		status code, 0 = successful. negative = errors						*/
/*----------------------------------------------------------------------------------*/
	long Hy8411_Configure(int vmeslot, char ipslot, int clockRate);

/*----------------------------------------------------------------------------------*/
/*Function:		Read 1 ADC channel of 8411											*/
/*----------------------------------------------------------------------------------*/
/*Description:	Read one channel data from 8411 ADC IP card							*/
/*Prototype:	long Hy8411_Read(int ADC_ch, USHORT *value)							*/
/*Arguements:	ADC_ch = ADC input channel 0 ~ 15									*/
/*				*value = pointer to the 16bit value variable						*/
/*Return:		0 is successfull, negative please see error define					*/
/*----------------------------------------------------------------------------------*/
	long Hy8411_Read(int ADC_ch, USHORT *value);
};

#if PLATFORM == 1									//windows
extern "C" HYTECAPI_API CHy8411* CreateCHy8411();
extern "C" HYTECAPI_API void DeleteCHy8411( CHy8411* instance );
#else											//linux
extern "C" CHy8411* CreateCHy8411();
extern "C" void DeleteCHy8411( CHy8411* instance );
#endif

