/********************************************************************************/
/*    H   H Y   Y TTTTT EEEEE  CCC       HYTEC ELECTRONICS LTD                  */
/*    H   H  Y Y    T   E     C          5 Cradock Road,                        */
/*    HHHHH   Y     T   EEE   C          Reading, Berks.    Tel: 0118 9757770	*/
/*    H   H   Y     T   E     C          RG2 0JT            Fax: 0118 9757566	*/
/*    H   H   Y     T   EEEEE  CCC       Web: www.hytec-electronics.co.uk       */
/********************************************************************************/
/*                                                                              */
/*				IOC9010 PROJECT	--- Hytec IP Card API Functions 				*/
/*							for Windows and Linux Platforms	    				*/
/*                                IP Card Type: 8413                            */
/********************************************************************************/
/*  Source file name      :-      Hy8413.h										*/
/*  Definition header file:		  HYTECIPAPI.h									*/
/*  Platform			  :		  Linux, Windows XP, Windows 2000               */
/*  Initial creation date :-      17-AUG-2008                                   */
/*                                                                              */
/*  Original Developers   :-                                                    */
/********************************************************************************/
/*                                                                              */
/*  Description :-                                                              */
/*                                                                              */
/*  This header file is designed for Hytec API function library of 8403 IP card.*/
/*  It defines the class prototype, error code, masks and registers of 8403.    */
/*																				*/
/*  The design of this interface is cross platforms, which means the code 		*/
/*	can be used in operating systems like Windows, Linux etc without changes.	*/
/*                                                                              */
/*  The language used is C++ but the final product -- the Dynamic Link library	*/
/*	(dll) or the shared library can be used by either C or C++ applications.	*/
/*																				*/
/*  (C)2008 Hytec Electronics Ltd.												*/
/*                                                                              */
/********************************************************************************/
/*                                                                              */
/*  Modification record :-                                                      */
/*                                                                              */
/*  Date modified |     Person    |   Description of modification | Version     */
/* ---------------|---------------|-------------------------------------------- */
/*                |	              |	  Initial copy				  |	 0.0.1		*/
/*    29/06/10    |     MW/KJ     |   Added averaging for 8413 	  |  0.0.2      */
/*    18/10/10    |   KJ          |   Added a support for than 1  |  1.0.0     */
/*    		      |   	          |   device                      |  		    */
/*                |               |                                             */
/*                                                                              */
/********************************************************************************/
#include "HytecIPAPI.h"

#if PLATFORM == 1

#define HYTECAPI_EXPORTS 1
#ifdef HYTECAPI_EXPORTS
#define HYTECAPI_API __declspec(dllexport)
#else
#define HYTECAPI_API __declspec(dllimport)
#endif
#include "C:\Hytec\IOC9010\IOCGeneric\IOCGeneric\IOCGeneric.h"

#else

#include "IOCGeneric.h"

#endif


//error code definition
#define NOT_8413_IP -8413001
#define CHANNEL_INVALID_8413 -8413002
#define AITYPE_INVALID_8413 -8413003
#define VECTOR_INVALID_8413 -8413004
#define ITRSTATE_INVALID_8413 -8413005
#define CLOCKSRC_INVALID_8413 -8413006
#define CLOCKRATE_INVALID_8413 -8413007
#define SAMPLES_INVALID_8413 -8413008
#define SPACING_INVALID_8413 -8413009
#define TRIGGER_INVALID_8413 -8413010
#define INHIBIT_INVALID_8413 -8413010
#define AV_DATA_NOT_READY_8413 -8413012
#define Hy8413_NOT_CONFIGURED -8413014

//signature definition
#define SIGNATURE_8413 0x8413

/* A couple of key parameters defining the ADC memory layout. */
#define BANKSIZE    0x4000   /* 8413 - number of samples/bank */
#define CHANNUM     16       /* 8413 - number of ADC channels on card */

#define CSR_8413    		0x0000
#define OFFSET_8413 		0x0002
#define NUMCON_8413 		0x0004
#define CLKRAT_8413 		0x0006
#define VECTOR_8413 		0x0008
#define PreTrigFIFO_8413 	0x000A
#define PostTrigFIFO_8413 	0x000C
#define FIFOCount_8413 		0x000E
#define DATA_8413 			0x0010 /* -> 0x002f */
#define Ref0V_8413    		0x0030
#define Ref2_5V_8413   		0x0032
#define AuxControl_8413		0x0034

/* define bits in the 8401 CSR */
#define CSR_8413_HF      0x0001  /* Set when lower conversion memory filled. */
#define CSR_8413_F       0x0002  /* Set when upper conversion memory filled. */     
#define CSR_8413_CC      0x0004  /* Set when requested sample count captured. */
#define CSR_8413_MIS     0x0008  /* Set when memory update is inhibited. */
#define CSR_8413_RST     0x0010  /* Reset int and ext FIFOs. */
#define CSR_8413_EII     0x0020  /* Set to enable MIS interrupt. */
/* Bit 0x0040 unused. */
#define CSR_8413_1M      0x0080  /* 1 => 64K samples, 0 => 128K samples per chan. */
#define CSR_8413_HE      0x0100  /* Enable interrupt on HF set. */
#define CSR_8413_FE      0x0200  /* Enable interrupt on F set. */
#define CSR_8413_EE      0x0400  /* Enable end of sample sequence interrupt. */
#define CSR_8413_ET      0x0800  /* Allow external enable to control ADC. */
#define CSR_8413_XC      0x1000  /* 0 => use internal clock, 1 => external clock. */
#define CSR_8413_ST      0x2000  /* Set to trigger capture of programmed samples. */
#define CSR_8413_EX      0x4000  /* 0 => continuous capture, 1 => on trigger. */
#define CSR_8413_ARM     0x8000  /* Set to enable capture into waveform RAM. */

#define AUXCSR_8413_RGE      0x0001  /* Set voltage range as +/-5V. */
#define AUXCSR_8413_NS      0x0004  /* Set normal operation. */
#define AUXCSR_8413_2C     0x0008  /* Set normal value format. */

/* Calibration data comes in one of two flavours: either three
 * breakpoints per channel or five breakpoints per channel, depending on
 * the version of the ADC. */
typedef union
{
    struct CAL3PT
    {
        USHORT ngFS;
        USHORT zero;
        USHORT psFS;
    } Cal3pt[CHANNUM];
    struct CAL5PT
    {
        USHORT ngFS;
        USHORT ngHS;
        USHORT zero;
        USHORT psHS;
        USHORT psFS;
    } Cal5pt[CHANNUM];
} CALIBRATION_DATA;


/* The application registers, ID PROM and calibration data are laid out
 * as a contiguous block. */
//typedef struct
//{
//    REGISTERS reg;                              /* 0x00 - 0x1F */
//    char unused[0x80-sizeof(REGISTERS)];        /* 0x20 - 0x80 */
//    ID_PROM prom;                               /* 0x80 - 0x9F */
//    CALIBRATION_DATA cal;                       /* 0xA0 -      */
//} IO_MEMORY;


// This class is exported from the HYTECAPI.dll
#if PLATFORM == 1									//windows
class HYTECAPI_API CHy8413 : public CIOCGeneric
#else											//linux
class CHy8413 : public CIOCGeneric
#endif
{
private:
	long m_ErrorCode;  // global error code holder
	int initIPSlot;    // remember the IP slot when configured
	int initVMESlot;   // remember the VME slot when configured

//    pthread_t readThread;
    int ThreadGo;      // use to control thread 0=stop

    void create_thread();   // creates a new thread and starts it
    friend void *proxy_8413(void*);

    long ADCDATA[16];   // Private Array to hold the 16 channels of ADC data

/* Private Data storage for ADC settings */
//	int Hy8413_vectornum;		// interrupt vector (0 = find one for me),
//	int Hy8413_itrState;		// whether or not interrupts should be enabled at initialisation,
//	int Hy8413_aiType;		// analogue input signal type (0 = diff, 1 = single-ended),
//	int Hy8413_clockSource;	// (0 = internal, 1 = external),
	int Hy8413_clockRate;		// (0 = 1Hz [... 2Hz, 5Hz, 10Hz, 20Hz ...] 15 = 100kHz),
//	int Hy8413_inhibit;		// front panel inhibit (0 = disable, 1 = enable),
	int Hy8413_samples;		// size of sample set for smoothing (1 = use registers),
	int Hy8413_spacing;		// step between samples in sample set (normally set to 1), 
//	int Hy8413_trigger;		// trigger mode (0 = continuous, >0 = triggered by HW or SW),
	int m_inputrange;		/* input range: 0 = +/-10V; 1 = +/-5V */
	int m_valueformat;		/* value format: 0 = 2's complement; 1 = normal 0000~FFFF */
	int m_caliType;			/* calibration type. 0 = none; 1 = 3 point; 2 = 5 point */
	int m_needCali;			/* whether need calibration or not. 0 = no need; 1 = yes */
	CALIBRATION_DATA m_cali_data;

/*----------------------------------------------------------------------------------*/
/*Function:     Periodically read the ADC Data Registers into local variables      	*/
/*----------------------------------------------------------------------------------*/
/*Description:	Private routine to read ADC Data Registers							*/
/*Prototype:	void Hy8403_timedReadThread()          	                            */
/*Arguments		ptr - pointer to thread argument                                    */
/*Return:	                                                						*/
/*----------------------------------------------------------------------------------*/
	void Hy8413_avReadThread();

/*----------------------------------------------------------------------------------*/
/*Function:		Read calibration data from 8413 IP card	(internal function)			*/
/*----------------------------------------------------------------------------------*/
/*Description:	This function populates 8413 calibration data structure by reading 	*/
/*				IDROM as per the range and format settings.							*/
/*Prototype:	long Hy8413_ReadCalibration(int vmeslot, int ipslot, int cc, 		*/
/*							int format)												*/
/*Arguements:	vmeslot = VME slot number											*/
/*				ipslot = IP card slot number 0 ~ 3 for VME, 0 ~ 5 for IOC			*/
/*				range		input range: 0 = +/-10V; 1 = +/-5V 						*/
/*				format		value format: 0 = 2's complement; 1 = normal 			*/
/*Return:		status code, 0 = successful. negative = errors						*/
/*----------------------------------------------------------------------------------*/
	long Hy8413_ReadCalibration(int vmeslot, int ipslot, int range, int format);

/*----------------------------------------------------------------------------------*/
/*Function:		Do calibration for each ADC value	(internal function)				*/
/*----------------------------------------------------------------------------------*/
/*Description:	This function calibrates the ADC value according to stored 			*/
/*				IDROM calibration data											 	*/
/*Prototype:	USHORT Hy8413_Calibrate(int channel, USHORT value) 					*/
/*Arguements:	value = raw data													*/
/*Return:		calibrated raw value if type != 0									*/
/*----------------------------------------------------------------------------------*/
	USHORT Hy8413_Calibrate(int channel, USHORT value);

public:
	CHy8413(void);
	CHy8413(char *devName);
	CHy8413(int vmeslot, char ipslot);
	~CHy8413(void);

	long GetErrorCode()
	{
		return m_ErrorCode;
	}


/*----------------------------------------------------------------------------------*/
/*Function:		Configure 8413 IP card -                       						*/
/*----------------------------------------------------------------------------------*/
/*Description:	8413 IP configuration routine to set up parameters for ADC			*/
/*Prototype:	long Hy8403_Configure(int vmeslot, char ipslot, int vectornum, 		*/
/*				int itrState, int aiType, int clocksource, int clockrate,			*/
/*				int inhibit, int samples, int spacing, int trigger)					*/
/*Arguments:	vmeslot 	- 	VME slot number										*/
/*				ipslot   	-  	IP slot number A ~ D for VME, A ~ F for IOC			*/
/*				vectornum   - 	interrupt vector number. 0=disabled					*/
/*			    itrState 	- 	whether or not interrupts should be enabled at init */
/*				aitype      -   0= differential,									*/
/*								1= single ended,									*/
/*				clocksource -   0=internal, 1=external								*/
/*				clockrate   -   Xilinx scanning freq(Htz)							*/
/*						        legal values - 1,2,5,10,20,50,100					*/
/*				inhibit  	-	front panel inhibit (0 = disable, 1 = enable)		*/
/*				samples		-	number of samples to average for ai 				*/
/*			                     1 means no averaging -use registers				*/
/*			                     0 means use 100 samples and 10 spacing 			*/
/*				spacing		-	spacing between samples to average for ai 			*/
/*  			trigger		-   0 continuous,										*/
/*			                    >0 trigger mode, convert trigger values				*/
/*			                    when triggered by HW or SW.							*/
/*			                    (trigger overrides samples in this case). 			*/
/* 																					*/
/*Return:		status code, 0 = successful. negative = errors						*/
/************************************************************************************/
long Hy8413_Configure(	int vmeslot, int ipslot, int vectornum,
	                    		int itrState, int aitype, int clocksource,
	                    		int clockrate, int inhibit, int samples,
			                    int spacing,  int trigger);

/*----------------------------------------------------------------------------------*/
/*Function:		Configure 8413 IP card	(Overloading)   							*/
/*----------------------------------------------------------------------------------*/
/*Description:	8413 IP configuration routine. This only change the card address.	*/
/*				One has to be very clear what he/she is doing by using this.		*/
/*				Not recommended to use in normal case. It is designed for EPICS 	*/
/*				device support to have one object to control multiple devices		*/
/*				when the devices are already configured by shell script       		*/
/*Prototype:	long Hy8413_Configure(int vmeslot, int ipslot)        				*/
/*Arguements:	vmeslot = VME slot number											*/
/*				ipslot = IP card slot number 0 ~ 3 for VME, 0 ~ 5 for IOC			*/
/*Return:		status code, 0 = successful. negative = errors						*/
/*----------------------------------------------------------------------------------*/
	long Hy8413_Configure(int vmeslot, int ipslot);

/*----------------------------------------------------------------------------------*/
/*Function:		Configure 8413 IP card - (overloading for auxiliary CSR)			*/
/*----------------------------------------------------------------------------------*/
/*Description:	8413 IP configuration routine to set up parameters for ADC			*/
/*Prototype:	long Hy8403_Configure(int vmeslot, char ipslot, int vectornum, 		*/
/*				int itrState, int aiType, int clocksource, int clockrate,			*/
/*				int inhibit, int samples, int spacing, int trigger, int inputrange, */
/*				int optmode, int valueformat)										*/
/*Arguments:	vmeslot 	- 	VME slot number										*/
/*				ipslot   	-  	IP slot number A ~ D for VME, A ~ F for IOC			*/
/*				vectornum   - 	interrupt vector number. 0=disabled					*/
/*			    itrState 	- 	whether or not interrupts should be enabled at init */
/*				aitype      -   0= differential,									*/
/*								1= single ended,									*/
/*				clocksource -   0=internal, 1=external								*/
/*				clockrate   -   Xilinx scanning freq(Htz)							*/
/*						        legal values - 1,2,5,10,20,50,100					*/
/*				inhibit  	-	front panel inhibit (0 = disable, 1 = enable)		*/
/*				samples		-	number of samples to average for ai 				*/
/*			                     1 means no averaging -use registers				*/
/*			                     0 means use 100 samples and 10 spacing 			*/
/*				spacing		-	spacing between samples to average for ai 			*/
/*  			trigger		-   0 continuous,										*/
/*			                    >0 trigger mode, convert trigger values				*/
/*			                    when triggered by HW or SW.							*/
/*			                    (trigger overrides samples in this case). 			*/
/*				inputrange		input range: 0 = +/-10V; 1 = +/-5V 					*/
/*				optmode			operation mode: 0 = standby; 1 = normal				*/
/*				valueformat		value format: 0 = 2's complement; 1 = normal 		*/
/*								0000~FFFF 											*/
/*				needCali		whether need calibration or not: 0 = no need 1 = yes*/
/* 																					*/
/*Return:		status code, 0 = successful. negative = errors						*/
/************************************************************************************/
	long Hy8413_Configure(	int vmeslot, int ipslot, int vectornum,
	                    		int itrState, int aitype, int clocksource,
	                    		int clockrate, int inhibit, int samples,
			                    int spacing,  int trigger, int inputrange,
								int optmode, int valueformat);

/*----------------------------------------------------------------------------------*/
/*Function:		Read 1 ADC channel of 8413											*/
/*----------------------------------------------------------------------------------*/
/*Description:	Read one channel data from 8413 ADC IP card							*/
/*Prototype:	long Hy8413_Read(int ADC_ch, ULONG *value)							*/
/*Arguements:	ADC_ch = ADC input channel 0 ~ 15									*/
/*				*value = pointer to the 16bit value variable						*/
/*Return:		0 is successfull, negative please see error define					*/
/*----------------------------------------------------------------------------------*/
long Hy8413_Read(int ADC_ch, USHORT *value);


/*----------------------------------------------------------------------------------*/
/*Function:		Read Internal FIFO from 8413										*/
/*----------------------------------------------------------------------------------*/
/*Description:	Read entire FIFO from 8413 ADC IP card								*/
/*Prototype:	long Hy8413_Read(int ADC_ch, ULONG *FIFO)							*/
/*Arguements:	*FIFO = pointer to the 16bit FIFO array variable					*/
/*Return:		0 is successfull, negative please see error define					*/
/*----------------------------------------------------------------------------------*/
long Hy8413_Read_INTFIFO(USHORT *FIFO);


/*----------------------------------------------------------------------------------*/
/*Function:		Read External FIFO from 8413										*/
/*----------------------------------------------------------------------------------*/
/*Description:	Read entire external FIFO from 8413 ADC IP card						*/
/*Prototype:	long Hy8413_Read(int ADC_ch, ULONG *FIFO)							*/
/*Arguements:	*FIFO = pointer to the 16bit FIFO array variable					*/
/*Return:		0 is successfull, negative please see error define					*/
/*----------------------------------------------------------------------------------*/
long Hy8413_Read_EXTFIFO(USHORT *FIFO);
};


#if PLATFORM == 1									//windows
extern "C" HYTECAPI_API CHy8413* CreateCHy8413();
extern "C" HYTECAPI_API void DeleteCHy8413( CHy8413* instance );
#else											//linux
extern "C" CHy8413* CreateCHy8413();
extern "C" void DeleteCHy8413( CHy8413* instance );
#endif

