/********************************************************************************/
/*    H   H Y   Y TTTTT EEEEE  CCC       HYTEC ELECTRONICS LTD                  */
/*    H   H  Y Y    T   E     C          5 Cradock Road,                        */
/*    HHHHH   Y     T   EEE   C          Reading, Berks.    Tel: 0118 9757770	*/
/*    H   H   Y     T   E     C          RG2 0JT            Fax: 0118 9757566	*/
/*    H   H   Y     T   EEEEE  CCC       Web: www.hytec-electronics.co.uk       */
/********************************************************************************/
/*                                                                              */
/*				IOC9010 PROJECT	--- Hytec IP Card API Functions 				*/
/*							for Windows and Linux Platforms	    				*/
/*                                IP Card Type: 85056                           */
/********************************************************************************/
/*  Source file name      :-      Hy85056.h										*/
/*  Definition header file:		  HYTECIPAPI.h									*/
/*  Platform			  :		  Linux, Windows XP, Windows 2000               */
/*  Initial creation date :-      11-AUG-2006                                   */
/*                                                                              */
/*  Original Developers   :-      Jim S Chen                                    */
/********************************************************************************/
/*                                                                              */
/*  Description :-                                                              */
/*                                                                              */
/*  This header file is designed for Hytec API function library of 8505 and 	*/
/*	8506 IP card. It defines the class prototype, error code, masks and 		*/
/*	registers of 8505/8506.														*/
/*																				*/
/*  The design of this interface is cross platforms, which means the code 		*/
/*	can be used in operating systems like Windows, Linux etc without changes.	*/
/*                                                                              */
/*  The language used is C++ but the final product -- the Dynamic Link library	*/
/*	(dll) or the shared library can be used by either C or C++ applications.	*/
/*																				*/
/*  (C)2006 Hytec Electronics Ltd.												*/
/*                                                                              */
/********************************************************************************/
/*                                                                              */
/*  Modification record :-                                                      */
/*                                                                              */
/*  Date modified |     Person    |   Description of modification | Version     */
/* ---------------|---------------|-------------------------------------------- */
/*    11/08/06    |	  Jim S Chen  |	  Initial copy				  |	0.0.1		*/
/*    26/03/09    |   JC          |   Added interrupt support     |  0.2.0      */
/*    13/05/09    |   JC          |   Added interrupt enable      |  0.2.1      */
/*                |               |   routine                     |             */
/*    18/10/10    |   KJ          |   Added a support for than 1  |  1.0.0      */
/*    		      |   	          |   device                      |  		    */
/*                |               |                                             */
/********************************************************************************/
#include "HytecIPAPI.h"

#if PLATFORM == 1

#define HYTECAPI_EXPORTS 1
#ifdef HYTECAPI_EXPORTS
#define HYTECAPI_API __declspec(dllexport)
#else
#define HYTECAPI_API __declspec(dllimport)
#endif
#include "C:\Hytec\IOC9010\IOCGeneric\IOCGeneric\IOCGeneric.h"

#else

#include "IOCGeneric.h"

#endif



//error code definition
#define NOT_8505_IP -8505010
#define NOT_8506_IP -8506010
#define Hy85056_NOT_CONFIGURED -85056012
#define DEBOUNCE_RATE_INVALID -85056013
#define SCAN_RATE_INVALID -85056014
#define DIRECTION_INVALID -85056015
#define PAUSE_WIDTH_INVALID -85056016
#define CLOCK_SETTING_INVALID -85056017
#define PORT_NUMBER_INVALID_85056 -85056018
#define WRITE_VIOLATION -85056019
#define BIT_NUMBER_INVALID_85056 -85056020

//signature definition
#define SIGNATURE_8505 0x8505
#define SIGNATURE_8506 0x8506

//register address defines
#define DATA_8505 0x0000
#define CSR_8505 0x0002
#define INTERRUPT_MASK_8505 0x0004
#define DEBOUNCE_MASK_8505 0x0006
#define PAUSE_MASK_8505 0x0008
#define PAUSE_SETTING_8505 0x000A
#define INTERRUPT_VECTOR_8505 0x000C

//mask definition
#define DEBOUNCE_BIT_MASK 0xFFE7
#define SCANRATE_BIT_MASK 0xFF9F
#define DIR_BIT_MASK 0xF3FF
#define CLOCK_BIT_MASK 0xFFF9
#define ENABLE_SCAN 0x0001
#define INVERT_INPUTOUTPUT 0x2000

#define ENABLE_INTERRUPT 0x4000			//added by Jim 26-03-2009 for interrupt implementation
#define CLEAR_INTERRUPT 0x7FFF

#define SIGNAL_INVERT 0x0004

// This class is exported from the HYTECAPI.dll
#if PLATFORM == 1									//windows
class HYTECAPI_API CHy85056 : public CIOCGeneric
#else											//linux
class CHy85056 : public CIOCGeneric
#endif
{
private:
	long m_ErrorCode;							// global error code holder
	int initIPSlot;								// remember the IP slot when configures
	int initVMESlot;
	int IPtype;                                 // remember 8505 or 8506
	int Hy85056_invert;                         // manual invert for 8505 ??
	int m_mask;
	char m_vector;
	int m_INTLevel;

/*----------------------------------------------------------------------------------*/
/*Function:		Private Configure for both 8505 and 8506 IP card					*/
/*----------------------------------------------------------------------------------*/
/*Description:	8505, 6 IP configuration routine									*/
/*				according to the settings.											*/
/*Prototype:	long Hy85056_Config(int vmeslot, char ipslot, int portnum,			*/
/*				int debrate, USHORT debmask, int scanrate, int dir,					*/
/*				iUSHORTnt vectornum, USHORT intmask, int pwidth,					*/
/*				USHORT pmask, int clock, , USHORT signature, USHORT errorcode)		*/
/*Arguements:	vmeslot = VME slot number											*/
/*				ipslot = IP card slot number 0 ~ 3 for VME, 0 ~ 5 for IOC			*/
/*				portnum = port number, 0-2											*/
/*				debrate = Debounce Rate (0=100Hz, 1=200Hz, 2=500Hz, 3=1kHz)			*/
/*				debmask = Debounce Bit Mask, 0x0000-0xFFFF							*/
/*				scanrate = input scan rate (0=1kHz, 1=10kHz, 2=100kHz, 3=1MHz)		*/
/*				dir = The direction of the I/O										*/
/*						0 = All 16 bits inputs.										*/
/*						1 = lower 8 bits outputs / high 8 bits inputs (8505 Only).	*/
/*						2 = lower 8 bits inputs / high 8 bits outputs (8505 Only).	*/
/*						3 = All 16 bits outputs.									*/
/*						Bits 2 & 3 (values 4 and 8, respectively) encode whether	*/
/*						the input and output values should be inverted.				*/
/*						(dir & 4) == 4  =>  Invert all input and output bits		*/
/*				vectornum = Interrupt Vector for this IP Card (= 0, find me one).	*/
/*				intmask = Interrupt Bit Mask, 0x0000 - 0xFFFF. 0x0000 means none	*/
/*				pwidth = pulse width, 0 ~ 9	see nmanual								*/
/*				pmask = Pulse Output Bit Mask, 0x0000 - 0xFFFF						*/
/*				clock = 0 = internal, 1 = external									*/
/*				signature = IP signature, 0x8505 for 8506 and 0x8506 for 8506		*/
/*				errorcode = error code if the IP is not the specified type			*/
/*Return:		status code, 0 = successful. negative = errors						*/
/*----------------------------------------------------------------------------------*/
	long Hy85056_Config(int vmeslot, char ipslot, int portnum, int debrate, USHORT debmask,
							int scanrate, int dir, USHORT vectornum, USHORT intmask,
							int pwidth, USHORT pmask, int clock, USHORT signature, long errorcode);

public:
	CHy85056(void);
	CHy85056(char *devName);
	CHy85056(int vmeslot, char ipslot);
	~CHy85056(void);

	long GetErrorCode()
	{
		return m_ErrorCode;
	}
	
/*----------------------------------------------------------------------------------*/
/*Function:		Enable 8505 interrupt												*/
/*----------------------------------------------------------------------------------*/
/*Description:	Enable interrupt for 8505 IP card          							*/
/*Prototype:	long Hy85056_EnableInterrupt(void)									*/
/*Arguements:	none																*/
/*Return:		0 is successfull, negative please see error define					*/
/*----------------------------------------------------------------------------------*/
    long Hy85056_EnableInterrupt(void);

/*----------------------------------------------------------------------------------*/
/*Function:		Disable 8506 interrupt             									*/
/*----------------------------------------------------------------------------------*/
/*Description:	Disable interrupt for 8506 IP card          					    */
/*Prototype:	long Hy85056_DisableInterrupt(int portnum)							*/
/*Arguements:	portnum, 1 ~ 3														*/
/*Return:		0 is successfull, negative please see error define					*/
/*----------------------------------------------------------------------------------*/
    long Hy85056_DisableInterrupt(int portnum);

/*----------------------------------------------------------------------------------*/
/*Function:		Enable 8506 interrupt (Overloading)									*/
/*----------------------------------------------------------------------------------*/
/*Description:	Enable interrupt for 8506 IP card          							*/
/*Prototype:	long Hy85056_EnableInterrupt(int portnum)							*/
/*Arguements:	none																*/
/*Return:		0 is successfull, negative please see error define					*/
/*----------------------------------------------------------------------------------*/
    long Hy85056_EnableInterrupt(int portnum);

/*----------------------------------------------------------------------------------*/
/*Function:		Reset 8505 interrupt												*/
/*----------------------------------------------------------------------------------*/
/*Description:	Reset interrupt flag bit15 for 8505 IP card							*/
/*Prototype:	long Hy85056_ResetInterrupt(void)									*/
/*Arguements:	none																*/
/*Return:		0 is successfull, negative please see error define					*/
/*----------------------------------------------------------------------------------*/
	long Hy85056_ResetInterrupt(void);

/*----------------------------------------------------------------------------------*/
/*Function:		Reset 8506 interrupt overloading									*/
/*----------------------------------------------------------------------------------*/
/*Description:	Reset interrupt flag bit15 for 8506 IP card							*/
/*Prototype:	long Hy85056_ResetInterrupt(int portnum)							*/
/*Arguements:	portnum = port number												*/
/*Return:		0 is successfull, negative please see error define					*/
/*----------------------------------------------------------------------------------*/
	long Hy85056_ResetInterrupt(int portnum);


/*----------------------------------------------------------------------------------*/
/*Function:		Configure 8505 IP card												*/
/*----------------------------------------------------------------------------------*/
/*Description:	8505 IP configuration routine										*/
/*				according to the settings.											*/
/*Prototype:	long Hy85056_Configure(int vmeslot, char ipslot, int debrate,		*/
/*				USHORT debmask, int scanrate, int dir, iUSHORTnt vectornum,			*/
/*				USHORT intmask,	int pwidth, USHORT pmask, int clock)				*/
/*Arguements:	vmeslot = VME slot number											*/
/*				ipslot = IP card slot number 0 ~ 3 for VME, 0 ~ 5 for IOC			*/
/*				debrate = Debounce Rate (0=100Hz, 1=200Hz, 2=500Hz, 3=1kHz)			*/
/*				debmask = Debounce Bit Mask, 0x0000-0xFFFF							*/
/*				scanrate = input scan rate (0=1kHz, 1=10kHz, 2=100kHz, 3=1MHz)		*/
/*				dir = The direction of the I/O										*/
/*						0 = All 16 bits inputs.										*/
/*						1 = lower 8 bits outputs / high 8 bits inputs (8505 Only).	*/
/*						2 = lower 8 bits inputs / high 8 bits outputs (8505 Only).	*/
/*						3 = All 16 bits outputs.									*/
/*						Bits 2 & 3 (values 4 and 8, respectively) encode whether	*/
/*						the input and output values should be inverted.				*/
/*						(dir & 4) == 4  =>  Invert all input and out put bits		*/
/*				vectornum = Interrupt Vector for this IP Card (= 0, find me one).	*/
/*				intmask = Interrupt Bit Mask, 0x0000 - 0xFFFF. 0x0000 means none	*/
/*				pwidth = pulse width, 0 ~ 9	see nmanual								*/
/*				pmask = Pulse Output Bit Mask, 0x0000 - 0xFFFF						*/
/*				clock = 0 = internal, 1 = external									*/
/*Return:		status code, 0 = successful. negative = errors						*/
/*----------------------------------------------------------------------------------*/
	long Hy85056_Configure(int vmeslot, char ipslot, int debrate, USHORT debmask,
							int scanrate, int dir, USHORT vectornum, USHORT intmask,
							int pwidth, USHORT pmask, int clock);

/*----------------------------------------------------------------------------------*/
/*Function:		Configure 8505 IP card (overloading)								*/
/*----------------------------------------------------------------------------------*/
/*Description:	8505 IP configuration routine										*/
/*				according to the settings.											*/
/*Prototype:	long Hy85056_Configure(int vmeslot, char ipslot, int debrate,		*/
/*				USHORT debmask, int scanrate, int dir, iUSHORTnt vectornum,			*/
/*				USHORT intmask,	int pwidth, USHORT pmask, int clock, 				*/
/*				int INTLevel, void (*callback)(void *), void *drive)				*/
/*Arguements:	vmeslot = VME slot number											*/
/*				ipslot = IP card slot number 0 ~ 3 for VME, 0 ~ 5 for IOC			*/
/*				debrate = Debounce Rate (0=100Hz, 1=200Hz, 2=500Hz, 3=1kHz)			*/
/*				debmask = Debounce Bit Mask, 0x0000-0xFFFF							*/
/*				scanrate = input scan rate (0=1kHz, 1=10kHz, 2=100kHz, 3=1MHz)		*/
/*				dir = The direction of the I/O										*/
/*						0 = All 16 bits inputs.										*/
/*						1 = lower 8 bits outputs / high 8 bits inputs (8505 Only).	*/
/*						2 = lower 8 bits inputs / high 8 bits outputs (8505 Only).	*/
/*						3 = All 16 bits outputs.									*/
/*						Bits 2 & 3 (values 4 and 8, respectively) encode whether	*/
/*						the input and output values should be inverted.				*/
/*						(dir & 4) == 4  =>  Invert all input and output bits		*/
/*				vectornum = Interrupt Vector for this IP Card (= 0, find me one).	*/
/*				intmask = Interrupt Bit Mask, 0x0000 - 0xFFFF. 0x0000 means none	*/
/*				pwidth = pulse width, 0 ~ 9	see nmanual								*/
/*				pmask = Pulse Output Bit Mask, 0x0000 - 0xFFFF						*/
/*				clock = 0 = internal, 1 = external									*/
/*				INTLevel = interrupt level. must be 1 ~ 7							*/
/*				callback = user callback function									*/
/*				drive = user data structure passed to the ISR						*/
/*Return:		status code, 0 = successful. negative = errors						*/
/*----------------------------------------------------------------------------------*/
	long Hy85056_Configure(int vmeslot, char ipslot, int debrate, USHORT debmask,
								 int scanrate, int dir, USHORT vectornum, USHORT intmask,
								 int pwidth, USHORT pmask, int clock,
								 int INTLevel, void (*callback)(void *), void *drive);

/*----------------------------------------------------------------------------------*/
/*Function:		Configure 8506 IP card	(Overloading for 8506)						*/
/*----------------------------------------------------------------------------------*/
/*Description:	8506 IP configuration routine										*/
/*				according to the settings.											*/
/*Prototype:	long Hy85056_Configure(int vmeslot, char ipslot, int portnum,		*/
/*				int debrate, USHORT debmask, int scanrate, int dir,					*/
/*				iUSHORTnt vectornum, USHORT intmask, int pwidth,					*/
/*				USHORT pmask, int clock)											*/
/*Arguements:	vmeslot = VME slot number											*/
/*				ipslot = IP card slot number 0 ~ 3 for VME, 0 ~ 5 for IOC			*/
/*				portnum = port number, 0-2											*/
/*				debrate = Debounce Rate (0=100Hz, 1=200Hz, 2=500Hz, 3=1kHz)			*/
/*				debmask = Debounce Bit Mask, 0x0000-0xFFFF							*/
/*				scanrate = input scan rate (0=1kHz, 1=10kHz, 2=100kHz, 3=1MHz)		*/
/*				dir = The direction of the I/O										*/
/*						0 = All 16 bits inputs.										*/
/*						3 = All 16 bits outputs.									*/
/*						Bits 2 & 3 (values 4 and 8, respectively) encode whether	*/
/*						the input and output values should be inverted.				*/
/*						(dir & 4) == 4  =>  Invert all input and output bits		*/
/*				vectornum = Interrupt Vector for this IP Card (= 0, find me one).	*/
/*				intmask = Interrupt Bit Mask, 0x0000 - 0xFFFF. 0x0000 means none	*/
/*				pwidth = pulse width, 0 ~ 9	see nmanual								*/
/*				pmask = Pulse Output Bit Mask, 0x0000 - 0xFFFF						*/
/*				clock = 0 = internal, 1 = external									*/
/*Return:		status code, 0 = successful. negative = errors						*/
/*----------------------------------------------------------------------------------*/
	long Hy85056_Configure(int vmeslot, char ipslot, int portnum, int debrate, USHORT debmask,
							int scanrate, int dir, USHORT vectornum, USHORT intmask,
							int pwidth, USHORT pmask, int clock);

/*----------------------------------------------------------------------------------*/
/*Function:		Configure 8506 IP card	(Overloading for 8506 and Interrupt)		*/
/*----------------------------------------------------------------------------------*/
/*Description:	8506 IP configuration routine										*/
/*				according to the settings.											*/
/*Prototype:	long Hy85056_Configure(int vmeslot, char ipslot, int portnum,		*/
/*				int debrate, USHORT debmask, int scanrate, int dir,					*/
/*				iUSHORTnt vectornum, USHORT intmask, int clockint INTLevel, 		*/
/*				void (*callback)(void *), void *drive)								*/
/*Arguements:	vmeslot = VME slot number											*/
/*				ipslot = IP card slot number 0 ~ 3 for VME, 0 ~ 5 for IOC			*/
/*				portnum = port number, 0-2											*/
/*				debrate = Debounce Rate (0=100Hz, 1=200Hz, 2=500Hz, 3=1kHz)			*/
/*				debmask = Debounce Bit Mask, 0x0000-0xFFFF							*/
/*				scanrate = input scan rate (0=1kHz, 1=10kHz, 2=100kHz, 3=1MHz)		*/
/*				dir = The direction of the I/O    									*/
/*						0 = All 16 bits inputs.										*/
/*						3 = All 16 bits outputs.									*/
/*						Bits 2 & 3 (values 4 and 8, respectively) encode whether	*/
/*						the input and output values should be inverted.				*/
/*						(dir & 4) == 4  =>  Invert all input and output bits		*/
/*				vectornum = Interrupt Vector for this IP Card (= 0, find me one).	*/
/*				intmask = Interrupt Bit Mask, 0x0000 - 0xFFFF. 0x0000 means none	*/
/*				pwidth = pulse width, 0 ~ 9	see nmanual								*/
/*				pmask = Pulse Output Bit Mask, 0x0000 - 0xFFFF						*/
/*				clock = 0 = internal, 1 = external									*/
/*				INTLevel = interrupt level. must be 1 ~ 7							*/
/*				callback = user callback function									*/
/*				drive = user data structure passed to the ISR						*/
/*Return:		status code, 0 = successful. negative = errors						*/
/*----------------------------------------------------------------------------------*/
	long Hy85056_Configure(int vmeslot, char ipslot, int portnum, int debrate, 
								 USHORT debmask, int scanrate, int dir, 
								 USHORT vectornum, USHORT intmask,
								 int pwidth, USHORT pmask, int clock,
								 int INTLevel, void (*callback)(void *), void *drive);

/*----------------------------------------------------------------------------------*/
/*Function:		Write output by word for 8505										*/
/*----------------------------------------------------------------------------------*/
/*Description:	Write data to the data register of 8505 IP card						*/
/*Prototype:	long Hy85056_Write(USHORT mask, USHORT value)						*/
/*Arguements:	mask = bit mask for the write										*/
/*				*value = pointer to the 16bit value variable which holds the data	*/
/*Return:		0 is successfull, negative please see error define					*/
/*----------------------------------------------------------------------------------*/
	long Hy85056_Write(USHORT mask, USHORT value);

/*----------------------------------------------------------------------------------*/
/*Function:		Write output by word for 8506 (overloading)							*/
/*----------------------------------------------------------------------------------*/
/*Description:	Write data to the data register of 8506 IP card						*/
/*Prototype:	long Hy85056_Write(int portnum, USHORT mask, USHORT value)			*/
/*Arguements:	mask = bit mask for the write										*/
/*				*value = pointer to the 16bit value variable which holds the data	*/
/*Return:		0 is successfull, negative please see error define					*/
/*----------------------------------------------------------------------------------*/
	long Hy85056_Write(int portnum, USHORT mask, USHORT value);

/*----------------------------------------------------------------------------------*/
/*Function:		Write one bit for 8505												*/
/*----------------------------------------------------------------------------------*/
/*Description:	Write one bit to 8505 data register									*/
/*Prototype:	long Hy85056_Write_Bit(int Bit, USHORT value)						*/
/*Arguements:	Bit = bit to write = 0 ~ 15											*/
/*				value = 1 to set, 0 to clear										*/
/*Return:		0 is successfull, negative please see error define					*/
/*----------------------------------------------------------------------------------*/
	long Hy85056_Write_Bit(int Bit, USHORT value);

/*----------------------------------------------------------------------------------*/
/*Function:		Write one bit to 8506 (overloading)									*/
/*----------------------------------------------------------------------------------*/
/*Description:	Write one bit to 8506 data register									*/
/*Prototype:	long Hy85056_Write_Bit(int portnum, int Bit, USHORT value)			*/
/*Arguements:	Bit = bit to write = 0 ~ 15											*/
/*				value = 1 to set, 0 to clear										*/
/*Return:		0 is successfull, negative please see error define					*/
/*----------------------------------------------------------------------------------*/
	long Hy85056_Write_Bit(int portnum, int Bit, USHORT value);
	
/*----------------------------------------------------------------------------------*/
/*Function:		Read status	by word													*/
/*----------------------------------------------------------------------------------*/
/*Description:	Read the last status from the data register of 85056 IP card		*/
/*Prototype:	long Hy85056_Read(USHORT *value, USHORT mask)						*/
/*Arguements:	mask = bit mask for the read										*/
/*				*value = pointer to the 16bit value variable which holds the return	*/
/*Return:		0 is successfull, negative please see error define					*/
/*----------------------------------------------------------------------------------*/
	long Hy85056_Read(USHORT *value, USHORT mask);

/*----------------------------------------------------------------------------------*/
/*Function:		Read status	by word	for 8506 (overloading)							*/
/*----------------------------------------------------------------------------------*/
/*Description:	Read the last status from the data register of 8506 port			*/
/*Prototype:	long Hy85056_Read(int portnum, USHORT *value, USHORT mask)			*/
/*Arguements:	mask = bit mask for the read										*/
/*				*value = pointer to the 16bit value variable which holds the return	*/
/*Return:		0 is successfull, negative please see error define					*/
/*----------------------------------------------------------------------------------*/
	long Hy85056_Read(int portnum, USHORT *value, USHORT mask);

/*----------------------------------------------------------------------------------*/
/*Function:		Read status	by bit													*/
/*----------------------------------------------------------------------------------*/
/*Description:	Read one bit of the last status register of 85056 IP card			*/
/*Prototype:	bool Hy85056_Read_Bit(int Bit)										*/
/*Arguements:	Bit = bit to read, 0 ~ 15											*/
/*Return:		true if bit = 1, false if bit = 0. if invert being set, return 		*/
/*				the inversed														*/
/*----------------------------------------------------------------------------------*/
	bool Hy85056_Read_Bit(int Bit);

/*----------------------------------------------------------------------------------*/
/*Function:		Read status	by bit from 8506 (overloading)							*/
/*----------------------------------------------------------------------------------*/
/*Description:	Read one bit of the last status register of 8506 port				*/
/*Prototype:	bool Hy85056_Read_Bit(int portnum, int Bit)							*/
/*Arguements:	Bit = bit to read, 0 ~ 15											*/
/*Return:		true if bit = 1, false if bit = 0. if invert being set, return 		*/
/*				the inversed														*/
/*----------------------------------------------------------------------------------*/
	bool Hy85056_Read_Bit(int portnum, int Bit);
};

#if PLATFORM == 1									//windows
extern "C" HYTECAPI_API CHy85056* CreateCHy85056();
extern "C" HYTECAPI_API void DeleteCHy85056( CHy85056* instance );
#else											//linux
extern "C" CHy85056* CreateCHy85056();
extern "C" void DeleteCHy85056( CHy85056* instance );
#endif

