/********************************************************************************/
/*    H   H Y   Y TTTTT EEEEE  CCC       HYTEC ELECTRONICS LTD                  */
/*    H   H  Y Y    T   E     C          5 Cradock Road,                        */
/*    HHHHH   Y     T   EEE   C          Reading, Berks.    Tel: 0118 9757770	*/
/*    H   H   Y     T   E     C          RG2 0JT            Fax: 0118 9757566	*/
/*    H   H   Y     T   EEEEE  CCC       Web: www.hytec-electronics.co.uk       */
/********************************************************************************/
/*                                                                              */
/*				IOC9010 PROJECT	--- Hytec IP Card API Functions 				*/
/*							for Windows and Linux Platforms	    				*/
/*                                IP Card Type: 8512                            */
/********************************************************************************/
/*  Source file name      :-      Hy8512.h										*/
/*  Definition header file:		  HYTECIPAPI.h									*/
/*  Platform			  :		  Linux, Windows XP, Windows 2000               */
/*  Initial creation date :-      15-OCT-2007                                   */
/*                                                                              */
/*  Original Developers   :-      Darrell Nineham                               */
/********************************************************************************/
/*                                                                              */
/*  Description :-                                                              */
/*                                                                              */
/*  This header file is designed for Hytec API function library of 8512 IP card.*/
/*  It defines the class prototype, error code, masks and registers of 8512.    */
/*																				*/
/*  The design of this interface is cross platforms, which means the code 		*/
/*	can be used in operating systems like Windows, Linux etc without changes.	*/
/*                                                                              */
/*  The language used is C++ but the final product -- the Dynamic Link library	*/
/*	(dll) or the shared library can be used by either C or C++ applications.	*/
/*																				*/
/*  (C)2007 Hytec Electronics Ltd.												*/
/*                                                                              */
/********************************************************************************/
/*                                                                              */
/*  Modification record :-                                                      */
/*                                                                              */
/*  Date modified |     Person    |   Description of modification  | Version    */
/* ---------------|---------------|-------------------------------------------- */
/*    15/10/07    |	  D Nineham   |	  Initial copy				   |  0.0.1     */
/*    18/10/10    |   KJ          |   Added a support for than 1   |  1.0.0     */
/*    		      |   	          |   device                       |  		    */
/*    13/12/10    |   K Jogia     |   Added callback function and  |  1.0.1     */
/*                |               |   interrupt functions for 8512 |            */
/*                |               |                                             */
/*                                                                              */
/********************************************************************************/
#include "HytecIPAPI.h"

#if PLATFORM == 1

#define HYTECAPI_EXPORTS 1
#ifdef HYTECAPI_EXPORTS
#define HYTECAPI_API __declspec(dllexport)
#else
#define HYTECAPI_API __declspec(dllimport)
#endif
#include "C:\Hytec\IOC9010\IOCGeneric\IOCGeneric\IOCGeneric.h"

#else

#include "IOCGeneric.h"

#endif


//error code definition
#define NOT_8512_IP             -8512010
#define Hy8512_INVALID_REG   	-8512012
#define Hy8512_NOT_CONFIGURED   -8512014
#define Hy8512_NOT_SUPPORTED   	-8512016

//signature definition
#define SIGNATURE_8512          0x8512

//register address defines
#define CSR_8512                0x0000
#define ARM_8512                0x0002
#define OVERFLOW_MONITOR_8512   0x0004
#define OVERFLOW_8512           0x0006
#define INTERVAL_ENABLE_8512    0x0008
#define BLOCK_MODE_8512         0x000A
#define DAISY_CHAIN_8512        0x000C
#define CLOCK_SOURCE_8512       0x000E
#define ARM_ENABLE_8512			0x0010
#define OVERFLOW_CLEAR_8512		0x0012
#define ARM_CLEAR_8512			0x0014
#define FIRST_CHANNEL_8512      0x0000      // First location in Memory

//register address defines for read and write register api functions
#define REG_CSR_8512                0x0000
#define REG_ARM_8512                0x0002
#define REG_OVERFLOW_8512		    0x0004
#define REG_IRQ_OVERFLOW_8512       0x0006
#define REG_INTERVAL_ENABLE_8512    0x0008
#define REG_BLOCK_MODE_8512         0x000A
#define REG_DAISY_CHAIN_8512        0x000C
#define REG_GATE_ENABLE_8512        0x000E
#define REG_ARM_ENABLE_8512			0x0010
#define REG_OVERFLOW_CLEAR_8512		0x0012
#define REG_ARM_CLEAR_8512			0x0014

// This class is exported from the HYTECAPI.dll
#if PLATFORM == 1									//windows
class HYTECAPI_API CHy8512 : public CIOCGeneric
#else											//linux
class CHy8512 : public CIOCGeneric
#endif
{
private:
	long   m_ErrorCode;            // global error code holder
	int    initIPSlot;             // remember the IP slot when configures
	int    initVMESlot;
	int	   m_revision;			   // Xylinx Revision
	char   m_vector;
	int    m_INTLevel;

    // Variables used to Shadow 8512 Registers - to restore setup    
    USHORT usShadowScaler; 
    USHORT usShadowClock; 
    char   cShadowVector; 
    USHORT usShadowOverflow;


public:
	CHy8512(void);
	CHy8512(char *devName);
	CHy8512(int vmeslot, char ipslot);
	~CHy8512(void);

	long GetErrorCode()
	{
		return m_ErrorCode;
	}
	
/*----------------------------------------------------------------------------------*/
/*Function:		Configure 8512 IP card												*/
/*----------------------------------------------------------------------------------*/
/*Description:	8512 IP configuration routine										*/
/*				according to the settings.											*/
/*Prototype:	long Hy8512_Configure(int vmeslot, char ipslot, int clockRate)		*/
/*Arguements:	vmeslot  = VME slot number											*/
/*				ipslot   = IP card slot number 0 ~ 3 for VME, 0 ~ 5 for IOC			*/
/*				scaler   = Bitwise Selection of which Scalers are armed / setup.    */
/*				clock    = Bitwise Selection Scaler Clock Source 0-Ext 1-Int 10MHz  */
/*				vector   = Interrupt Vector (For Future Use - Presently Ignorred).  */
/*				overflow = Interrupt Mask (For Future Use - Presently Ignorred).    */
/*Return:		status code, 0 = successful. negative = errors						*/
/*----------------------------------------------------------------------------------*/
	long Hy8512_Configure(int vmeslot, char ipslot, USHORT scaler, USHORT clock, char vector, USHORT overflow);

/*----------------------------------------------------------------------------------*/
/*Function:		Configure 8512 IP card	 (Overloading)								*/
/*----------------------------------------------------------------------------------*/
/*Description:	8512 IP configuration routine										*/
/*				according to the settings.											*/
/*Prototype:	long Hy8512_Configure(int vmeslot, char ipslot, int clockRate)		*/
/*Arguements:	vmeslot  = VME slot number											*/
/*				ipslot   = IP card slot number 0 ~ 3 for VME, 0 ~ 5 for IOC			*/
/*				scaler   = Bitwise Selection of which Scalers are armed / setup.    */
/*				clock    = Bitwise Selection Scaler Clock Source 0-Ext 1-Int 10MHz  */
/*				vector   = Interrupt Vector (For Future Use - Presently Ignorred).  */
/*				overflow = Interrupt Mask (For Future Use - Presently Ignorred).    */
/*				callback = user callback function									*/
/*				drive = user data structure passed to the ISR						*/
/*Return:		status code, 0 = successful. negative = errors						*/
/*----------------------------------------------------------------------------------*/
	long Hy8512_Configure(int vmeslot, char ipslot, USHORT scaler,
							USHORT clock, char vector, USHORT overflow,
							int INTLevel, void (*callback)(void *), void *drive);

/*----------------------------------------------------------------------------------*/
/*Function:		Enable 8512 interrupt												*/
/*----------------------------------------------------------------------------------*/
/*Description:	Enable interrupt for 8512 IP card          							*/
/*Prototype:	long Hy8512_EnableInterrupt(void)									*/
/*Arguements:	none																*/
/*Return:		0 is successfull, negative please see error define					*/
/*----------------------------------------------------------------------------------*/
	long Hy8512_EnableInterrupt(void);

/*----------------------------------------------------------------------------------*/
/*Function:		Disable 8512 interrupt             									*/
/*----------------------------------------------------------------------------------*/
/*Description:	Disable interrupt for 8512 IP card          					    */
/*Prototype:	long Hy8512_DisableInterrupt(void)									*/
/*Arguements:	none																*/
/*Return:		0 is successfull, negative please see error define					*/
/*----------------------------------------------------------------------------------*/
	long Hy8512_DisableInterrupt(void);

/*----------------------------------------------------------------------------------*/
/*Function:		Reset 8512 interrupt												*/
/*----------------------------------------------------------------------------------*/
/*Description:	Reset interrupt flag bit15 for 8512 IP card							*/
/*Prototype:	long Hy8512_ResetInterrupt(USHORT reason)							*/
/*Arguements:	none																*/
/*Return:		0 is successfull, negative please see error define					*/
/*----------------------------------------------------------------------------------*/
	long Hy8512_ResetInterrupt(USHORT reason);

/*----------------------------------------------------------------------------------*/
/*Function:		Get Interrupt Reason 8512 											*/
/*----------------------------------------------------------------------------------*/
/*Description:	Reset interrupt flag bit15 for 8512 IP card							*/
/*Prototype:	long Hy8512_INTReason(USHORT *data)									*/
/*Arguements:	none																*/
/*Return:		0 is successfull, negative please see error define					*/
/*----------------------------------------------------------------------------------*/
	long Hy8512_INTReason(USHORT *data);

/*----------------------------------------------------------------------------------*/
/*Function:		Read All channels of 8512											*/
/*----------------------------------------------------------------------------------*/
/*Description:	Read all channels data from 8512 Scaler IP card						*/
/*Prototype:	long Hy8512_Read(ULONG *value, boolean clear)     					*/
/*Arguements:	*value = pointer to the 32 bit value variable array                 */
/*              clear  = 1 - Clear All Channels to zero When Read.  				*/
/*Return:		0 is successfull, negative please see error define					*/
/*----------------------------------------------------------------------------------*/
	long Hy8512_Read(ULONG *value, bool clear);

/*----------------------------------------------------------------------------------*/
/*Function:		Read a single channel of 8512 (overloading)    						*/
/*----------------------------------------------------------------------------------*/
/*Description:	Read a single channel from 8512 Scaler IP card						*/
/*Prototype:	long Hy8512_Read(int scaler, ULONG *value, boolean clear)     		*/
/*Arguements:	scaler = which scaler (0-15) to read.						        */
/*              *value = pointer to the 32 bit value variable						*/
/*              clear  = 1 - Clear Scaler Channel to zero When Read.   				*/
/*Return:		0 is successfull, negative please see error define					*/
/*----------------------------------------------------------------------------------*/
	long Hy8512_Read(int scaler, ULONG *value, bool clear);

/*----------------------------------------------------------------------------------*/
/*Function:		Write a single channel of 8512                        	            */  
/*----------------------------------------------------------------------------------*/
/*Description:	Write a single channel from 8512 Scaler IP card						*/
/*Prototype:	long Hy8512_Write(int scaler, ULONG value, boolean clear)    		*/
/*Arguements:	scaler = which scaler (0-15) to update.						        */
/*              value = 32 bit value to be written					   			    */
/*Return:		0 is successfull, negative please see error define					*/
/*----------------------------------------------------------------------------------*/
	long Hy8512_Write(int scaler, ULONG value);

/*----------------------------------------------------------------------------------*/
/*Function:		Read a single application register on 8512                        	*/  
/*----------------------------------------------------------------------------------*/
/*Description:	Read an individual application register from 8512 Scaler IP card	*/
/*Prototype:	long Hy8512_Read_Reg(int reg, USHORT *value)			    		*/
/*Arguements:	reg = which register (0-7) to read.							        */
/*              *value = Pointer to 16 bit variable for register value 			    */
/*Return:		0 is successfull, negative please see error define					*/
/*----------------------------------------------------------------------------------*/
	long Hy8512_Read_Reg(int reg, USHORT *value);

/*----------------------------------------------------------------------------------*/
/*Function:		Update a single application register on 8512                       	*/  
/*----------------------------------------------------------------------------------*/
/*Description:	Overwrite an individual application register on 8512 Scaler IP card	*/
/*Prototype:	long Hy8512_Write_Reg(int reg, USHORT value)			    		*/
/*Arguements:	reg = which register (0-7) to read.							        */
/*              *value = 16 bit value to write to register			 			    */
/*Return:		0 is successfull, negative please see error define					*/
/*----------------------------------------------------------------------------------*/
	long Hy8512_Write_Reg(int reg, USHORT value);

/*----------------------------------------------------------------------------------*/
/*Function:		ARM individual scalers on 8512                        	            */  
/*----------------------------------------------------------------------------------*/
/*Description:	ARM individual scalers on 8512 Scaler IP card						*/
/*Prototype:	long Hy8512_ArmEnable(USHORT scaler_mask)					    	*/
/*Arguements:	scaler_mask = mask of which scalers to ARM (0 - 0xffff)				*/
/*				(1=ARM  0=Ignore)													*/
/*Return:		0 is successfull, negative please see error define					*/
/*----------------------------------------------------------------------------------*/
	long Hy8512_ArmEnable(USHORT scaler_mask);

/*----------------------------------------------------------------------------------*/
/*Function:		DISARM individual scalers on 8512                      	            */  
/*----------------------------------------------------------------------------------*/
/*Description:	DISARM individual scalers on 8512 Scaler IP card					*/
/*Prototype:	long Hy8512_ArmClear(USHORT scaler_mask)					    	*/
/*Arguements:	scaler_mask = mask of which scalers to DISARM (0 - 0xffff)			*/
/*				(1=DISARM  0=Ignore)												*/
/*Return:		0 is successfull, negative please see error define					*/
/*----------------------------------------------------------------------------------*/
	long Hy8512_ArmClear(USHORT scaler_mask);

/*----------------------------------------------------------------------------------*/
/*Function:		Clear Overflow bit for individual scalers on 8512                   */  
/*----------------------------------------------------------------------------------*/
/*Description:	Clear Overflow bit for individual scalers on 8512 Scaler IP card	*/
/*Prototype:	long Hy8512_OverflowClear(USHORT scaler_mask)					    */
/*Arguements:	scaler_mask = mask of which scalers to update (0 - 0xffff)			*/
/*				(1=Clear Overflow bit  0=Ignore)									*/
/*Return:		0 is successfull, negative please see error define					*/
/*----------------------------------------------------------------------------------*/
	long Hy8512_OverflowClear(USHORT scaler_mask);
};

#if PLATFORM == 1									//windows
extern "C" HYTECAPI_API CHy8512* CreateCHy8512();
extern "C" HYTECAPI_API void DeleteCHy8512( CHy8512* instance );
#else											//linux
extern "C" CHy8512* CreateCHy8512();
extern "C" void DeleteCHy8512( CHy8512* instance );
#endif

