/********************************************************************************/
/*    H   H Y   Y TTTTT EEEEE  CCC       HYTEC ELECTRONICS LTD                  */
/*    H   H  Y Y    T   E     C          5 Cradock Road,                        */
/*    HHHHH   Y     T   EEE   C          Reading, Berks.    Tel: 0118 9757770	*/
/*    H   H   Y     T   E     C          RG2 0JT            Fax: 0118 9757566	*/
/*    H   H   Y     T   EEEEE  CCC       Web: www.hytec-electronics.co.uk       */
/********************************************************************************/
/*                                                                              */
/*				IOC9010 PROJECT	--- Hytec IP Card API Functions 				*/
/*							for Windows and Linux Platforms	    				*/
/*                                IP Card Type: 85156                           */
/********************************************************************************/
/*  Source file name      :-      Hy85156.h										*/
/*  Definition header file:		  HYTECIPAPI.h									*/
/*  Platform			  :		  Linux, Windows XP, Windows 2000               */
/*  Initial creation date :-      11-AUG-2006                                   */
/*                                                                              */
/*  Original Developers   :-      Jim S Chen                                    */
/********************************************************************************/
/*                                                                              */
/*  Description :-                                                              */
/*                                                                              */
/*  This header file is designed for Hytec API function library of 8515 and		*/
/*	8516 IP card. It defines the class prototype, error code, masks and 		*/
/*	registers of 8515/8516.    													*/
/*																				*/
/*  The design of this interface is cross platforms, which means the code 		*/
/*	can be used in operating systems like Windows, Linux etc without changes.	*/
/*                                                                              */
/*  The language used is C++ but the final product -- the Dynamic Link library	*/
/*	(dll) or the shared library can be used by either C or C++ applications.	*/
/*																				*/
/*  (C)2006 Hytec Electronics Ltd.												*/
/*                                                                              */
/********************************************************************************/
/*                                                                              */
/*  Modification record :-                                                      */
/*                                                                              */
/*  Date modified |     Person    |   Description of modification | Version     */
/* ---------------|---------------|-------------------------------------------- */
/*    11/08/06    |	  Jim S Chen  |	  Initial copy				  |	0.0.1		*/
/*    26/03/09    |   JC          |   Added interrupt support     |  0.2.0      */
/*    21/10/09    |   JC          |   Added Half/full duplex      |  0.3.0      */
/*                |               |   support by overloading      |             */
/*                |               |   ComSetup and configure funs.|             */
/*                |               |   Correct baud rate           |             */
/*                |               |   setting when devided by 4.  |             */
/*    18/10/10    |   KJ          |   Added a support for than 1  |  1.0.0      */
/*    		      |   	          |   device                      |  		    */
/*                |               |                               |             */
/********************************************************************************/
#include "HytecIPAPI.h"

#if PLATFORM == 1

#define HYTECAPI_EXPORTS 1
#ifdef HYTECAPI_EXPORTS
#define HYTECAPI_API __declspec(dllexport)
#else
#define HYTECAPI_API __declspec(dllimport)
#endif
#include "C:\Hytec\IOC9010\IOCGeneric\IOCGeneric\IOCGeneric.h"

#else

#include "IOCGeneric.h"

#endif


//error code definition
#define NOT_85156_IP -85156010
#define Hy85156_NOT_CONFIGURED -85156011
#define PORT_INVALID_85156 -85156012
#define BAUD_RATE_INVALID -85156013
#define PARITY_INVALID -85156014
#define DATA_BITS_INVALID -85156015
#define STOP_BITS_INVALID -85156016
#define FLOW_CONTROL_INVALID -85156017
#define INT_TRIGGERLEVEL_INVALID -85156018
#define FALSE_INTERRUPT -85156019
#define RS485_DELAYERROR -85156020
#define RS485_DUPLEXERROR -85156021
#define GENERAL_PURPOSE_INVALID -85156022

//signature definition
#define SIGNATURE_8515 0x8515
#define SIGNATURE_8516 0x8516

//register address defines
#define INTERRUPT_REG_85156 0x00*2
#define INT_SOURCE1_85156 0x01*2
#define INT_SOURCE2_85156 0x02*2
#define INT_SOURCE3_85156 0x03*2
#define TIMER_CONTROL_85156 0x04*2
#define TIMER_CONTROL0_85156 0x05*2
#define TIMER_COUNTER_LS_85156 0x06*2
#define TIMER_COUNTER_MS_85156 0x07*2
#define MODE_REG_85156 0x08*2
#define DUPLEX_REG_85156 0x09*2
#define RESET_REG_85156 0x0A*2
#define SLEEP_REG_85156 0x0B*2
#define REVISION_REG_85156 0x0C*2
#define ID_REG_85156 0x0D*2
#define CSR_85156 0x0E*2

#define ENABLE_INTERRUPT_85156 0x4000
#define PORT_OFFSET 0x20

#define RHR_85156 0x00
#define THR_85156 RHR_85156
#define DLL_85156 RHR_85156
#define DLM_85156 0x01
#define IER_85156 DLM_85156
#define ISR_85156 0x02
#define FCR_85156 ISR_85156
#define LCR_85156 0x03
#define MCR_85156 0x04
#define LSR_85156 0x05
#define MSR_85156 0x06
#define SPR_85156 0x07
#define FCTR_85156 0x08
#define EFR_85156 0x09
#define TXCNT_85156 0x0A
#define TXTRG_85156 TXCNT_85156
#define RXCNT_85156 0x0B
#define RXTRG_85156 RXCNT_85156
#define RCHAR_85156 0x0C
#define XOFF1_85156 RCHAR_85156
#define XOFF2_85156 0x0D
#define XON1_85156 0x0E
#define XON2_85156 0x0F

//mask definition
#define PARITY_MASK_85156 0xC7
#define NO_PARITY_85156 0x00
#define ODD_PARITY_85156 0x08
#define EVEN_PARITY_85156 0x18

#define DATABIT_MASK_85156 0xFC
#define DATA_5_85156 0x00
#define DATA_6_85156 0x01
#define DATA_7_85156 0x02
#define DATA_8_85156 0x03

#define STOPBIT_MASK_85156 0xFB
#define STOPBIT_1_85156 0x00
#define STOPBIT_2_85156 0x04

#define ENABLE_DIVISOR_MASK_85156 0x80
#define ENABLE_FIFO_85156 0x01
#define THR_EMPTY_85156 0x20
#define TSR_EMPTY_85156 0x40
#define ENABLE_RHR_INT_85156 0x01
#define ENABLE_THR_INT_85156 0x02
#define ENABLE_RCVLINE_INT_85156 0x04
#define ENABLE_MODEM_INT_85156 0x08
#define FCR_RXFRESET_85156 0x02
#define FCR_TXFRESET_85156 0x04
#define ENABLE_LEVEL_TRIGGER_85156 0xC0

/******* Interrupt reasons ******/
/* Major reason */
#define INTERRUPT_REASON_MSR 0x00
#define INTERRUPT_REASON_LASER INTERRUPT_REASON_MSR                 /* This macro is purely for Sky Scanner job */
#define INTERRUPT_REASON_NONE 0x01
#define INTERRUPT_REASON_TXRDY 0x02
#define INTERRUPT_REASON_RXRDY 0x04
#define INTERRUPT_REASON_LSR 0x06
#define INTERRUPT_REASON_RXRDYTO 0x0C
#define INTERRUPT_REASON_RXRDYXONOFF 0x10
/* Detail reason */
#define INTERRUPT_REASON_MSR_CTS 0x0100
#define INTERRUPT_REASON_MSR_DSR 0x0200
#define INTERRUPT_REASON_MSR_RI 0x0400
#define INTERRUPT_REASON_MSR_CD 0x0800
#define INTERRUPT_REASON_LASER_READY INTERRUPT_REASON_MSR_CD        /* This macro is purely for Sky Scanner job */

#define INTERRUPT_REASON_LSR_RX_OVERRUN 0x0200
#define INTERRUPT_REASON_LSR_RX_PARITY_ERROR 0x0400
#define INTERRUPT_REASON_LSR_RX_FRAMING_ERROR 0x0800
#define INTERRUPT_REASON_LSR_RX_BREAK 0x1000
/******* Interrupt reasons end******/


/* general purpose line definition */
#define GENERAL_PURPOSE_DTR_85156		0x01
#define GENERAL_PURPOSE_RTS_85156		0x02
#define GENERAL_PURPOSE_USE_DTR_85156	0x04


/* baud rate settings */
#define DIVIDED_BY_FOUR_85156			0x80
#define AUTO_RS485_ENABLE_85156			0x20

#define BAUDRATE_100_DLL_85156			0x00
#define BAUDRATE_100_DLM_85156			0x09

#define BAUDRATE_400_DLL_85156			0x00
#define BAUDRATE_400_DLM_85156			0x09

#define BAUDRATE_600_DLL_85156			0x80
#define BAUDRATE_600_DLM_85156			0x01

#define BAUDRATE_1200_DLL_85156			0xC0
#define BAUDRATE_1200_DLM_85156			0x00

#define BAUDRATE_2400_DLL_85156			0x80
#define BAUDRATE_2400_DLM_85156			0x00

#define BAUDRATE_4800_DLL_85156			0xC0
#define BAUDRATE_4800_DLM_85156			0x00

#define BAUDRATE_9600_DLL_85156			0x60
#define BAUDRATE_9600_DLM_85156			0x00

#define BAUDRATE_19200_DLL_85156		0x30
#define BAUDRATE_19200_DLM_85156		0x00

#define BAUDRATE_38400_DLL_85156		0x18
#define BAUDRATE_38400_DLM_85156		0x00

#define BAUDRATE_57600_DLL_85156		0x04
#define BAUDRATE_57600_DLM_85156		0x00

#define BAUDRATE_76800_DLL_85156		0x0C
#define BAUDRATE_76800_DLM_85156		0x00

#define BAUDRATE_115200_DLL_85156		0x02
#define BAUDRATE_115200_DLM_85156		0x00

#define BAUDRATE_153600_DLL_85156		0x06
#define BAUDRATE_153600_DLM_85156		0x00

#define BAUDRATE_230400_DLL_85156		0x04
#define BAUDRATE_230400_DLM_85156		0x00

#define BAUDRATE_460800_DLL_85156		0x02
#define BAUDRATE_460800_DLM_85156		0x00

#define BAUDRATE_921600_DLL_85156		0x01
#define BAUDRATE_921600_DLM_85156		0x00

#define USE_HARDWARE_FLOW_TEST_85156	0x10
#define CTS_DTR_TEST_85156			0x20
#define RTS_DTR_CTS_DSR_ENABLE_85156		0xC0
#define DTR_DSR_SELECT_85156		0x04
#define USE_SOFTWARE_FLOW_TEST_85156	0x0F
#define SOFTWARE_FLOW_MASK_85156	0x0F

#define FLUSH_TX_FIFO 0x04
#define FLUSH_RX_FIFO 0x02
#define COM_0_TX_FLUSH 0x0001
#define COM_0_RX_FLUSH 0x0002
#define COM_1_TX_FLUSH 0x0004
#define COM_1_RX_FLUSH 0x0008
#define COM_2_TX_FLUSH 0x0010
#define COM_2_RX_FLUSH 0x0020
#define COM_3_TX_FLUSH 0x0040
#define COM_3_RX_FLUSH 0x0080
#define COM_4_TX_FLUSH 0x0100
#define COM_4_RX_FLUSH 0x0200
#define COM_5_TX_FLUSH 0x0400
#define COM_5_RX_FLUSH 0x0800
#define COM_6_TX_FLUSH 0x1000
#define COM_6_RX_FLUSH 0x2000
#define COM_7_TX_FLUSH 0x4000
#define COM_7_RX_FLUSH 0x8000

#define FIFO_SIZE 64
#define FIFO_SIZE 64
#define MAX_PORT_NO 7


// This class is exported from the HYTECAPI.dll
#if PLATFORM == 1									//windows
class HYTECAPI_API CHy85156 : public CIOCGeneric
#else											//linux
class CHy85156 : public CIOCGeneric
#endif
{
private:
	long m_ErrorCode;							// global error code holder
	int initIPSlot;								// remember the IP slot when configures
	int initVMESlot;/*Prototype:	long Hy85156_WriteLine(int port, int lineMASK, int DTRdata, int RTSdata)*/
/*Arguements:	port = serial channel 0 ~ 7											*/
/*				line = general purpose line when modem is not used					*/
/*                  could be either GENERAL_PURPOSE_DTR_85156                       */              
/*                               or GENERAL_PURPOSE_RTS_85156                       */
/*                               or both                                            */
/*				DTRdata = logic 1 or logic 0                 					    */
/*				RTSdata = logic 1 or logic 0                 					    */

    int m_moduleID;
    int m_duplex;                               // for remembering half/full duplex setting
    int m_withecho;                             // for remembering echoing requirement in half duplex
	int m_mask;
	char m_vector;
	int m_INTLevel;
	int m_flowControl;

public:
	CHy85156(void);
	CHy85156(char *devName);
	CHy85156(int vmeslot, char ipslot);
	~CHy85156(void);

	long GetErrorCode()
	{
		return m_ErrorCode;
	}
	
/*----------------------------------------------------------------------------------*/
/*Function:		Configure 85156 IP card												*/
/*----------------------------------------------------------------------------------*/
/*Description:	85156 IP configuration routine according to the settings.			*/
/*				Note: if use multiple channels, this is used for the first channel	*/
/*				only																*/	
/*Prototype:	long Hy85156_Configure(int vmeslot, char ipslot, char vector,		*/
/*				USHORT delay, int halfduplex, int delay485)							*/
/*Arguements:	vmeslot = The number of the VME carrier card on which the IP card	*/
/*					resides. The carrier must be initialised before this routine.	*/
/*					This param is only for VME.										*/
/*																					*/
/*				ipslot = IP card slot number 0 ~ 3 for VME, 0 ~ 5 for IOC			*/
/*																					*/
/*				vector = The interrupt vector number [1..255] designated to this IP	*/
/*					card. The card will not work without interrupts. Different		*/
/*					physical 8515/8516 cards may share the same interrupt number.	*/
/*					segment of the interrupt vector. only the upper 8 bits			*/
/*																					*/
/*				delay = For Positive Values -										*/
/*					Determine the number of clock ticks between timer interrupts.	*/
/*					The clock frequency is 125 KHz. The driver checks all active	*/
/*					channels of the serial card for characters that need to be read */	
/*					or written at every interrupt delay.							*/
/*																					*/
/*					For Negative Values -											*/
/*					The modulus of the Value sets the depth at which the RX FIFO	*/
/*					is filled and/or the TX FIFO is emptied before the serail IC	*/
/*					generates an Interrupt. Once an interrupt is detected, the		*/
/*					driver checks all active channels of the serial card for		*/
/*					characters that need to be read or written at every interrupt	*/
/*					delay.															*/
/*																					*/
/*					The modulus of intdelay should not be chosen to be too big for	*/
/*					a given data rate, keeping in mind that all channels have 64	*/
/*					byte FIFOs.														*/
/*																					*/
/*				halfduplex = delay the interrupt vector.							*/
/*					0:  Full duplex mode. Select this when using the 8515,			*/
/*						(for RS232) or full duplex RS485 mode when using on the		*/
/*						8516.														*/
/*					1:  Selects 'auto 485 mode' of the XR16L788 UART chip on the	*/
/*						8516. Do not select this option when using the 8515.		*/
/*																					*/
/*				delay485 = A number between 0 and 15. This parameter is only used if*/
/*					halfduplexmode = 1. It specifies the delay time in terms of		*/
/*					bits (depends on baudrate) after the last stop bit of the last	*/
/*					transmitted character before the state of the RTS# output is	*/
/*					changed to enable receiving.									*/
/*																					*/
/*Return:		status code, 0 = successful. negative = errors						*/
/*----------------------------------------------------------------------------------*/
	long Hy85156_Configure(int vmeslot, char ipslot, char vector, int delay,
							int halfduplex, int delay485);

/*----------------------------------------------------------------------------------*/
/*Function:		Register 85156 IP card overloading									*/
/*----------------------------------------------------------------------------------*/
/*Description:	Register 85156 IP card. This is used by other channels other than	*/
/*				the first one. So when use multiple channels, call the one above	*/
/*				for the first channel's configuration and call this function for  	*/	
/*				the subsequent channel(s)											*/
/*Prototype:	long Hy85156_Configure(int vmeslot, char ipslot)					*/
/*Arguements:	vmeslot = The number of the VME carrier card on which the IP card	*/
/*					resides. The carrier must be initialised before this routine.	*/
/*					This param is only for VME.										*/
/*				ipslot = IP card slot number 0 ~ 3 for VME, 0 ~ 5 for IOC			*/
/*Return:		status code, 0 = successful. negative = errors						*/
/*----------------------------------------------------------------------------------*/
	long Hy85156_Configure(int vmeslot, char ipslot);

/*----------------------------------------------------------------------------------*/
/*Function:		Correct 85156 IP configuration (overloading)       					*/
/*----------------------------------------------------------------------------------*/
/*Description:	The original configure function wrongly takes duplex and RS485 delay*/
/*				settings at this level. Those should be set for each individual     */
/*				UART. They don't do anything though. Hence this is the correct  	*/	
/*				configuration function wiht dummy value for duplex and RS485        */
/*              delay passed to the original function								*/
/*Prototype:	long Hy85156_Configure(int vmeslot, char ipslot, char vector, int delay)	*/
/*Arguements:	same as the first form (the original one)                         	*/
/*Return:		status code, 0 = successful. negative = errors						*/
/*----------------------------------------------------------------------------------*/
    long Hy85156_Configure(int vmeslot, char ipslot, char vector, int delay);

/*----------------------------------------------------------------------------------*/
/*Function:		Correct 85156 IP configuration (overloading with interrupt callback)*/
/*----------------------------------------------------------------------------------*/
/*Description:	The original configure function wrongly takes duplex and RS485 delay*/
/*				settings at this level. Those should be set for each individual     */
/*				UART. They don't do anything though. Hence this is the correct  	*/	
/*				configuration function wiht dummy value for duplex and RS485        */
/*              delay passed to the original function								*/
/*Prototype:	long Hy85156_Configure(int vmeslot, char ipslot, char vector,		*/
/*				int delay, int INTLevel, void (*callback)(void *), void *drive)		*/
/*Arguements:	same as the first form (the original one)                         	*/
/*				INTLevel = interrupt level											*/
/*				callback = user callback											*/
/*				drive = user data structure passed to the ISR						*/
/*Return:		status code, 0 = successful. negative = errors						*/
/*----------------------------------------------------------------------------------*/
	long Hy85156_Configure(int vmeslot, char ipslot, char vector, int delay,
								 int INTLevel, void (*callback)(void *), void *drive);

/*----------------------------------------------------------------------------------*/
/*Function:		Configure 85156 COM Port											*/
/*----------------------------------------------------------------------------------*/
/*Description:	set up one 85156 COM port											*/
/*Prototype:	Hy85156_ComsSetup(int port, USHORT baud, char parity,				*/
/*				int numstopbits, int numdatabits, char flowctr)						*/
/*Arguements:	port = serial channel 0 ~ 7											*/
/*				baud = baud rate divisor. This can be the follwoing number only		*/
/*					baud setting	value to register	internal loopback			*/
/*					------------------------------------------------------			*/
/*						100			2304 (0x0900)			1						*/
/*						400			2304 (0x0900)			0						*/
/*						600			 384 (0x0180)			1						*/
/*						1200		 192 (0x00C0)			1						*/
/*						2400		 128 (0x0080)			0						*/
/*						4800		 192 (0x00C0)			0						*/
/*						9600		  96 (0x0060)			0						*/
/*						19200		  48 (0x0030)			0						*/
/*						38400		  24 (0x0018)			0						*/
/*						57600		   4 (0x0004)			1						*/
/*						76800		  12 (0x000C)			0						*/
/*						115200		   2 (0x0002)			1						*/
/*						153600		   6 (0x0006)			0						*/
/*						230400		   4 (0x0004)			0						*/
/*						460800		   2 (0x0002)			0						*/
/*						921600		   1 (0x0001)			0						*/
/*				parity = parity setting												*/
/*					"N"	no parity													*/
/*					"O"	odd parity													*/
/*					"E"	even parity													*/
/*				numdatabits = data bits												*/
/*					=5, 6, 7, 8	for 5,6,7,8 bits									*/
/*				numstopbits = sstop bits											*/
/*					1	1 stop bit													*/
/*					4	1 1/2 stop bit if data bit = 5								*/
/*					4	2 stop bit if data bit = 6,7,8								*/
/*				flowctr = flow control byte											*/
/*					0 0 X X X X X X			FCR_RXFRESET_85156						*/
/*					[_]	| | [_____]													*/
/*					 |	| |	   |													*/
/*			   no used	| |	software flow control									*/
/*						| | 0 0 0 0	no software flow control						*/
/*						| | 1 1 1 1	transmit Xon1 and Xon2 / Xoff1 and Xoff2		*/
/*						| |			and receiver compares  Xon1 and Xon2 / Xoff1	*/
/*						| |			and Xoff2										*/
/*						| =0 disable hardware flow control, =1 enable				*/
/*					    =0 use RTS/CTS, =1 use DTR/DSR pair							*/
/*Return:		status code, 0 = successful. negative = errors						*/
/*----------------------------------------------------------------------------------*/
	long Hy85156_ComsSetup(int port, ULONG baud, char parity, int numstopbits, int numdatabits,
							char flowctr);

/*----------------------------------------------------------------------------------*/
/*Function:		Configure 85156 COM Port (overloading for 8516 RS-485)     			*/
/*----------------------------------------------------------------------------------*/
/*Description:	set up one 85156 COM port											*/
/*Prototype:	Hy85156_ComsSetup(int port, USHORT baud, char parity,				*/
/*				int numstopbits, int numdatabits, char flowctr, int halfduplex,     */
/*              int delay485)                 						*/
/*Arguements:	same as above             											*/
/*				halfduplex = 0 Full duplex                                  		*/
/*					       = 1 Half duplex with echo                       			*/
/*					       = 2 Half duplex without echo                   			*/
/*				delay485 = A number between 0 and 15. This parameter is only used if*/
/*					halfduplexmode = 1. It specifies the delay time in terms of		*/
/*					bits (depends on baudrate) after the last stop bit of the last	*/
/*					transmitted character before the state of the RTS# output is	*/
/*					changed to enable receiving.									*/
/*Return:		status code, 0 = successful. negative = errors						*/
/*----------------------------------------------------------------------------------*/
    long Hy85156_ComsSetup(int port, ULONG baud, char parity, int numstopbits, 
								 int numdatabits, char flowctr, int halfduplex, 
                                 int delay485);

/*----------------------------------------------------------------------------------*/
/*Function:		Enable Port Interrupt      										    */
/*----------------------------------------------------------------------------------*/
/*Description:	set up one or more interrupt enable bits for the specified port     */
/*Prototype:	Hy85156_EnableInterrupt(int port, int mask, int rcvTrigLevel,       */
/*              int transTrigLevel)                                                 */
/*Arguements:	port = serial channel 0 ~ 7											*/
/*				mask = interrupt enable mask                          	            */
/*                   = bit 7  6  5  4  3  2  1  0                                   */
/*                         |  |  |  |  |  |  |  |                                   */
/*                         |  |  |  |  |  |  | RX Data or RxFIFO up to a level INT  */
/*                         |  |  |  |  |  | TX Empty or TxFIFO down to a level INT  */
/*                         |  |  |  |  | RX Line Status INT                         */
/*                         |  |  |  | Modem Status INT                              */
/*                         |  |  | Not used                                         */
/*                         |  | Xon/Xoff Special Chat INT                           */
/*                         | RTS/DTR# INT                                           */
/*                        CTS/DSR# INT                                              */
/*				rcvTrigLevel = 1 ~ 64, interrupt trigger level on receiving			*/
/*                         only needed when mask bit0 aand/or bit2 is set           */
/*				transTrigLevel = 1 ~ 64, interrupt trigger level on transmitting	*/
/*                         only needed when mask bit1 is set                        */
/*Return:		status code, 0 = successful. negative = errors						*/
/*----------------------------------------------------------------------------------*/
    long Hy85156_EnableInterrupt(int port, int mask, int rcvTrigLevel, int transTrigLevel);

/*----------------------------------------------------------------------------------*/
/*Function:		Disable Port Interrupt      										*/
/*----------------------------------------------------------------------------------*/
/*Description:	set up one or more interrupt enable bits for the specified port     */
/*Prototype:	Hy85156_DisableInterrupt(int port)                                   */
/*Arguements:	port = serial channel 0 ~ 7											*/
/*Return:		status code, 0 = successful. negative = errors						*/
/*----------------------------------------------------------------------------------*/
    long Hy85156_DisableInterrupt(int port);

/*----------------------------------------------------------------------------------*/
/*Function:		Get Interrupt Source												*/
/*----------------------------------------------------------------------------------*/
/*Description:	Get which channel cause the interrtupt and what reason				*/
/*Prototype:	long Hy85156_INTReason(int* port, int *reason)						*/
/*Arguements:	*port = integer pointer to return the interrupt channel 0 ~ 7		*/
/*				*reason = integert pointer to teh reason of the interrtupt 0 ~ 7	*/
/*                      LSB (bit0 ~ bit7) major reason                              */
/*						= 0x00 MSR (Modem Status Register)							*/
/*						= 0x01 None													*/
/*						= 0x02 TXRDY (Transmitter Holding Register Empty)			*/
/*						= 0x04 RXRDY (received Data Ready)							*/
/*						= 0x06 LSR (Receiver Line Status Register)					*/
/*						= 0x0C RXRDY (Receive Data Timeout)							*/
/*						= 0x10 RXRDY (Received Xon/Xoff or Special character)		*/
/*						= 0x20 CTS#/DSR#, RTS#/DTR# change of state					*/
/*                      MSB (bit8 ~ bit15) detail reason                            */
/*                          when LSB = MSR                                          */
/*                              bit12 ~ bit15: not used                             */
/*                              bit11: CD# line interrupt                           */
/*                              bit10: RI# line interrupt                           */
/*                              bit9: DSR# line interrupt                           */
/*                              bit8: CTS# line interrupt                           */
/*                          when LSB = LSR                                          */
/*                              bit13 ~ bit15: not used                             */
/*                              bit12: RX Break                                     */
/*                              bit11: RX Framing Error                             */
/*                              bit10: RX Parity Error                              */
/*                              bit9: RX Overrun                                    */
/*                              bit8: not used                                      */
/*Return:		0 = OK																*/
/*				if error,  negative code in m_errorcode								*/
/*----------------------------------------------------------------------------------*/
	long Hy85156_INTReason(int* port, int *reason);

/*----------------------------------------------------------------------------------*/
/*Function:		Reset Interrupt														*/
/*----------------------------------------------------------------------------------*/
/*Description:	Reset interrtupt													*/
/*Prototype:	long Hy85156_ResetInterrupt(int port, int reason)					*/
/*Arguements:	port = interrupt channel 0 ~ 7										*/
/*				reason = the reason of the interrtupt 								*/
/*						= 0x00 MSR (Modem Status Register)							*/
/*						= 0x01 None													*/
/*						= 0x02 TXRDY (Transmitter Holding Register Empty)			*/
/*						= 0x04 RXRDY (received Data Ready)							*/
/*						= 0x06 LSR (Receiver Line Status Register)					*/
/*						= 0x0C RXRDY (Receive Data Timeout)							*/
/*						= 0x10 RXRDY (Received Xon/Xoff or Special character)		*/
/*						= 0x20 CTS#/DSR#, RTS#/DTR# change of state					*/
/*Return:		0 = OK																*/
/*				if error,  negative code in m_errorcode								*/
/*----------------------------------------------------------------------------------*/
	long Hy85156_ResetInterrupt(int port, int reason);

/*----------------------------------------------------------------------------------*/
/*Function:		Fire laser beam. This is a special function for Sky Scanner project */
/*----------------------------------------------------------------------------------*/
/*Description:	Fire laser. Actually it calls write to general purpose line			*/
/*Prototype:	long Hy85156_FireLaser(int port, int pulseLength)                   */
/*Arguements:	port = serial channel 0 ~ 7											*/
/*              pilseLength = pulse length in micro-second. If it is 0, then    	*/
/*                            it is a level (stay at logic 1)                       */
/*Return:		0 if successful      												*/
/*				if error, negative code in m_errorcode								*/
/*----------------------------------------------------------------------------------*/
    long Hy85156_FireLaser(int port, int pulseLength);

/*----------------------------------------------------------------------------------*/
/*Function:		Write to general purpose line      									*/
/*----------------------------------------------------------------------------------*/
/*Description:	Write logic 1 or logic 0 to specified general purpose line			*/
/*Prototype:	long Hy85156_WriteLine(int port, int line, int data)                */
/*Arguements:	port = serial channel 0 ~ 7											*/
/*				line = 0 to use RTS                               					*/
/*                   = 1 to use DTR                                                 */              
/*                   It cannot use both at the same time                            */
/*				data = logic 1 or logic 0                    					    */
/*Return:		0 if successful      												*/
/*				if error, negative code in m_errorcode								*/
/*----------------------------------------------------------------------------------*/
    long Hy85156_WriteLine(int port, int line, int data);

/*----------------------------------------------------------------------------------*/
/*Function:		Send Data															*/
/*----------------------------------------------------------------------------------*/
/*Description:	Write data to specified serial channel								*/
/*Prototype:	long Hy85156_Write(int port, UCHAR *value, int length)				*/
/*Arguements:	port = serial channel 0 ~ 7											*/
/*				*value = pointer to the 8 bit value array							*/
/*				length = length of the 8 bit value array							*/
/*Return:		number of bytes written												*/
/*				if error, negative code in m_errorcode								*/
/*----------------------------------------------------------------------------------*/
	long Hy85156_Write(int port, UCHAR *value, int length);

/*----------------------------------------------------------------------------------*/
/*Function:		Read receving values												*/
/*----------------------------------------------------------------------------------*/
/*Description:	Read from receiving buffer of one channel in the 8515 IP card		*/
/*Prototype:	long Hy85156_Read(int port, UCHAR *value)							*/
/*Arguements:	port = serial channel 0 ~ 7											*/
/*				*value = pointer to the 8 bit value array							*/
/*Return:		number of bytes read												*/
/*				if error,  negative code in m_errorcode								*/
/*----------------------------------------------------------------------------------*/
	long Hy85156_Read(int port, UCHAR *value);

/*----------------------------------------------------------------------------------*/
/*Function:		Flush the serial buffer												*/
/*----------------------------------------------------------------------------------*/
/*Description:	Flush a serial buffer of the specified channel						*/
/*Prototype:	long Hy85156_Flush(int port, bool TX, bool RX)						*/
/*Arguements:	port = serial channel 0 ~ 7											*/
/*				TX = true to flush the sending buffer, false not					*/
/*				RX = true to flush the receiving buffer, false not					*/
/*Return:		0 is successfull, negative please see error define					*/
/*----------------------------------------------------------------------------------*/
	long Hy85156_Flush(int port, bool TX, bool RX);
	
/*----------------------------------------------------------------------------------*/
/*Function:		Flush multi-serial buffers											*/
/*----------------------------------------------------------------------------------*/
/*Description:	Flush serial buffers according to the mask							*/
/*Prototype:	long Hy85156_Flush(USHORT mask)										*/
/*Arguements:	mask = serial channel mask											*/
/*Return:		0 is successfull, negative please see error define					*/
/*----------------------------------------------------------------------------------*/
	long Hy85156_Flush(USHORT mask);

};

#if PLATFORM == 1									//windows
extern "C" HYTECAPI_API CHy85156* CreateCHy85156();
extern "C" HYTECAPI_API void DeleteCHy85156( CHy85156* instance );
#else											//linux
extern "C" CHy85156* CreateCHy85156();
extern "C" void DeleteCHy85156( CHy85156* instance );
#endif

