/********************************************************************************/
/*    H   H Y   Y TTTTT EEEEE  CCC       HYTEC ELECTRONICS LTD                  */
/*    H   H  Y Y    T   E     C          5 Cradock Road,                        */
/*    HHHHH   Y     T   EEE   C          Reading, Berks.    Tel: 0118 9757770	*/
/*    H   H   Y     T   E     C          RG2 0JT            Fax: 0118 9757566	*/
/*    H   H   Y     T   EEEEE  CCC       Web: www.hytec-electronics.co.uk       */
/********************************************************************************/
/*                                                                              */
/*				IOC9010 PROJECT	--- TEWS Technologies IP Card API Functions		*/
/*							for Windows and Linux Platforms	    				*/
/*                                IP Card Type: TIP120                          */
/********************************************************************************/
/*  Source file name      :-      HyTip120.h									*/
/*  Definition header file:		  HYTECIPAPI.h									*/
/*  Platform			  :		  Linux, Windows XP, Windows 2000               */
/*  Initial creation date :-      26-NOV-2009                                   */
/*                                                                              */
/*  Original Developers   :-      Jim S Chen                                    */
/********************************************************************************/
/*                                                                              */
/*  Description :-                                                              */
/*                                                                              */
/*  This header file is designed for using TEWS Technologies TIP120 IP card.	*/
/*  It defines the class prototype, error code, masks and registers of Tip120.  */
/*																				*/
/*  The design of this interface is cross platforms, which means the code 		*/
/*	can be used in operating systems like Windows, Linux etc without changes.	*/
/*                                                                              */
/*  The language used is C++ but the final product -- the Dynamic Link library	*/
/*	(dll) or the shared library can be used by either C or C++ applications.	*/
/*																				*/
/*  (C)2009 Hytec Electronics Ltd.												*/
/*                                                                              */
/********************************************************************************/
/*                                                                              */
/*  Modification record :-                                                      */
/*                                                                              */
/*  Date modified |     Person    |   Description of modification | Version     */
/* ---------------|---------------|-------------------------------------------- */
/*    26/11/09    |	  Jim S Chen  |	  Initial copy				  |	0.0.1		*/
/*    20/11/12    |	  Jim S Chen  |	  Added support for more than |	0.0.2		*/
/*                |               |   one device                  |             */
/*                                                                              */
/********************************************************************************/
#include "HytecIPAPI.h"

#if PLATFORM == 1

#define HYTECAPI_EXPORTS 1
#ifdef HYTECAPI_EXPORTS
#define HYTECAPI_API __declspec(dllexport)
#else
#define HYTECAPI_API __declspec(dllimport)
#endif
#include "C:\Hytec\IOC9010\IOCGeneric\IOCGeneric\IOCGeneric.h"

#else

#include "IOCGeneric.h"

#endif



//error code definition
#define NOT_TIP120_IP -120010
#define TIP120_NOT_CONFIGURED -120011
#define TIP120_AXIS_INVALID -120012
#define TIP120_MOVETIME_OVERFLOW -120013
#define TIP120_STARTSPEED_INVALID -120014
#define TIP120_MAXSPEED_INVALID -120015
#define TIP120_RAMP_INVALID -120016
#define TIP120_BUSY_ERROR -120017
#define TIP120_INTLEVEL_ERROR -120018
#define TIP120_INTCALLBACK_NOTSET -120019

//INT mask definition
#define INT_INPUT3 0x0400
#define INT_INPUT2 0x0200
#define INT_INPUT1 0x0100
#define INT_BREAKPOINT 0x0040
#define INT_POSITIONERROR 0x0020
#define INT_WRAPAROUND 0x0010
#define INT_INDEXPULSE 0x0008
#define INT_TRAJECTORYCOMPLETE 0x0004
#define INT_COMMANDERROR 0x0002

#define INT_ENABLELM628 0x0008

//status bit definition
#define STATUS_HOST_INTERRUPT 0x8000
#define STATUS_ACCELERATION_LOADED 0x4000
#define STATUS_UDF_EXECUTED 0x2000
#define STATUS_FORWARD_DIRECTION 0x1000
#define STATUS_VELOCITY_MODE 0x0800
#define STATUS_ON_TARGET 0x0400
#define STATUS_OFF_ONERROR 0x0200
#define STATUS_8BIT_OUTPUT 0x0100
#define STATUS_MOTOR_OFF 0x0080
#define STATUS_BREAKPOINT INT_BREAKPOINT
#define STATUS_POSITIONERROR INT_POSITIONERROR 
#define STATUS_WRAPAROUND INT_WRAPAROUND
#define STATUS_INDEXPULSE INT_INDEXPULSE 
#define STATUS_TRAJECTORYCOMPLETE INT_TRAJECTORYCOMPLETE 
#define STATUS_COMMANDERROR INT_COMMANDERROR 
#define STATUS_BUSY_BIT 0x0001

//signature definition
#define MANUFACTURER_ID 0x00B3
#define SIGNATURE_TIP120 0x001E

//register address defines
#define CSR_TIP120 0x0000
#define DATA_TIP120 0x0002
#define OUTPUT_TIP120 0x0004
#define INPUT_TIP120 0x0006
#define INT_STATUS_TIP120 0x0008
#define INT_CTRL_TIP120 0x000A
#define INT_ENABLE_TIP120 0x000C
#define RESERVE_TIP120 0x000E

#define INT_VECTOR_TIP120 0x0020

//LM628 Commands
#define RESET_LM628 	0x00					// reset
#define STT_LM628 		0x01					// move command
#define DFH_LM628 		0x02					// define home
#define SIP_LM628 		0x03					// set index position
#define UDF_LM628 		0x04					// filter data update command
#define PORT12_LM628 	0x06					// set 12 bit DAC output
#define RDDV_LM628 		0x07					// read desired velocity
#define RDDP_LM628 		0x08					// read desired position
#define RDIP_LM628 		0x09					// read index position
#define RDRP_LM628 		0x0A					// read real position
#define RDRV_LM628 		0x0B					// read real velocity
#define RDSIGS_LM628 	0x0C					// read signals
#define RDSUM_LM628 	0x0D					// read real velocity
#define LPES_LM628 		0x1A					// load position error for stopping
#define LPEI_LM628 		0x1B					// load position error for interrupt
#define MSKI_LM628 		0x1C					// interrupt mask
#define RSTI_LM628 		0x1D					// reset interrupt
#define LFIL_LM628 		0x1E					// filter data command
#define LTRJ_LM628 		0x1F					// trajectory data command
#define SBPA_LM628 		0x20					// set breakpoint in terms of absolute position
#define SBPR_LM628 		0x21					// set breakpoint in terms of relative position

//Motor Control Commands
#define VELOCITY_MODE_TIP120 0x0800
#define FORWARD_DIRECTION_TIP120 0x1000
#define STOP_SMOOTHLY_TIP120 0x0400
#define STOP_ABRUPTLY_TIP120 0x0200
#define TURN_OFF_TIP120 0x0100
#define ACCELERATION_LOAD_TIP120 0x0020
#define ACCELERATION_RELATIVE_TIP120 0x0010
#define VELOCITY_LOAD_TIP120 0x0008
#define VELOCITY_RELATIVE_TIP120 0x0004
#define POSITION_LOAD_TIP120 0x0002
#define POSITION_RELATIVE_TIP120 0x0001

//trajectory parameters
//#define ENCODER_LINES 400
#define ENCODER_LINES 8192
#define ENCODER_RATIO 4
#define SAMPLE_TIME 256
#define VERTICAL_RATIO 24
#define HORIZONTAL_RATIO 50
#define DEFAULT_VELOCITY 3000
#define DEFAULT_ACCER 200

//PID parameters default

#define DSI_PID 0
#define KP_PID 1 //4
#define KI_PID 2
#define KD_PID 0
#define IL_PID 10
/*
#define DSI_PID 4
#define KP_PID 320
#define KI_PID 5
#define KD_PID 6000
#define IL_PID 10
*/
#define LOADPIDPARAM_PID 0x0F

//motor stop mode
#define STOP_SMOOTHLY 0
#define STOP_ABRUPTLY 1
#define STOP_ANDOFF 2


#define DATA_STATUS 0x0001
#define DATA_REAL_VELOCITY 0x0002
#define DATA_DESIRED_VELOCITY 0x0003
#define DATA_INTEGRATION 0x0004
#define DATA_REAL_POSITION 0x0005
#define DATA_DESIRED_POSITION 0x0006
#define DATA_INDEX_POSITION 0x0007

// This class is exported from the HYTECIPS.dll
#if PLATFORM == 1									//windows
class HYTECAPI_API CHyTip120 : public CIOCGeneric
#else												//linux
class CHyTip120 : public CIOCGeneric
#endif
{
private:
	long m_ErrorCode;								// global error code holder
	int initIPSlot;									// remember the IP slot when configures
	int initVMESlot;
	int m_mode[2];									// remember the mode of the axis
	int m_encoder_line[2];
	int m_encoder_ratio[2];
	double m_sample_time[2];
	double m_gearbox_ratio[2];
	int m_position[2];
	int m_mask[2];
	char m_vector;
	int m_INTLevel;
	int m_sem[2];
/*----------------------------------------------------------------------------------*/
/*Function:		Write to the LM628 chip (Internal Function)							*/
/*----------------------------------------------------------------------------------*/
/*Description:	This routine write a chunk of data to the chip						*/
/*				if it is not busy													*/
/*Prototype:	long read(int axis, USHORT cmd, int len, ULONG *rt)					*/
/*Arguements:	axis = motor channel (0 ~ 3)										*/
/*				cmd = command of the write operation								*/
/*				len = bytes to read													*/
/*				*rt = pointer to hold the return value								*/
/*Return:		status code, 0 = successful. negative = errors						*/
/*----------------------------------------------------------------------------------*/
	long read(int axis, USHORT cmd, int len, ULONG *rt);
/*----------------------------------------------------------------------------------*/
/*Function:		Write to the LM628 chip (Internal Function)							*/
/*----------------------------------------------------------------------------------*/
/*Description:	This routine write a chunk of data to the chip						*/
/*				if it is not busy													*/
/*Prototype:	long write(int axis, USHORT cmd, char *data, int len)				*/
/*Arguements:	axis = motor channel (0 ~ 3)										*/
/*				cmd = command of the write operation								*/
/*				*data = data array to be written									*/
/*				len = length of the data array										*/
/*Return:		status code, 0 = successful. negative = errors						*/
/*----------------------------------------------------------------------------------*/
	long write(int axis, USHORT cmd, char *data, int len);
/*----------------------------------------------------------------------------------*/
/*Function:		Check LM628 chip is busy (Internal Function)						*/
/*----------------------------------------------------------------------------------*/
/*Description:	This routine checks the busy bit in the status register and then	*/
/*				write to the LM628 chip if it is not busy							*/
/*Prototype:	long write(int axis, USHORT cmd, char *data, int len)				*/
/*Arguements:	axis = motor channel (0 ~ 3)										*/
/*Return:		status code, 0 = successful. negative = errors						*/
/*----------------------------------------------------------------------------------*/
	long checkBusy(int axis);

public:
	CHyTip120(void);
	CHyTip120(char *devName);
	CHyTip120(int vmeslot, char ipslot);
	~CHyTip120(void);

	long GetErrorCode()
	{
		return m_ErrorCode;
	}
	
/*----------------------------------------------------------------------------------*/
/*Function:		Configure Tip120 IP card											*/
/*----------------------------------------------------------------------------------*/
/*Description:	Tip120 IP configuration routine										*/
/*				according to the settings.											*/
/*Prototype:	long HyTip120_Configure(int vmeslot, char ipslot, int INTLevel,		*/
/*				char vector, void (*callback)(void *), void *drive)					*/
/*Arguements:	vmeslot = VME slot number											*/
/*				ipslot = IP card slot number 0 ~ 3 for VME, 0 ~ 5 for IOC			*/
/*				INTLevel = Interrupt level. Note: all 8002s will share one level	*/
/*				vector = interrupt vector, only bit 1 ~ bit 7 can be set. bit 0 	*/
/*						 is determined by the hardware. 0 for channel 1, 1 for 		*/
/*						 channel 2.													*/
/*				*callback = interrupt callback function pointer						*/
/*				*drive = user data structure pointer passed to the ISR				*/
/*Return:		status code, 0 = successful. negative = errors						*/
/*----------------------------------------------------------------------------------*/
	long HyTip120_Configure(int vmeslot, char ipslot, int INTLevel, char vector, void (*callback)(void *), void *drive);

/*----------------------------------------------------------------------------------*/
/*Function:		Set absolute position												*/
/*----------------------------------------------------------------------------------*/
/*Description:	Set software absolute position.										*/
/*Prototype:	long HyTip120_SetPosition(int axis, double position)		 		*/
/*Arguements:	axis = motor channel (0 ~ 1)										*/
/*				position, position to set in degree						 			*/
/*Return:		status code, 0 = successful. negative = errors						*/
/*----------------------------------------------------------------------------------*/
	long HyTip120_SetPosition(int axis, int position);

/*----------------------------------------------------------------------------------*/
/*Function:		Enable motor driver													*/
/*----------------------------------------------------------------------------------*/
/*Description:	Enable motor driver for the specified channel.						*/
/*Prototype:	long HyTip120_EnableDriver(int axis, int enable)			 		*/
/*Arguements:	axis = motor channel (0 ~ 1)										*/
/*				enable = 1, enable, = 0, disable						 			*/
/*Return:		status code, 0 = successful. negative = errors						*/
/*----------------------------------------------------------------------------------*/
	long HyTip120_EnableDriver(int axis, int enable);

/*----------------------------------------------------------------------------------*/
/*Function:		Enable Tip120 IP card Interrupt										*/
/*----------------------------------------------------------------------------------*/
/*Description:	Enable Tip120 IP interrupt. Set up callback etc.					*/
/*Prototype:	long HyTip120_EnableInterrupt(int axis)								*/
/*Arguements:	side = 0, A side; = 1, B side										*/
/*Return:		status code, 0 = successful. negative = errors						*/
/*----------------------------------------------------------------------------------*/
	long HyTip120_EnableInterrupt(int axis);

/*----------------------------------------------------------------------------------*/
/*Function:		Disable Tip120 IP card Interrupt									*/
/*----------------------------------------------------------------------------------*/
/*Description:	Enable Tip120 IP interrupt. Set up callback etc.					*/
/*Prototype:	long HyTip120_DisableInterrupt(int axis)							*/
/*Arguements:	side = 0, A side; = 1, B side										*/
/*Return:		status code, 0 = successful. negative = errors						*/
/*----------------------------------------------------------------------------------*/
    long HyTip120_DisableInterrupt(int axis);

/*----------------------------------------------------------------------------------*/
/*Function:		Reset Tip120 IP card Interrupt										*/
/*----------------------------------------------------------------------------------*/
/*Description:	Reset Tip120 IP interrupt. 											*/
/*Prototype:	long HyTip120_EnableInterrupt(int axis, int reason)					*/
/*Arguements:	axis = 0, A side; = 1, B side										*/
/*				reason = the reason of the interrupt. Same as get reason			*/
/*Return:		status code, 0 = successful. negative = errors						*/
/*----------------------------------------------------------------------------------*/
	long HyTip120_ResetInterrupt(int axis, int reason);

/*----------------------------------------------------------------------------------*/
/*Function:		Setup one Tip120 motor channel (overloading)						*/
/*----------------------------------------------------------------------------------*/
/*Description:	Tip120 IP motor channel setup routine								*/
/*				according to the specified channel settings.						*/
/*Prototype:	long HyTip120_MotorSetup(int axis, int mode)						*/
/*Arguements:	axis = motor channel (0 ~ 3)										*/
/*				mode = 0, position mode, 1, velocity mode				 			*/
/*				No interrupt											 			*/
/*				Use default PID parameters											*/
/*Return:		status code, 0 = successful. negative = errors						*/
/*----------------------------------------------------------------------------------*/
	long HyTip120_MotorSetup(int axis, int mode);

/*----------------------------------------------------------------------------------*/
/*Function:		Setup one Tip120 motor channel (overloading)						*/
/*----------------------------------------------------------------------------------*/
/*Description:	Tip120 IP motor channel setup routine								*/
/*				according to the specified channel settings.						*/
/*Prototype:	long HyTip120_MotorSetup(int axis, int mode, int maskINT, 			*/
/*				int edgeINT, int errorPosition, int errorPositionStop) 				*/
/*Arguements:	axis = motor channel (0 ~ 3)										*/
/*				mode = 0, position mode, 1, velocity mode				 			*/
/*				maskINT = interrupt	mask								 			*/
/*					bit 15 ~ 11, not used											*/
/*					bit 10 input 3 													*/
/*					bit 9 input 2													*/
/*					bit 8 input 1													*/
/*					bit 7, Not used													*/
/*					bit 6, enable(1)/disable(0) breakpoint interrupt				*/
/*					bit 5, enable(1)/disable(0) position error interrupt			*/
/*					bit 4, enable(1)/disable(0) wrap around interrupt				*/
/*					bit 3, enable(1)/disable(0) index-pulse interrupt				*/
/*					bit 2, enable(1)/disable(0) trajectory complete interrupt 		*/
/*					bit 1, enable(1)/disable(0) command-error interrupt				*/
/*					bit 0, Not used													*/
/*				edgeINT = 0, interrupt on High to Low. 1 interrupt on Low to High	*/
/*					bit 7 ~ bit 3: not used											*/
/*					bit 2 ~ bit 0: input 3 ~ 1. 0, High->Low interrupt; 			*/
/*												1, Low->High interrupt				*/
/*				errorPosition = maximum position allowed, passing this causes		*/
/*					excessive position error interrupt. in degree. When it is set	*/
/*					to 0 means don't set											*/	
/*				errorPositionStop = whether to stop the motor when excessive 		*/
/*					position error happens. If errorPosition is set to 0, this will	*/
/*					be ignored.														*/
/*				Use default PID parameters								 			*/
/*Return:		status code, 0 = successful. negative = errors						*/
/*----------------------------------------------------------------------------------*/
	long HyTip120_MotorSetup(int axis, int mode, int maskINT, int edgeINT, 
			int errorPosition, int errorPositionStop);

/*----------------------------------------------------------------------------------*/
/*Function:		Setup one Tip120 motor channel										*/
/*----------------------------------------------------------------------------------*/
/*Description:	Tip120 IP motor channel setup routine								*/
/*				according to the specified channel settings.						*/
/*Prototype:	long HyTip120_MotorSetup(int axis, int maskINT, int edgeINT, 		*/
/*							int errorPosition, int errorPositionStop, 				*/
/*							int mode, int dsi, int kp, int ki, int kd, int il)		*/
/*Arguements:	axis = motor channel (0 ~ 3)										*/
/*				mode = 0, position mode, 1, velocity mode				 			*/
/*				maskINT = interrupt	mask. if set, enable interrupt		 			*/
/*					bit 15 ~ 11, not used											*/
/*					bit 10 input 3 													*/
/*					bit 9 input 2													*/
/*					bit 8 input 1													*/
/*					bit 7, Not used													*/
/*					bit 6, enable(1)/disable(0) breakpoint interrupt				*/
/*					bit 5, enable(1)/disable(0) position error interrupt			*/
/*					bit 4, enable(1)/disable(0) wrap around interrupt				*/
/*					bit 3, enable(1)/disable(0) index-pulse interrupt				*/
/*					bit 2, enable(1)/disable(0) trajectory complete interrupt 		*/
/*					bit 1, enable(1)/disable(0) command-error interrupt				*/
/*					bit 0, Not used													*/
/*				edgeINT = 0, interrupt on High to Low. 1 interrupt on Low to High	*/
/*					bit 7 ~ bit 3: not used											*/
/*					bit 2 ~ bit 0: input 3 ~ 1. 0, High->Low interrupt; 			*/
/*												1, Low->High interrupt				*/
/*				errorPosition = maximum position allowed, passing this causes		*/
/*					excessive position error interrupt. in degree. When it is set	*/
/*					to 0 means don't set											*/
/*				errorPositionStop = whether to stop the motor when excessive 		*/
/*					position error happens. If errorPosition is set to 0, this will	*/
/*					be ignored.														*/
/*				dsi = derivative sampling interval. value = 0 ~ 256 in 256us steps	*/
/*				kp = proportion coefficient								 			*/
/*				ki = integral coefficient								 			*/
/*				kd = derivative coefficient								 			*/
/*				il = integral limit										 			*/
/*Return:		status code, 0 = successful. negative = errors						*/
/*----------------------------------------------------------------------------------*/
	long HyTip120_MotorSetup(int axis, int mode, int maskINT, int edgeINT, 
		int errorPosition, int errorPositionStop, 
		int dsi, int kp, int ki, int kd, int il);

/*----------------------------------------------------------------------------------*/
/*Function:		Set up PID parameters												*/
/*----------------------------------------------------------------------------------*/
/*Description:	This routine sets the PID coefficients								*/
/*Prototype:	long HyTIP120_PID_Param(int axis, int dsi, int kp, int ki, int kd, int il)	*/
/*Arguements:	axis = motor channel (0 ~ 3)										*/
/*				dsi = derivative sampling interval. value = 0 ~ 256 in 256us steps	*/
/*				kp = proportion coefficient								 			*/
/*				ki = integral coefficient								 			*/
/*				kd = derivative coefficient								 			*/
/*				il = integral limit										 			*/
/*Return:		status code, 0 = successful. negative = errors						*/
/*----------------------------------------------------------------------------------*/
	long HyTip120_PID_Param(int axis, int dsi, int kp, int ki, int kd, int il);

/*----------------------------------------------------------------------------------*/
/*Function:		Run motor to a certain speed										*/
/*----------------------------------------------------------------------------------*/
/*Description:	This routine commands the motor to run at a certain speed			*/
/*Prototype:	long HyTIP120_Run(int axis, double rpm, double ramp, int dir)		*/
/*Arguements:	axis = motor channel (0 ~ 3)										*/
/*				rpm = intended velocity	(RMP). 										*/
/*				ramp = intended acceleration (rev/second/second). 					*/
/*				dir = direction. 1 = forward, -1 = backwards	 					*/
/*Return:		status code, 0 = successful. negative = errors						*/
/*----------------------------------------------------------------------------------*/
	long HyTip120_Run(int axis, double rpm, double ramp, int dir);

/*----------------------------------------------------------------------------------*/
/*Function:		Move Steps for one motor channel (overloading)						*/
/*----------------------------------------------------------------------------------*/
/*Description:	This routine sets the absolute degree to move						*/
/*				then initilise the movement.										*/
/*Prototype:	long HyTIP120_Move(int axis, double degree)							*/
/*Arguements:	axis = motor channel (0 ~ 3)										*/
/*				degree = degree to go. 0 ~ 70 with 0.05 degree accuracy				*/
/*Return:		status code, 0 = successful. negative = errors						*/
/*----------------------------------------------------------------------------------*/
	long HyTip120_Move(int axis, double degree);

/*----------------------------------------------------------------------------------*/
/*Function:		Move Steps for one motor channel									*/
/*----------------------------------------------------------------------------------*/
/*Description:	This routine sets the absolute degree to move						*/
/*				then initilise the movement.										*/
/*Prototype:	long HyTIP120_Move(int axis, double rpm, double ramp, double degree)*/
/*Arguements:	axis = motor channel (0 ~ 3)										*/
/*				rpm = intended velocity	(RMP). if 0 will use last time settings		*/
/*				ramp = intended acceleration (rev/second/second). if 0 use last time*/
/*				degree = degree to go. 0 ~ 70 with 0.05 degree accuracy				*/
/*Return:		status code, 0 = successful. negative = errors						*/
/*----------------------------------------------------------------------------------*/
	long HyTip120_Move(int axis, double rpm, double ramp, double degree);

/*----------------------------------------------------------------------------------*/
/*Function:		Read data from one motor channel									*/
/*----------------------------------------------------------------------------------*/
/*Description:	This routine reads the status, real velocity, desired velocity,		*/
/*				real position, desired position, index position, integration etc	*/
/*Prototype:	long HyTip120_ReadData(int axis, unsigned long *data, int type)		*/
/*Arguements:	axis = motor channel (0 ~ 1)										*/
/*				*data = pointer to the variable that holds the returned value		*/
/*				type = DATA_STATUS, status, 2 bytes									*/
/*					 = DATA_REAL_VELOCITY, real velocity, 4 bytes					*/
/*					 = DATA_DESIRED_VELOCITY, real velocity, 4 bytes				*/
/*					 = DATA_INTEGRATION, integration, 2 bytes						*/
/*					 = DATA_REAL_POSITION, real position, 4 bytes					*/
/*					 = DATA_DESIRED_POSITION, real position, 4 bytes				*/
/*					 = DATA_INDEX_POSITION, real position, 4 bytes					*/
/*Return:		status code, 0 = successful. negative = errors						*/
/*----------------------------------------------------------------------------------*/
	long HyTip120_ReadData(int axis, unsigned long *data, int type);

/*----------------------------------------------------------------------------------*/
/*Function:		Stop motion															*/
/*----------------------------------------------------------------------------------*/
/*Description:	This routine stops the motor from motion							*/
/*Prototype:	long HyTip120_Stop(int axis, int stopmode)							*/
/*Arguements:	axis = motor channel (0 ~ 1)										*/
/*				mode = 0, smoothly; 1, abruptly; 2, turn off						*/
/*Return:		status code, 0 = successful. negative = errors						*/
/*----------------------------------------------------------------------------------*/
	long HyTip120_Stop(int axis, int stopmode);

/*----------------------------------------------------------------------------------*/
/*Function:		Define Home															*/
/*----------------------------------------------------------------------------------*/
/*Description:	This routine define the motor's home position of the specified		*/
/*				motor channel														*/
/*Prototype:	long HyTip120_DefineHome(int axis)									*/
/*Arguements:	axis = motor channel (0 ~ 1)										*/
/*Return:		status code, 0 = successful. negative = errors						*/
/*----------------------------------------------------------------------------------*/
	long HyTip120_DefineHome(int axis);

/*----------------------------------------------------------------------------------*/
/*Function:		Set Index Position													*/
/*----------------------------------------------------------------------------------*/
/*Description:	This routine set index position										*/
/*Prototype:	long HyTip120_SetIndex(int axis)									*/
/*Arguements:	axis = motor channel (0 ~ 1)										*/
/*Return:		status code, 0 = successful. negative = errors						*/
/*----------------------------------------------------------------------------------*/
	long HyTip120_SetIndex(int axis);

/*----------------------------------------------------------------------------------*/
/*Function:		Reset Motor			 												*/
/*----------------------------------------------------------------------------------*/
/*Description:	This routine resets the specified motor							 	*/
/*Prototype:	long HyTIP120_Reset(int axis, bool value)							*/
/*Arguements:	axis = motor channel (0 ~ 3)										*/
/*				value = true resets, false do not reset								*/
/*Return:		status code, 0 = successful. negative = errors						*/
/*----------------------------------------------------------------------------------*/
	long HyTip120_Reset(int axis);

/*----------------------------------------------------------------------------------*/
/*Function:		Get Interrupt reason 												*/
/*----------------------------------------------------------------------------------*/
/*Description:	This routine get the reason that causes the interrupt				*/
/*Prototype:	long HyTip120_INTReason(int axis, int *reason)						*/
/*Arguements:	axis = motor channel (0 ~ 1)										*/
/*				*reason = pointer to the user variable that will hold the reason	*/
/*				of the interrupt as below											*/
/*						0 0 0 0 0 x x x 0 x x x x x x 0								*/
/*						  		  | | |   | | |	| | | 					 			*/
/*						  		  | | |   | | | | | Command error					*/
/*						  		  | | |   | | | | Trajectory complete				*/
/*						  		  | | |	  | | |	Index pulse acquired				*/
/*						  		  | | |	  | | Wraparound occurred					*/
/*						  		  | | |   |	Excessive position error				*/
/*						  		  | | |   Breakpoint reached	 					*/
/*						  		  | | Input 1 interrupted							*/
/*								  | Input 2 interrupted								*/
/*								  Input 3 interrupted								*/
/*						0 -- not used												*/
/*Return:		status code, 0 = successful. negative = errors						*/
/*----------------------------------------------------------------------------------*/
	long HyTip120_INTReason(int axis, int *reason);

};

#if PLATFORM == 1									//windows
extern "C" HYTECAPI_API CHyTip120* CreateCHyTip120();
extern "C" HYTECAPI_API void DeleteCHyTip120( CHyTip120* instance );
#else											//linux
extern "C" CHyTip120* CreateCHyTip120();
extern "C" void DeleteCHyTip120( CHyTip120* instance );
#endif

