/********************************************************************************/
/*    H   H Y   Y TTTTT EEEEE  CCC       HYTEC ELECTRONICS LTD                  */
/*    H   H  Y Y    T   E     C          5 Cradock Road,                        */
/*    HHHHH   Y     T   EEE   C          Reading, Berks.    Tel: 0118 9757770	*/
/*    H   H   Y     T   E     C          RG2 0JT            Fax: 0118 9757566	*/
/*    H   H   Y     T   EEEEE  CCC       Web: www.hytec-electronics.co.uk       */
/********************************************************************************/
/*                                                                              */
/*				IOC9010 PROJECT	--- Hytec IP Card API Functions 				*/
/*							for Windows and Linux Platforms	    				*/
/*                                     Generic    		                        */
/********************************************************************************/
/*  Source file name      :-      HytecIPAPI.h									*/
/*  Platform			  :		  Linux, Windows XP, Windows 2000               */
/*  Initial creation date :-      11-AUG-2006                                   */
/*                                                                              */
/*  Original Developers   :-      Jim S Chen                                    */
/********************************************************************************/
/*                                                                              */
/*  Description :-                                                              */
/*                                                                              */
/*  This header file define generic macros shared by all other libraries.		*/
/*                                                                              */
/*  The design of this interface is cross platforms, which means the code 		*/
/*	can be used in operating systems like Windows, Linux etc without changes.	*/
/*                                                                              */
/*  The language used is C++ but the final product -- the Dynamic Link library	*/
/*	(dll) or the shared library can be used by either C or C++ applications.	*/
/*																				*/
/*  (C)2006 Hytec Electronics Ltd.												*/
/*                                                                              */
/********************************************************************************/
/*                                                                              */
/*  Modification record :-                                                      */
/*                                                                              */
/*  Date modified |     Person    |   Description of modification | Version     */
/* ---------------|---------------|-------------------------------------------- */
/*    11/08/06    |	  Jim S Chen  |	  Initial copy				   |  0.0.1		*/
/*                |               |                                |            */
/*    11/09/06    |   JC          |   Implement vmeslot parameter  |  0.0.2     */
/*                |               |   in Generic library           |            */
/*                |               |                                |            */
/*    15/10/07    |	  D Nineham   |	  Added Support for 8512       |  0.0.3		*/
/*                |               |                                |            */
/*    02/08/08    |	  M Woodward  |	  Added Support for 8005       |  0.0.n		*/
/*                |               |                                |            */
/*    10/08/08    |	  M Woodward  |	  Added Support for 8403       |  0.0.n		*/
/*                |               |                                |            */
/*    03/09/08    |	  M Woodward  |	  Added Support for 8413       |  0.0.n		*/
/*                |               |                                |            */
/*    07/11/08    |	  M Woodward  |	  More support for 8512        |  0.0.n		*/
/*                |               |                                |            */
/*    20/11/08    |	  M Woodward  |	  Added 8512 read/write Reg    |  0.0.n		*/
/*                |               |                                |            */
/*    11/12/08    |	  M Woodward  |	  Fixed problem with invalid   |  0.0.n		*/
/*                |               |   ipslot detection             |            */ 
/*                |               |                                |            */
/*    17/02/09    |	  M Woodward  |	  Added 3 new routines for     |            */ 
/*                |               |   8512. Enable/Clear ARM bits  |            */
/*                |               |   and clear overflow bits in   |            */
/*                |               |   the relavent registers. Done |            */
/*                |               |   as atomic instruction.       |            */
/*    26/03/09    |	  J Chen      |	  Added interrupt support for  |            */ 
/*                |               |   8505/6, 8515/6               |            */
/*    07/05/09    |   J Chen      |   Resumed calibration for 8402 |  3.0.0     */
/*    13/05/09    |   J Chen      |   Added interrupt enable       |  3.0.1     */
/*                |               |   routine for 8505/6           |            */
/*    15/05/09    |   J Chen      |   Changed 8505/6 back to it    |  3.0.2     */
/*                |               |   was before doing interrupt   |            */
/*    19/05/09    |   J Chen      |   Added CSR clear in 8505/6    |  3.0.3     */
/*                |               |   config routine               |            */
/*    12/06/09    |   J Chen      |   Fixed a bug in 8515 which    |  3.0.4     */
/*                |               |   reset the whole chip each    |            */
/*                |               |   time to initialise an UART   |            */
/*    15/06/09    |   J Chen      |   Changed 8515 configure to    |  3.0.5     */
/*                |               |   what it was before		   |            */
/*    07/07/09    |   J Chen      |   Improved 8515 read function  |  3.0.6     */
/*                |               |   to read till FIFO is empty   |            */
/*    27/10/09    |   J Chen      |   1.Fixed baud rate setting bug|  3.1.0     */
/*                |               |   for 8515/6.                  |            */
/*                |               |   2.Rewrote 85156 configure    |            */
/*                |               |   and ComSetup functions by    |            */
/*                |               |   overloading to implement     |            */
/*                |               |   RS485 support for 8516 with  |            */
/*                |               |   half/full duplex selectable  |            */
/*                |               |   3.Fixed the half duplex      |            */
/*                |               |   echo bug                     |            */
/*    28/10/09    |   J Chen      |   Added flush receiving FIFO   |  3.1.1     */
/*                |               |   after 8516 UART transmit to  |            */
/*                |               |   clear sperious 0 data        |            */
/*    30/10/09    |   J Chen      |   Combined echo setting with   |  3.1.2     */
/*                |               |   halfduplex param for 8516    |            */
/*                |               |   as suggested by Floyd.       |            */
/*    06/11/09    |   J Chen      |   Killed open device with read |  3.1.3     */
/*                |               |   and write in generic lib     |            */
/*    04/01/10    |   J Chen      |   Changed only clear CSR in    |  3.1.4     */
/*                |               |   85056 configure when vector  |            */
/*                |               |   is not 0                     |            */
/*    12/01/10    |   J Chen      |   1. Unified the interrupt     |  3.2.0     */
/*                |               |   process for 8505/6, 8515/6,  |            */
/*                |               |   8601 and Tip120 by providing |            */
/*                |               |   user callabck functions      |            */
/*                |               |   2. Added Tews Tip120 motor   |            */
/*                |               |   IP card support              |            */
/*    15/04/10    |   J Chen      |   Added position error and     |  3.2.1     */
/*                |               |   set index function for Tip120|            */
/*    05/05/10    |   J Chen      |   Added calibration routine	   |  3.2.2     */
/*                |               |   and range selection for 8413 |            */
/*    10/06/10    |   J Chen      |   Added Tip120 amplifier	   |  3.2.3     */
/*                |               |   enable in cinfiguration	   |            */
/*    29/06/10    |   M Woodward/ |   Added averaging for 8413 	   |  3.2.4     */
/*                |   K Jogia     |                          	   |            */
/*    06/07/10    |   J Chen      |   Merged changes for Tip120    |  3.2.5     */
/*    19/07/10    |   J Chen      |   Fixed one warning message    |  3.2.6     */
/*                |               |   print in 8413 read func  	   |            */
/*    10/08/10    |   K Jogia     |   Added waveform support for   |  3.2.7     */
/*                |   J Chen      |   Hytec 8402 DAC		  	   |            */
/*    07/10/10    |   J Chen      |   Fixed bugs for Tip120 that   |  3.2.8     */
/*                |               |   the settings are messed up   |            */
/*                |               |   between the two axes  	   |            */
/*    15/10/10    |   J Chen      |   Added disable interrupt for  |  3.2.9     */
/*                |               |   those who support interrupt  |            */
/*                |               |   mechanism					   |            */
/*    13/12/10    |   K Jogia     |   Added callback function and  |  3.2.10    */
/*                |               |   interrupt functions for 8512 |            */
/*    02/03/11    |   M Woodward  |   Fixed 8403 calibration issue |  3.2.11    */
/*                |   J Chen      |   when using different gain	   |            */
/*                |   MRN         |   settings.					   |            */
/*    17/11/11    |   J Chen  	  |   8413:Fixed +/-5V calibration |  3.2.13    */
/*                |   		      |   data fetching issue and fixed|            */
/*                |               |   calibration calculation	   |            */
/*                |               |   problem when raw value is    |            */
/*                |               |   0x8000.  					   |            */
/*    29/11/11    |   J Chen  	  |   Fixed status not assigned    |  3.2.14    */
/*                |               |   warning message			   |            */
/*    17/01/12    |   K Jogia  	  |   Added support for 8414 & 8415|  3.3.0     */
/*    16/05/12    |   J Chen  	  |   Fixed 8402 write bug that    |  3.3.1     */
/*                |               |   didn't check the busy bit	   |            */
/*                |               |   before updating DAC		   |            */
/*    18/05/12    |   J Chen  	  |   Fixed 8402 calibration data  |  3.3.2     */
/*                |               |   fetching too many error	   |            */
/*                |               |   which causes the FAIL led	   |            */
/*                |               |   flashing on IOC9010		   |            */
/*    30/08/12    |   J Chen  	  |   Added Read function for 8415 |  3.3.3     */
/*                |               |   to get last written value of |            */
/*                |               |   a channel					   |            */
/*    20/11/12    |   J Chen  	  |   Added constructors for all 5 |  3.3.4     */
/*                |               |   to support multiple devices  |            */
/*                |               |   Corrected a module check	   |            */
/*                |               |   error in 8515/8516 code	   |            */
/*    23/11/12    |   J Chen  	  |   Fixed 8402 calibration hbug  |  3.3.5     */
/*    05/12/13    |   J Chen  	  |   Fixed 8403 constant array    |  3.3.6     */
/*                |               |   population compiling error   |            */
/*                |               |   for some distros			   |            */
/*    14/03/14    |	  M Woodward  |	  Added support for 8511	   |  3.3.7		*/
/*                |               |   and 8510					   |            */
/*    02/04/14    |	  J Chen  	  |	  Fixed conflicts with CCT	   |  3.3.8		*/
/*                |               |   data type define			   |            */
/*    19/02/15    |	  M Woodward  |	  Added read_CSR for 8511	   |  3.3.9		*/
/*                |               |   and 8510.					   |            */
/*                |               |   Fixed 8414 set gain problem. |            */
/*                |               |   Added O/C mode for 8960 set  |            */
/*                |               |   timing routine.   		   |            */
/*    26/02/15    |   J Chen  	  |   Fixed 8515/8516 module	   |  3.4.0     */
/*                |               |   detection error.			   |            */
/*                |               |   Fixed 8515/8516 hardware 	   |            */
/*                |               |   flow control settings 	   |            */
/*                |               |                                |            */
/********************************************************************************/
//Hytec IP API generic defines
#define PLATFORM	0		//1 defined for Windows, 0 defined for Linux

#define IP_SLOT_INVALID -9010004

#define INTERRUPTLEVEL 7		//define interrupt level

#define IP_SLOT_NUMBER_IOC 6
#define IP_SLOT_NUMBER_VME 4
#define IP_NOT_CONFIGURED 100
#define HYTEC_CODE_HIGH 0x0080
#define HYTEC_CODE_LOW 0x0300
#define MEMORY_BLOCK 0x0200000

